@@ -124,7 +124,28 @@ typedef struct Avision_HWEntry {
     AV_INT_BUTTON = (1<<11),
 
     /* send acceleration table ... */
-    AV_ACCEL_TABLE = (1<<12)
+    AV_ACCEL_TABLE = (1<<12),
+
+    /* non-interlaced scanns up to 300 dpi (AV32xx / AV83xx) */
+    AV_NON_INTERLACED_DUPLEX_300 = (1<<13),
+
+    /* do not send 3x3 matrix */
+    AV_NO_MATRIX = (1<<14),
+
+    /* force channel-by-channel calibration */
+    AV_MULTI_CALIB_CMD = (1<<15),
+
+    /* non color scans are faster with a filter applied (AV32xx) */
+    AV_FASTER_WITH_FILTER = (1<<16),
+
+    /* interlaced data with 1 line distance */
+    AV_2ND_LINE_INTERLACED = (1<<17),
+
+    /* does not keep the window though it advertices so */
+    AV_DOES_NOT_KEEP_WINDOW = (1<<18),
+
+    /* advertises ADF is BGR order, but isn't (or vice versa) */
+    AV_ADF_BGR_ORDER_INVERT = (1<<18)
 
     /* maybe more ...*/
   } feature_type;
@@ -177,7 +198,7 @@ enum Avision_Option
   OPT_MODE_GROUP,
   OPT_MODE,
   OPT_RESOLUTION,
-#define OPT_RESOLUTION_DEFAULT 300
+#define OPT_RESOLUTION_DEFAULT 150
   OPT_SPEED,
   OPT_PREVIEW,
   
@@ -201,7 +222,7 @@ enum Avision_Option
   OPT_GAMMA_VECTOR_B,
   
   /* too bad the SANE API does not allow bool vectors ... */
-  OPT_BUTTON_0,           /* scanner button pressed */
+  OPT_BUTTON_0,          /* scanner button pressed */
   OPT_BUTTON_1,
   OPT_BUTTON_2,
   OPT_BUTTON_3,
@@ -210,10 +231,13 @@ enum Avision_Option
   OPT_BUTTON_6,
   OPT_BUTTON_7,
   OPT_BUTTON_LAST = OPT_BUTTON_7,
+  OPT_MESSAGE,           /* optional message from the scanner display */
 
-  OPT_MESSAGE,
-  
   OPT_FRAME,             /* Film holder control */
+
+  OPT_POWER_SAVE_TIME,   /* set power save time to the scanner */
+  
+  OPT_NVRAM,             /* retrieve NVRAM values as pretty printed text */
   
   NUM_OPTIONS            /* must come last */
 };
@@ -253,14 +277,20 @@ typedef struct Avision_Device
 
   asic_type inquiry_asic_type;
   SANE_Bool inquiry_new_protocol;
+
+  SANE_Bool inquiry_nvram_read;
+  SANE_Bool inquiry_power_save_time;
+
   SANE_Bool inquiry_light_box;
   SANE_Bool inquiry_adf;
   SANE_Bool inquiry_duplex;
   SANE_Bool inquiry_duplex_interlaced;
-  SANE_Bool inquiry_duplex_mode_two;
+  SANE_Bool inquiry_paper_length;
   SANE_Bool inquiry_detect_accessories;
   SANE_Bool inquiry_needs_calibration;
   SANE_Bool inquiry_needs_gamma;
+  SANE_Bool inquiry_keeps_gamma;
+  SANE_Bool inquiry_keeps_window;
   SANE_Bool inquiry_calibration;
   SANE_Bool inquiry_3x3_matrix;
   SANE_Bool inquiry_needs_software_colorpack;
@@ -301,7 +331,7 @@ typedef struct Avision_Device
 
   int inquiry_channels_per_pixel;
   int inquiry_bits_per_channel;
-
+  
   int scsi_buffer_size; /* nice to have SCSI buffer size */
 
   /* additional information - read delayed until sane_open() */
@@ -336,8 +366,9 @@ typedef struct Avision_Scanner
   
   /* Parsed option values and variables that are valid only during
      the actual scan: */
-  
+  SANE_Bool prepared;			/* first page marker */
   SANE_Bool scanning;           /* scan in progress */
+
   SANE_Parameters params;       /* scan window */
   Avision_Dimensions avdimen;   /* scan window - detailed internals */
   
@@ -354,9 +385,9 @@ typedef struct Avision_Scanner
   /* Avision HW Access Connection (SCSI/USB abstraction) */
   Avision_Connection av_con;
 
-  pid_t reader_pid;		/* process id of reader */
-  int reader_fds;		/* pipe write handler for reader */
-  int pipe;			/* pipe to reader process */
+  pid_t reader_pid;	/* process id of reader */
+  int read_fds;		/* pipe reading end */
+  int write_fds;	/* pipe writing end */
   int line;			/* current line number during scan */
   
 } Avision_Scanner;
@@ -391,6 +422,14 @@ typedef struct Avision_Scanner
 #define AVISION_SCSI_OP_TRANS_CALIB_GRAY    0x04
 #define AVISION_SCSI_OP_TRANS_CALIB_COLOR   0x05
 
+#define AVISION_FILTER_NONE	0
+#define AVISION_FILTER_RED	1
+#define AVISION_FILTER_GREEN	2
+#define AVISION_FILTER_BLUE	3
+#define AVISION_FILTER_RGB	4
+#define AVISION_FILTER_CMYK	5
+#define AVISION_FILTER_GRAY	6
+
 /* The SCSI structures that we have to send to an avision to get it to
    do various stuff... */
 
@@ -441,6 +480,42 @@ typedef struct command_send
   u_int8_t reserved1;
 } command_send;
 
+typedef struct nvram_data
+{
+  u_int8_t pad_scans [4];
+  u_int8_t adf_simplex_scans [4];
+  u_int8_t adf_duplex_scans [4];
+  u_int8_t flatbed_scans [4];
+
+  u_int8_t flatbed_leading_edge [2];
+  u_int8_t flatbed_side_edge [2];
+  u_int8_t adf_leading_edge [2];
+  u_int8_t adf_side_edge [2];
+  u_int8_t adf_rear_leading_edge [2];
+  u_int8_t adf_rear_side_edge [2];
+
+  u_int8_t born_month [2];
+  u_int8_t born_day [2];
+  u_int8_t born_year [2];
+
+  u_int8_t first_scan_month [2];
+  u_int8_t first_scan_day [2];
+  u_int8_t first_scan_year [2];
+
+  u_int8_t vertical_magnification [2];
+  u_int8_t horizontal_magnification [2];
+
+  u_int8_t ccd_type;
+  u_int8_t scan_speed;
+
+  u_int8_t serial [24];
+
+  u_int8_t power_saving_time [2];
+
+  u_int8_t reserved [56];
+} nvram_data;
+
+
 typedef struct command_set_window_window
 {
   struct {
@@ -482,7 +557,7 @@ typedef struct command_set_window_window
     u_int8_t line_width [2];
     u_int8_t line_count [2];
     
-    /* the tail is quite version and model sepecific */
+    /* the tail is quite version and model specific */
     union {
       struct {
 	u_int8_t bitset2;
@@ -554,16 +629,22 @@ typedef struct calibration_format
   u_int8_t channels;
 } calibration_format;
 
+typedef struct matrix_3x3
+{
+  u_int16_t v[9];
+} matrix_3x3;
+
 typedef struct acceleration_info 
 {
-  u_int16_t accel_step_count;
-  u_int16_t stable_step_count;
+  u_int16_t total_steps;
+  u_int16_t stable_steps;
   u_int32_t table_units;
   u_int32_t base_units;
   u_int16_t start_speed;
   u_int16_t target_speed;
   u_int8_t ability;
   u_int8_t table_count;
+  u_int8_t reserved[6];
 } acceleration_info;
 
 /* set/get SCSI highended (big-endian) variables. Declare them as an array
@@ -583,7 +664,7 @@ typedef struct acceleration_info
 #define get_double(var) ((*var << 8) + *(var + 1))
 
 #define get_triple(var) ((*var << 16) + \
-                         (*(var + 1) << 8) + * *var)
+                         (*(var + 1) << 8) + *(var + 2))
 
 #define get_quad(var)   ((*var << 24) + \
                          (*(var + 1) << 16) + \
