@@ -1,7 +1,7 @@
 /* sane - Scanner Access Now Easy.
 
    Copyright (C) 2002 Sergey Vlasov <vsu@altlinux.ru>
-   Copyright (C) 2002 - 2005 Henning Meier-Geinitz <henning@meier-geinitz.de>
+   Copyright (C) 2002 - 2006 Henning Meier-Geinitz <henning@meier-geinitz.de>
 
    This file is part of the SANE package.
    
@@ -48,10 +48,12 @@
 
 #include "../include/sane/config.h"
 
-#define BUILD 71
+#define BUILD 81
 #define MAX_DEBUG
 #define WARMUP_TIME 60
 #define CALIBRATION_HEIGHT 2.5
+#define SHORT_TIMEOUT (1 * 1000)
+#define LONG_TIMEOUT (30 * 1000)
 
 /* Use a reader process if possible (usually faster) */
 #if defined (HAVE_SYS_SHM_H) && (!defined (USE_PTHREAD)) && (!defined (HAVE_OS2_H))
@@ -600,6 +602,11 @@ init_options (GT68xx_Scanner * s)
   s->val[OPT_COARSE_CAL].w = SANE_TRUE;
   if (!debug_options)
     DISABLE (OPT_COARSE_CAL);
+  if (s->dev->model->flags & GT68XX_FLAG_SHEET_FED)
+    {
+      s->val[OPT_COARSE_CAL].w = SANE_FALSE;
+      DISABLE (OPT_COARSE_CAL);
+    }
 
   /* coarse calibration only once */
   s->opt[OPT_COARSE_CAL_ONCE].name = "coarse-calibration-once";
@@ -616,6 +623,8 @@ init_options (GT68xx_Scanner * s)
   s->val[OPT_COARSE_CAL_ONCE].w = SANE_FALSE;
   if (!debug_options)
     DISABLE (OPT_COARSE_CAL_ONCE);
+  if (s->dev->model->flags & GT68XX_FLAG_SHEET_FED)
+    DISABLE (OPT_COARSE_CAL_ONCE);
 
   /* calibration */
   s->opt[OPT_QUALITY_CAL].name = SANE_NAME_QUALITY_CAL;
@@ -627,6 +636,11 @@ init_options (GT68xx_Scanner * s)
   s->val[OPT_QUALITY_CAL].w = SANE_TRUE;
   if (!debug_options)
     DISABLE (OPT_QUALITY_CAL);
+  if (s->dev->model->flags & GT68XX_FLAG_SHEET_FED)
+    {
+      s->val[OPT_QUALITY_CAL].w = SANE_FALSE;
+      DISABLE (OPT_QUALITY_CAL);
+    }
 
   /* backtrack lines */
   s->opt[OPT_BACKTRACK_LINES].name = "backtrack-lines";
@@ -640,7 +654,7 @@ init_options (GT68xx_Scanner * s)
   s->opt[OPT_BACKTRACK_LINES].unit = SANE_UNIT_NONE;
   s->opt[OPT_BACKTRACK_LINES].constraint_type = SANE_CONSTRAINT_RANGE;
   s->opt[OPT_BACKTRACK_LINES].constraint.range = &u8_range;
-  if (s->dev->model->is_cis)
+  if (s->dev->model->is_cis && !(s->dev->model->flags & GT68XX_FLAG_SHEET_FED))
     s->val[OPT_BACKTRACK_LINES].w = 0x10;
   else
     s->val[OPT_BACKTRACK_LINES].w = 0x3f;
@@ -1187,7 +1201,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 	}
       else if (strcmp (word, "afe") == 0)
 	{
-	  GT68xx_AFE_Parameters afe;
+	  GT68xx_AFE_Parameters afe = {0, 0, 0, 0, 0, 0};
 	  SANE_Status status;
 
 	  free (word);
@@ -1462,6 +1476,9 @@ sane_close (SANE_Handle handle)
 
   dev = s->dev;
   gt68xx_scanner_free (s);
+  
+  gt68xx_device_fix_descriptor (dev);
+
   gt68xx_device_deactivate (dev);
   gt68xx_device_close (dev);
 
@@ -1626,6 +1643,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 		}
 	      s->first_scan = SANE_TRUE;
 	      myinfo |= SANE_INFO_RELOAD_PARAMS | SANE_INFO_RELOAD_OPTIONS;
+	      gettimeofday (&s->lamp_on_time, 0);
 	    }
 	  break;
 	case OPT_MODE:
@@ -1792,7 +1810,8 @@ sane_start (SANE_Handle handle)
   if (!(s->dev->model->flags & GT68XX_FLAG_NO_STOP))
     RIE (gt68xx_device_stop_scan (s->dev));
 
-  RIE (gt68xx_device_carriage_home (s->dev));
+  if (!(s->dev->model->flags & GT68XX_FLAG_SHEET_FED))
+    RIE (gt68xx_device_carriage_home (s->dev));
 
   gt68xx_scanner_wait_for_positioning (s);
   gettimeofday (&s->start_time, 0);
@@ -1813,6 +1832,7 @@ sane_start (SANE_Handle handle)
     scan_request.backtrack_lines = 0;
 
   RIE (gt68xx_scanner_calibrate (s, &scan_request));
+  
   RIE (gt68xx_scanner_start_scan (s, &scan_request, &scan_params));
   for (i = 0; i < scan_params.overscan_lines; ++i)
     RIE (gt68xx_scanner_read_line (s, buffer_pointers));
@@ -2059,7 +2079,7 @@ sane_cancel (SANE_Handle handle)
     {
       s->scanning = SANE_FALSE;
       if (s->total_bytes != (s->params.bytes_per_line * s->params.lines))
-	DBG (0, "sane_cancel: warning: scanned %d bytes, expected %d "
+	DBG (1, "sane_cancel: warning: scanned %d bytes, expected %d "
 	     "bytes\n", s->total_bytes,
 	     s->params.bytes_per_line * s->params.lines);
       else
@@ -2080,9 +2100,23 @@ sane_cancel (SANE_Handle handle)
 #endif
 
 	}
+      /* some scanners don't like this command when cancelling a scan */
+      sanei_usb_set_timeout (SHORT_TIMEOUT);
+      gt68xx_device_fix_descriptor (s->dev);
       gt68xx_scanner_stop_scan (s);
-      gt68xx_scanner_wait_for_positioning (s);
-      gt68xx_device_carriage_home (s->dev);
+      sanei_usb_set_timeout (LONG_TIMEOUT);
+
+      if (s->dev->model->flags & GT68XX_FLAG_SHEET_FED)
+	{
+	  gt68xx_device_paperfeed (s->dev);
+	} 
+      else
+	{
+	  sanei_usb_set_timeout (SHORT_TIMEOUT);
+	  gt68xx_scanner_wait_for_positioning (s);
+	  sanei_usb_set_timeout (LONG_TIMEOUT);
+	  gt68xx_device_carriage_home (s->dev);
+	}
       if (s->gamma_table)
 	free (s->gamma_table);
       s->gamma_table = 0;
