@@ -1,7 +1,7 @@
 /* sane - Scanner Access Now Easy.
 
    Copyright (C) 2002 Sergey Vlasov <vsu@altlinux.ru>
-   Copyright (C) 2002 - 2005 Henning Meier-Geinitz <henning@meier-geinitz.de>
+   Copyright (C) 2002 - 2006 Henning Meier-Geinitz <henning@meier-geinitz.de>
    
    This file is part of the SANE package.
    
@@ -78,6 +78,7 @@ static GT68xx_Command_Set mustek_gt6816_command_set = {
   gt6816_is_moving,
   gt68xx_generic_move_relative,
   gt6816_carriage_home,
+  /* gt68xx_generic_paperfeed */ NULL,
   gt68xx_generic_start_scan,
   gt68xx_generic_read_scanned_data,
   gt6816_stop_scan,
@@ -87,6 +88,45 @@ static GT68xx_Command_Set mustek_gt6816_command_set = {
   gt68xx_generic_get_id
 };
 
+static GT68xx_Command_Set mustek_gt6816_sheetfed_command_set = {
+  "mustek-gt6816",		/* Name of this command set */
+
+  0x40,				/* Request type */
+  0x01,				/* Request */
+
+  0x200c,			/* Memory read - wValue */
+  0x200b,			/* Memory write - wValue */
+
+  0x2010,			/* Send normal command - wValue */
+  0x3f40,			/* Send normal command - wIndex */
+  0x2011,			/* Receive normal result - wValue */
+  0x3f00,			/* Receive normal result - wIndex */
+
+  0x2012,			/* Send small command - wValue */
+  0x3f40,			/* Send small command - wIndex */
+  0x2013,			/* Receive small result - wValue */
+  0x3f00,			/* Receive small result - wIndex */
+
+  /* activate */ NULL,
+  /* deactivate */ NULL,
+  gt6816_check_firmware,
+  gt6816_download_firmware,
+  gt6816_get_power_status,
+  gt6816_get_ta_status,
+  gt6816_lamp_control,
+  gt6816_is_moving,
+  gt68xx_generic_move_relative,
+  /* gt6816_carriage_home */ NULL,
+  gt68xx_generic_paperfeed,
+  gt68xx_generic_start_scan,
+  gt68xx_generic_read_scanned_data,
+  gt6801_stop_scan,
+  gt68xx_generic_setup_scan,
+  gt68xx_generic_set_afe,
+  gt68xx_generic_set_exposure_time,
+  gt68xx_generic_get_id
+};
+
 static GT68xx_Command_Set mustek_gt6801_command_set = {
   "mustek-gt6801",
 
@@ -117,6 +157,7 @@ static GT68xx_Command_Set mustek_gt6801_command_set = {
   gt6801_is_moving,
   /* gt68xx_generic_move_relative *** to be tested */ NULL,
   gt6801_carriage_home,
+  /* gt68xx_generic_paperfeed */ NULL,
   gt68xx_generic_start_scan,
   gt68xx_generic_read_scanned_data,
   gt6801_stop_scan,
@@ -155,6 +196,7 @@ static GT68xx_Command_Set plustek_gt6801_command_set = {
   gt6801_is_moving,
   /* gt68xx_generic_move_relative *** to be tested */ NULL,
   gt6801_carriage_home,
+  /* gt68xx_generic_paperfeed */ NULL,
   gt68xx_generic_start_scan,
   gt68xx_generic_read_scanned_data,
   gt6801_stop_scan,
@@ -551,7 +593,7 @@ static GT68xx_Model mustek_2400cuplus_model = {
   SANE_FIX (1.5),		/* Default gamma value */
 
   SANE_TRUE,			/* Is this a CIS scanner? */
-  0				/* Which flags are needed for this scanner? */
+  GT68XX_FLAG_NO_STOP		/* Which flags are needed for this scanner? */
     /* Setup and tested */
 };
 
@@ -979,7 +1021,7 @@ static GT68xx_Model plustek_op1248u_model = {
 
   SANE_FIX (3.5),		/* Start of scan area in mm  (x) */
   SANE_FIX (7.5),		/* Start of scan area in mm (y) */
-  SANE_FIX (218.0),		/* Size of scan area in mm (x) */
+  SANE_FIX (216.0),		/* Size of scan area in mm (x) */
   SANE_FIX (299.0),		/* Size of scan area in mm (y) */
 
   SANE_FIX (0.0),		/* Start of white strip in mm (y) */
@@ -1058,6 +1100,106 @@ static GT68xx_Model plustek_u16b_model = {
        issue. */
 };
 
+static GT68xx_Model plustek_ops12_model = {
+  "plustek-opticpro-s12",	/* Name */
+  "Plustek",			/* Device vendor string */
+  "OpticPro S12",		/* Device model name */
+  "ccd548.fw",			/* Name of the firmware file */
+  SANE_FALSE,			/* Dynamic allocation flag */
+
+  &mustek_gt6816_command_set,	/* Command set used by this scanner */
+
+  600,				/* maximum optical sensor resolution */
+  600,				/* maximum motor resolution */
+  600,				/* base x-res used to calculate geometry */
+  600,				/* base y-res used to calculate geometry */
+  1200,				/* if ydpi is equal or higher, disable backtracking */
+  SANE_TRUE,			/* Use base_ydpi for all resolutions */
+
+  {600, 300, 150, 75, 0},	/* possible x-resolutions */
+  {600, 300, 150, 75, 0},	/* possible y-resolutions */
+  {12, 8, 0},			/* possible depths in gray mode */
+  {12, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (3.5),		/* Start of scan area in mm  (x) */
+  SANE_FIX (7.5),		/* Start of scan area in mm (y) */
+  SANE_FIX (218.0),		/* Size of scan area in mm (x) */
+  SANE_FIX (299.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (1.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  0, 8, 16,			/* RGB CCD Line-distance correction in pixel */
+  0,				/* CCD distcance for CCD with 6 lines) */
+
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
+  {0x1c, 0x29, 0x1c, 0x2c, 0x1c, 0x2b},	/* Default offset/gain */
+  {0x157, 0x157, 0x157},	/* Default exposure parameters */
+  SANE_FIX (2.0),		/* Default gamma value */
+
+  SANE_FALSE,			/* Is this a CIS scanner? */
+  GT68XX_FLAG_UNTESTED | GT68XX_FLAG_OFFSET_INV | GT68XX_FLAG_ALWAYS_LINEMODE	/* Which flags are needed for this scanner? */
+    /* Untested */
+};
+
+static GT68xx_Model plustek_ops24_model = {
+  "plustek-opticpro-s24",	/* Name */
+  "Plustek",			/* Device vendor string */
+  "OpticPro S24",	/* Device model name */
+  "ccd569.fw",			/* Name of the firmware file */
+  SANE_FALSE,			/* Dynamic allocation flag */
+
+  &mustek_gt6816_command_set,	/* Command set used by this scanner */
+
+  600,				/* maximum optical sensor resolution */
+  1200,				/* maximum motor resolution */
+  600,				/* base x-res used to calculate geometry */
+  600,				/* base y-res used to calculate geometry */
+  1200,				/* if ydpi is equal or higher, disable backtracking */
+  SANE_TRUE,			/* Use base_ydpi for all resolutions */
+
+  {600, 300, 200, 100, 50, 0},	/* possible x-resolutions */
+  {1200, 600, 300, 200, 100, 50, 0},	/* possible y-resolutions */
+  {12, 8, 0},			/* possible depths in gray mode */
+  {12, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (4.5),		/* Start of scan area in mm  (x) */
+  SANE_FIX (8.5),		/* Start of scan area in mm (y) */
+  SANE_FIX (218.0),		/* Size of scan area in mm (x) */
+  SANE_FIX (299.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (1.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  48, 24, 0,			/* RGB CCD Line-distance correction in pixel */
+  0,				/* CCD distcance for CCD with 6 lines) */
+
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
+  {0x18, 0x1c, 0x16, 0x12, 0x18, 0x1c},	/* Default offset/gain */
+  {0x157, 0x157, 0x157},	/* Default exposure parameters */
+  SANE_FIX (2.0),		/* Default gamma value */
+
+  SANE_FALSE,			/* Is this a CIS scanner? */
+  GT68XX_FLAG_OFFSET_INV | GT68XX_FLAG_ALWAYS_LINEMODE | GT68XX_FLAG_UNTESTED
+  				/* Which flags are needed for this scanner? */
+    /* Untested. Based on genius Colorpage Vivid 1200 X*/
+};
+
+
 static GT68xx_Model genius_vivid4_model = {
   "genius-colorpage-vivid4",	/* Name */
   "Genius",			/* Device vendor string */
@@ -1306,6 +1448,57 @@ static GT68xx_Model genius_vivid3xe_model = {
     /* mostly untested, based on the Genius Vivid3x */
 };
 
+static GT68xx_Model genius_vivid1200x_model = {
+  "genius-colorpage-vivid-1200-x",	/* Name */
+  "Genius",			/* Device vendor string */
+  "Colorpage Vivid 1200 X",	/* Device model name */
+  "ccd569.fw",			/* Name of the firmware file */
+  SANE_FALSE,			/* Dynamic allocation flag */
+
+  &mustek_gt6816_command_set,	/* Command set used by this scanner */
+
+  600,				/* maximum optical sensor resolution */
+  1200,				/* maximum motor resolution */
+  600,				/* base x-res used to calculate geometry */
+  600,				/* base y-res used to calculate geometry */
+  1200,				/* if ydpi is equal or higher, disable backtracking */
+  SANE_TRUE,			/* Use base_ydpi for all resolutions */
+
+  {600, 300, 200, 100, 50, 0},	/* possible x-resolutions */
+  {1200, 600, 300, 200, 100, 50, 0},	/* possible y-resolutions */
+  {12, 8, 0},			/* possible depths in gray mode */
+  {12, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (4.5),		/* Start of scan area in mm  (x) */
+  SANE_FIX (8.5),		/* Start of scan area in mm (y) */
+  SANE_FIX (218.0),		/* Size of scan area in mm (x) */
+  SANE_FIX (299.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (1.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  48, 24, 0,			/* RGB CCD Line-distance correction in pixel */
+  0,				/* CCD distcance for CCD with 6 lines) */
+
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
+  {0x18, 0x1c, 0x16, 0x12, 0x18, 0x1c},	/* Default offset/gain */
+  {0x157, 0x157, 0x157},	/* Default exposure parameters */
+  SANE_FIX (2.0),		/* Default gamma value */
+
+  SANE_FALSE,			/* Is this a CIS scanner? */
+  GT68XX_FLAG_OFFSET_INV | GT68XX_FLAG_ALWAYS_LINEMODE
+  				/* Which flags are needed for this scanner? */
+    /* Tested. */
+};
+
+
 static GT68xx_Model genius_vivid1200xe_model = {
   "genius-colorpage-vivid-1200-xe",	/* Name */
   "Genius",			/* Device vendor string */
@@ -1355,6 +1548,54 @@ static GT68xx_Model genius_vivid1200xe_model = {
     /* Tested by hmg */
 };
 
+static GT68xx_Model plustek_opticslim_m12_model = {
+  "plustek-opticslim-m12",	/* Name */
+  "Plustek",				/* Device vendor string */
+  "OpticSlim M12",			/* Device model name */
+  "cism216.fw",				/* Name of the firmware file */
+  SANE_FALSE,				/* Dynamic allocation flag */
+
+  &mustek_gt6816_sheetfed_command_set,	/* Command set used by this scanner */
+
+  600,					/* maximum optical sensor resolution */
+  1200,					/* maximum motor resolution */
+  600,					/* base x-res used to calculate geometry */
+  600,					/* base y-res used to calculate geometry */
+  1200,   				/* if ydpi is equal or higher, disable backtracking */
+  SANE_FALSE,			/* Use base_ydpi for all resolutions */
+
+  {600, 300, 200, 150, 100, 0},	/* possible x-resolutions */
+  {600, 300, 200, 150, 100, 0},	/* possible y-resolutions */
+  {16, 8, 0},			/* possible depths in gray mode */
+  {16, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (1.0),		/* Start of scan area in mm  (x) */
+  SANE_FIX (9.5),		/* Start of scan area in mm (y) */
+  SANE_FIX (218.0),		/* Size of scan area in mm (x) */
+  SANE_FIX (299.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (10.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (140.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  0, 0, 0,				/* RGB CCD Line-distance correction in pixel */
+  0,					/* CCD distcance for CCD with 6 lines) */
+
+  COLOR_ORDER_BGR,		/* Order of the CCD/CIS colors */
+  {0x24, 0x0a, 0x23, 0x0f, 0x23, 0x0b},	/* Default offset/gain */
+  {0x157, 0x157, 0x157},	/* Default exposure parameters */
+  SANE_FIX (2.0),			/* Default gamma value */
+
+  SANE_TRUE,				/* Is this a CIS scanner? */
+  GT68XX_FLAG_NO_POWER_STATUS | GT68XX_FLAG_SHEET_FED
+};
+
 static GT68xx_Model plustek_opticslim1200_model = {
   "plustek-opticslim-1200",	/* Name */
   "Plustek",			/* Device vendor string */
@@ -1503,6 +1744,7 @@ static GT68xx_Model visioneer_onetouch_7300_model = {
 /* Tested by Jason Novek. Based on Plustek OpticSlim 2400. */
 
 
+
 static GT68xx_USB_Device_Entry gt68xx_usb_device_list[] = {
   {0x10000, 0x10000, &unknown_model},	/* used for yet unknown scanners */
   {0x055f, 0x0218, &mustek_2400ta_model},
@@ -1524,6 +1766,9 @@ static GT68xx_USB_Device_Entry gt68xx_usb_device_list[] = {
   {0x07b3, 0x0401, &plustek_op1248u_model},	/* Same scanner, different id? */
   {0x07b3, 0x0402, &plustek_u16b_model},
   {0x07b3, 0x0403, &plustek_u16b_model},	/* two ids? 403 seems to be more common */
+  {0x07b3, 0x040b, &plustek_ops12_model},
+  {0x07b3, 0x040e, &plustek_ops24_model},
+  {0x07b3, 0x0412, &plustek_opticslim_m12_model},
   {0x07b3, 0x0413, &plustek_opticslim1200_model},
   {0x07b3, 0x0422, &plustek_opticslim2400_model},
   {0x0458, 0x2011, &genius_vivid3x_model},
@@ -1531,6 +1776,7 @@ static GT68xx_USB_Device_Entry gt68xx_usb_device_list[] = {
   {0x0458, 0x2017, &genius_vivid3xe_model},
   {0x0458, 0x201a, &genius_vivid4xe_model},
   {0x0458, 0x201b, &genius_vivid4x_model},
+  {0x0458, 0x201d, &genius_vivid1200x_model},
   {0x0458, 0x201f, &genius_vivid1200xe_model},
   {0x04a7, 0x0444, &visioneer_onetouch_7300_model},
   {0, 0, NULL}
