@@ -96,10 +96,9 @@ gt68xx_generic_read_scanned_data (GT68xx_Device * dev, SANE_Bool * ready)
 
   RIE (gt68xx_device_req (dev, req, req));
 
+  *ready = SANE_FALSE;
   if (req[0] == 0)
     *ready = SANE_TRUE;
-  else
-    *ready = SANE_FALSE;
 
   return SANE_STATUS_GOOD;
 }
@@ -110,11 +109,10 @@ gt68xx_generic_fix_gain (SANE_Int gain)
   if (gain < 0)
     gain = 0;
   else if (gain > 31)
-    {
-      gain = gain / 2 + 32;
-      if (gain > 63)
-	gain = 63;
-    }
+    gain += 12;
+  else if (gain > 51)
+    gain = 63;
+    
   return gain;
 }
 
@@ -195,6 +193,19 @@ gt68xx_generic_get_id (GT68xx_Device * dev)
   return SANE_STATUS_GOOD;
 }
 
+SANE_Status
+gt68xx_generic_paperfeed (GT68xx_Device * dev)
+{
+  GT68xx_Packet req;
+  SANE_Status status;
+
+  memset (req, 0, sizeof (req));
+  req[0] = 0x83;
+  req[1] = 0x01;
+
+  RIE (gt68xx_device_req (dev, req, req));
+  return SANE_STATUS_GOOD;
+}
 
 #define MAX_PIXEL_MODE 15600
 
@@ -465,7 +476,7 @@ gt68xx_generic_setup_scan (GT68xx_Device * dev,
       DBG (6,
 	   "gt68xx_generic_setup_scan: using pixel mode (GT68XX_FLAG_NO_LINEMODE)\n");
     }
-  else if (model->is_cis)
+  else if (model->is_cis && !(model->flags & GT68XX_FLAG_CIS_LAMP))
     {
       line_mode = SANE_TRUE;
       DBG (6, "gt68xx_generic_setup_scan: using line mode (CIS)\n");
@@ -581,7 +592,7 @@ gt68xx_generic_setup_scan (GT68xx_Device * dev,
       req[0x08] = LOBYTE (abs_xs);
       req[0x09] = HIBYTE (abs_xs);
       req[0x0a] = color_mode_code;
-      if (model->is_cis)
+      if (model->is_cis && !(model->flags & GT68XX_FLAG_CIS_LAMP))
 	req[0x0b] = 0x60;
       else
 	req[0x0b] = 0x20;
