@@ -611,6 +611,16 @@ gt68xx_device_carriage_home (GT68xx_Device * dev)
     return SANE_STATUS_UNSUPPORTED;
 }
 
+SANE_Status
+gt68xx_device_paperfeed (GT68xx_Device * dev)
+{
+  CHECK_DEV_ACTIVE (dev, "gt68xx_device_paperfeed");
+  if (dev->model->command_set->paperfeed)
+    return (*dev->model->command_set->paperfeed) (dev);
+  else
+    return SANE_STATUS_UNSUPPORTED;
+}
+
 SANE_Status
 gt68xx_device_start_scan (GT68xx_Device * dev)
 {
@@ -991,7 +1001,7 @@ gt68xx_device_check_result (GT68xx_Packet res, SANE_Byte command)
 {
   if (res[0] != 0)
     {
-      DBG (0, "gt68xx_device_check_result: result was %2X %2X "
+      DBG (1, "gt68xx_device_check_result: result was %2X %2X "
 	   "(expected: %2X %2X)\n", res[0], res[1], 0, command);
       return SANE_STATUS_IO_ERROR;
     }
@@ -1013,4 +1023,12 @@ gt68xx_device_get_id (GT68xx_Device * dev)
     return SANE_STATUS_UNSUPPORTED;
 }
 
+static void 
+gt68xx_device_fix_descriptor (GT68xx_Device * dev)
+{
+  SANE_Byte data[8];
+  sanei_usb_control_msg (dev->fd, 0x80, 0x06, 0x01 << 8, 0, 8, data);
+}
+
+
 /* vim: set sw=2 cino=>2se-1sn-1s{s^-1st0(0u0 smarttab expandtab: */
