@@ -42,7 +42,7 @@
 */
 
 /*
-   $Id: leo.c,v 1.11 2005/07/07 11:55:42 fzago-guest Exp $
+   $Id: leo.c,v 1.12 2005/09/19 19:57:48 fzago-guest Exp $
    Across FS-1130
 */
 
@@ -433,7 +433,7 @@ leo_identify_scanner (Leo_Scanner * dev)
 
 /* SCSI sense handler. Callback for SANE. */
 static SANE_Status
-leo_sense_handler (int scsi_fd, unsigned char *result, void __unused__ *arg)
+leo_sense_handler (int scsi_fd, unsigned char *result, void __sane_unused__ *arg)
 {
   int asc, ascq, sensekey;
   int len;
@@ -1290,7 +1290,7 @@ leo_send_halftone_pattern (Leo_Scanner * dev)
 /* Sane entry points */
 
 SANE_Status
-sane_init (SANE_Int * version_code, SANE_Auth_Callback __unused__ authorize)
+sane_init (SANE_Int * version_code, SANE_Auth_Callback __sane_unused__ authorize)
 {
   FILE *fp;
   char dev_name[PATH_MAX];
@@ -1337,7 +1337,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback __unused__ authorize)
 }
 
 SANE_Status
-sane_get_devices (const SANE_Device *** device_list, SANE_Bool __unused__ local_only)
+sane_get_devices (const SANE_Device *** device_list, SANE_Bool __sane_unused__ local_only)
 {
   Leo_Scanner *dev;
   int i;
@@ -1911,7 +1911,7 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
 }
 
 SANE_Status
-sane_set_io_mode (SANE_Handle __unused__ handle, SANE_Bool __unused__ non_blocking)
+sane_set_io_mode (SANE_Handle __sane_unused__ handle, SANE_Bool __sane_unused__ non_blocking)
 {
   SANE_Status status;
   Leo_Scanner *dev = handle;
@@ -1938,7 +1938,7 @@ sane_set_io_mode (SANE_Handle __unused__ handle, SANE_Bool __unused__ non_blocki
 }
 
 SANE_Status
-sane_get_select_fd (SANE_Handle __unused__ handle, SANE_Int __unused__ * fd)
+sane_get_select_fd (SANE_Handle __sane_unused__ handle, SANE_Int __sane_unused__ * fd)
 {
   DBG (DBG_proc, "sane_get_select_fd: enter\n");
 
