@@ -3,7 +3,7 @@
    Copyright (C) 2000 Mustek.
    Originally maintained by Tom Wang <tom.wang@mustek.com.tw>
 
-   Copyright (C) 2001, 2002 by Henning Meier-Geinitz.
+   Copyright (C) 2001 - 2004 by Henning Meier-Geinitz.
 
    This file is part of the SANE package.
 
@@ -46,7 +46,7 @@
    This file implements a SANE backend for Mustek 1200UB and similar 
    USB flatbed scanners.  */
 
-#define BUILD 16
+#define BUILD 18
 
 #include "../include/sane/config.h"
 
@@ -67,7 +67,6 @@
 #include "../include/sane/sane.h"
 #include "../include/sane/sanei.h"
 #include "../include/sane/saneopts.h"
-#include "mustek_usb.h"
 
 #define BACKEND_NAME mustek_usb
 
@@ -75,6 +74,7 @@
 #include "../include/sane/sanei_config.h"
 #include "../include/sane/sanei_usb.h"
 
+#include "mustek_usb.h"
 #include "mustek_usb_high.c"
 
 #ifndef SANE_I18N
@@ -86,8 +86,8 @@ static Mustek_Usb_Device *first_dev;
 static Mustek_Usb_Scanner *first_handle;
 static const SANE_Device **devlist = 0;
 
-/* Maximum amount of data read in one turn from USB */
-static SANE_Word max_block_size = 8 * 1024;
+/* Maximum amount of data read in one turn from USB. */
+static SANE_Word max_block_size = (8 * 1024);
 
 /* Array of newly attached devices */
 static Mustek_Usb_Device **new_dev;
@@ -764,11 +764,25 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 	  word = 0;
 	  cp = sanei_config_get_string (cp, &word);
 
+	  if (!word)
+	    {
+	      DBG (1, "sane_init: config file line %d: missing quotation mark?\n",
+		   linenumber);
+	      continue;
+	    }
+
 	  if (strcmp (word, "max_block_size") == 0)
 	    {
 	      free (word);
 	      word = 0;
 	      cp = sanei_config_get_string (cp, &word);
+	      if (!word)
+		{
+		  DBG (1, "sane_init: config file line %d: missing quotation mark?\n",
+		       linenumber);
+		  continue;
+		}
+
 	      errno = 0;
 	      max_block_size = strtol (word, &end, 0);
 	      if (end == word)
