@@ -7,7 +7,7 @@
  *  @brief The interface functions to the USB driver stuff.
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2005 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2006 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.40 - starting version of the USB support
@@ -39,6 +39,12 @@
  *        - cleanup
  * - 0.48 - added function usb_CheckAndCopyAdjs()
  * - 0.49 - changed autodetection
+ *        - added support for LiDE25 (pid 0x2220)
+ * - 0.50 - minor fix for startup reset
+ *          removed unnecessary calls to usbio_ResetLM983x()
+ *          1200DPI CIS devices don't use GrayFromColor any longer
+ * - 0.51 - added Syscan to the vendor list
+ *        - added SCANFLAG_Calibration handling
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -84,24 +90,25 @@
 /** useful for description tables
  */
 typedef struct {
-	int	  id;
+	int   id;
 	char *desc;
+	char *desc_alt;
 } TabDef, *pTabDef;
 
 /** to allow different vendors...
  */
 static TabDef usbVendors[] = {
 
-	{ 0x07B3, "Plustek"         },
-	{ 0x0400, "Mustek"          },  /* this in fact is not correct  */
-                                    /* but is used for the BearPaws */
-	{ 0x0458, "KYE/Genius"      },
-	{ 0x03F0, "Hewlett-Packard" },
-	{ 0x04B8, "Epson"           },
-	{ 0x04A9, "Canon"           },
- 	{ 0x1606, "UMAX"            },
- 	{ 0x049F, "Compaq"          },
-	{ 0xFFFF, NULL              }
+	{ 0x07B3, "Plustek",         NULL     },
+	{ 0x0400, "NSC",             "Mustek" },
+	{ 0x0458, "KYE/Genius",      NULL     },
+	{ 0x03F0, "Hewlett-Packard", NULL     },
+	{ 0x04B8, "Epson",           NULL     },
+	{ 0x04A9, "Canon",           NULL     },
+	{ 0x1606, "UMAX",            NULL     },
+	{ 0x049F, "Compaq",          NULL     },
+	{ 0x0A82, "Syscan",          NULL     },
+	{ 0xFFFF, NULL,              NULL     }
 };
 
 /** we use at least 8 megs for scanning... */
@@ -166,7 +173,8 @@ static void usb_CheckAndCopyAdjs( Plustek_Device *dev )
 /**
  * assign the values to the structures used by the currently found scanner
  */
-static void usb_initDev( pPlustek_Device dev, int idx, int handle, int vendor )
+static void
+usb_initDev( Plustek_Device *dev, int idx, int handle, int vendor )
 {
 	char     *ptr;
 	char      tmp_str1[PATH_MAX];
@@ -248,6 +256,9 @@ static void usb_initDev( pPlustek_Device dev, int idx, int handle, int vendor )
 
 		if( usbVendors[i].id == vendor ) {
 			dev->sane.vendor = usbVendors[i].desc;
+			if (dev->usbDev.Caps.workaroundFlag & _WAF_USE_ALT_DESC )
+				if (usbVendors[i].desc_alt )
+					dev->sane.vendor = usbVendors[i].desc_alt;
 			DBG( _DBG_INFO, "Vendor adjusted to: >%s<\n", dev->sane.vendor );
 			break;
 		}
@@ -260,7 +271,6 @@ static void usb_initDev( pPlustek_Device dev, int idx, int handle, int vendor )
 	if( dev->initialized >= 0 )
 		return;
 
-	usbio_ResetLM983x ( dev );
 	usb_IsScannerReady( dev );
 
 	sParam.bBitDepth     = 8;
@@ -277,20 +287,21 @@ static void usb_initDev( pPlustek_Device dev, int idx, int handle, int vendor )
 
 	/* create calibration-filename */
 	ptr = getenv ("HOME");
-    if( !usb_normFileName( dev->usbDev.ModelStr, tmp_str1, PATH_MAX )) {
+	if( !usb_normFileName( dev->usbDev.ModelStr, tmp_str1, PATH_MAX )) {
 		strcpy( tmp_str1, "plustek-default" );
 	}
 	
-    if( NULL == ptr ) {
-		sprintf( tmp_str2, "/tmp/%s-%s.cal",
-				 dev->sane.vendor, tmp_str1 );
+	if( NULL == ptr ) {
+		sprintf( tmp_str2, "/tmp/%s-%s",
+		         dev->sane.vendor, tmp_str1 );
 	} else {
-		sprintf( tmp_str2, "%s/.sane/%s-%s.cal",
-				 ptr, dev->sane.vendor, tmp_str1 );
+		sprintf( tmp_str2, "%s/.sane/%s-%s",
+		         ptr, dev->sane.vendor, tmp_str1 );
 	}
 	dev->calFile = strdup( tmp_str2 );
-	DBG( _DBG_INFO, "Calibration file-name set to:\n" );
-	DBG( _DBG_INFO, ">%s<\n", dev->calFile );
+	DBG( _DBG_INFO, "Calibration file-names set to:\n" );
+	DBG( _DBG_INFO, ">%s-coarse.cal<\n", dev->calFile );
+	DBG( _DBG_INFO, ">%s-fine.cal<\n", dev->calFile );
 
 	/* initialize the ASIC registers */
 	usb_SetScanParameters( dev, &sParam );
@@ -305,27 +316,25 @@ static void usb_initDev( pPlustek_Device dev, int idx, int handle, int vendor )
 /**
  * will be used for retrieving a Plustek device
  */
-static int usb_CheckForPlustekDevice( int handle, pPlustek_Device dev )
+static int usb_CheckForPlustekDevice( int handle, Plustek_Device *dev )
 {
 	char   tmp[50];
-    char   pcbStr[10];
-    u_char reg59[3], reg59s[3], pcbID;
-    int    i, result;
+	char   pcbStr[10];
+	u_char reg59[3], reg59s[3], pcbID;
+	int    i, result;
 
 	/*
 	 * Plustek uses the misc IO 12 to get the PCB ID
 	 * (PCB = printed circuit board), so it's possible to have one
 	 * product ID and up to 7 different devices...
 	 */
-   	DBG( _DBG_INFO, "Trying to get the pcbID of a Plustek device...\n" );
+	DBG( _DBG_INFO, "Trying to get the pcbID of a Plustek device...\n" );
 
-    /*
-   	 * get the PCB-ID
-     */
+	/* get the PCB-ID */
 	result = sanei_lm983x_read( handle, 0x59, reg59s, 3, SANE_TRUE );
-   	if( SANE_STATUS_GOOD != result ) {
+	if( SANE_STATUS_GOOD != result ) {
 		sanei_usb_close( handle );
-       	return -1;
+		return -1;
 	}
 
 	reg59[0] = 0x22;	/* PIO1: Input, PIO2: Input         */
@@ -333,42 +342,42 @@ static int usb_CheckForPlustekDevice( int handle, pPlustek_Device dev )
 	reg59[2] = 0x03;
 
 	result = sanei_lm983x_write( handle, 0x59,  reg59, 3, SANE_TRUE );
-    if( SANE_STATUS_GOOD != result ) {
+	if( SANE_STATUS_GOOD != result ) {
 		sanei_usb_close( handle );
-       	return -1;
+		return -1;
 	}
 
 	result = sanei_lm983x_read ( handle, 0x02, &pcbID, 1, SANE_TRUE );
-    if( SANE_STATUS_GOOD != result ) {
+	if( SANE_STATUS_GOOD != result ) {
 		sanei_usb_close( handle );
-       	return -1;
+		return -1;
 	}
 
 	pcbID = (u_char)((pcbID >> 2) & 0x07);
 
 	result = sanei_lm983x_read( handle, 0x59, reg59s, 3, SANE_TRUE );
-    if( SANE_STATUS_GOOD != result ) {
+	if( SANE_STATUS_GOOD != result ) {
 		sanei_usb_close( handle );
-       	return -1;
+		return -1;
 	}
 
-   	DBG( _DBG_INFO, "pcbID=0x%02x\n", pcbID );
+	DBG( _DBG_INFO, "pcbID=0x%02x\n", pcbID );
 
-    /* now roam through the setting list... */
-    strncpy( tmp, dev->usbId, 13 );
+	/* now roam through the setting list... */
+	strncpy( tmp, dev->usbId, 13 );
 	tmp[13] = '\0';
 	
 	sprintf( pcbStr, "-%u", pcbID );
 	strcat ( tmp, pcbStr );
 
-    DBG( _DBG_INFO, "Checking for device >%s<\n", tmp );
+	DBG( _DBG_INFO, "Checking for device >%s<\n", tmp );
 
 	for( i = 0; NULL != Settings[i].pIDString; i++ ) {
 
 		if( 0 == strcmp( Settings[i].pIDString, tmp )) {
-	    	DBG(_DBG_INFO, "Device description for >%s< found.\n", tmp );
+			DBG(_DBG_INFO, "Device description for >%s< found.\n", tmp );
 			usb_initDev( dev, i, handle, dev->usbDev.vendor );
-	    	return handle;
+			return handle;
 		}
 	}
 	
@@ -390,10 +399,10 @@ static void usbDev_shutdown( Plustek_Device *dev  )
 	}
 
 	if( SANE_STATUS_GOOD == sanei_usb_open( dev->sane.name, &handle )) {
-		
-    	dev->fd = handle;
-		
-    	DBG( _DBG_INFO, "Waiting for scanner-ready...\n" );
+
+		dev->fd = handle;
+
+		DBG( _DBG_INFO, "Waiting for scanner-ready...\n" );
 		usb_IsScannerReady( dev );
 	
 		if( 0 != dev->usbDev.bLampOffOnEnd ) {
@@ -401,11 +410,9 @@ static void usbDev_shutdown( Plustek_Device *dev  )
 			DBG( _DBG_INFO, "Switching lamp off...\n" );
 			usb_LampOn( dev, SANE_FALSE, SANE_FALSE );
 		}
-		
 		dev->fd = -1;
 		sanei_usb_close( handle );
 	}
-
 	usb_StopLampTimer( dev );
 }
 
@@ -424,9 +431,8 @@ static SANE_Bool usb_IsDeviceInList( char *usbIdStr )
 	for( i = 0; NULL != Settings[i].pIDString; i++ ) {
 
 		if( 0 == strncmp( Settings[i].pIDString, usbIdStr, 13 ))
-	    	return SANE_TRUE;
-	}			
-
+			return SANE_TRUE;
+	}
 	return SANE_FALSE;
 }
 
@@ -440,7 +446,7 @@ getLast( DevList *l )
 
 	while( l->next != NULL )
 		l = l->next;
-	return l; 
+	return l;
 }
 
 /** add a new entry to our internal device list, when a device is detected
@@ -699,9 +705,10 @@ static int usbDev_open( Plustek_Device *dev, DevList *devs, int keep_lock )
 		return -1;
 	}
 
+	/* need to set the handle and the detected chiptype... */
 	dev->fd = handle;
-	usbio_ResetLM983x ( dev );
-	usb_IsScannerReady( dev );
+	dev->usbDev.HwSetting.chip = (version==3 ? _LM9831:_LM9832);
+	usbio_ResetLM983x( dev );
 	dev->fd = -1;
 
 	dev->usbDev.vendor  = vendor;
@@ -725,7 +732,7 @@ static int usbDev_open( Plustek_Device *dev, DevList *devs, int keep_lock )
 		if( handle >= 0 ) {
 			if( !keep_lock )
 				sanei_access_unlock( dev->sane.name );
-			return handle;	
+			return handle;
 		}
 		
 	} else {
@@ -782,7 +789,7 @@ static int usbDev_close( Plustek_Device *dev )
  */
 static int usbDev_getCaps( Plustek_Device *dev )
 {
-	pDCapsDef scaps = &dev->usbDev.Caps;
+	DCapsDef *scaps = &dev->usbDev.Caps;
 
 	DBG( _DBG_INFO, "usbDev_getCaps()\n" );
 
@@ -802,7 +809,7 @@ static int usbDev_getCaps( Plustek_Device *dev )
 /** usbDev_getCropInfo
  * function to set the image relevant stuff
  */
-static int usbDev_getCropInfo( Plustek_Device *dev, pCropInfo ci )
+static int usbDev_getCropInfo( Plustek_Device *dev, CropInfo *ci )
 {
 	WinInfo size;
 
@@ -861,57 +868,67 @@ static int usbDev_setMap( Plustek_Device *dev, SANE_Word *map,
 
 /**
  */
-static int usbDev_setScanEnv( Plustek_Device *dev, pScanInfo si )
+static int
+usbDev_setScanEnv( Plustek_Device *dev, ScanInfo *si )
 {
+	ScanDef  *scan  = &dev->scanning;
 	DCapsDef *caps = &dev->usbDev.Caps;
 
 	DBG( _DBG_INFO, "usbDev_setScanEnv()\n" );
 
 	/* clear all the stuff */
-	memset( &dev->scanning, 0, sizeof(ScanDef));
+	memset( scan, 0, sizeof(ScanDef));
 
 	if((si->ImgDef.dwFlag & SCANDEF_Adf) &&
 	   (si->ImgDef.dwFlag & SCANDEF_ContinuousScan)) {
-		dev->scanning.sParam.dMCLK = dMCLK_ADF;
+		scan->sParam.dMCLK = dMCLK_ADF;
 	}
 
     /* Save necessary informations */
-	dev->scanning.fGrayFromColor = 0;
+	scan->fGrayFromColor = 0;
 
-	if( si->ImgDef.wDataType == COLOR_256GRAY ) {
+	/* for some devices and settings, we tweak the physical settings
+	 * how to get the image - but not in calibration mode
+	 */
+	if((si->ImgDef.dwFlag & SCANFLAG_Calibration) == 0) {
 
-		if( !(si->ImgDef.dwFlag & SCANDEF_Adf ) &&
-		  (dev->usbDev.Caps.OpticDpi.x == 1200 && si->ImgDef.xyDpi.x <= 300)) {
-			dev->scanning.fGrayFromColor = 2;
-			si->ImgDef.wDataType = COLOR_TRUE24;
-			DBG( _DBG_INFO, "* Gray from color set!\n" );
-		}
+		if( si->ImgDef.wDataType == COLOR_256GRAY ) {
 
-		if( caps->workaroundFlag & _WAF_GRAY_FROM_COLOR ) {
-			DBG( _DBG_INFO, "* Gray(8-bit) from color set!\n" );
-			dev->scanning.fGrayFromColor = 2;
-			si->ImgDef.wDataType = COLOR_TRUE24;
-		}
+			if( !(si->ImgDef.dwFlag & SCANDEF_Adf) && !usb_IsCISDevice(dev) &&
+			   (caps->OpticDpi.x == 1200 && si->ImgDef.xyDpi.x <= 300)) {
+				scan->fGrayFromColor = 2;
+				si->ImgDef.wDataType = COLOR_TRUE24;
+				DBG( _DBG_INFO, "* Gray from color set!\n" );
+			}
 
-	} else if ( si->ImgDef.wDataType == COLOR_GRAY16 ) {
-		if( caps->workaroundFlag & _WAF_GRAY_FROM_COLOR ) {
-			DBG( _DBG_INFO, "* Gray(16-bit) from color set!\n" );
-			dev->scanning.fGrayFromColor = 2;
-			si->ImgDef.wDataType = COLOR_TRUE48;
-		}
-	} else if ( si->ImgDef.wDataType == COLOR_BW ) {
-		if( caps->workaroundFlag & _WAF_BIN_FROM_COLOR ) {
-			DBG( _DBG_INFO, "* Binary from color set!\n" );
-			dev->scanning.fGrayFromColor = 10;
-			si->ImgDef.wDataType = COLOR_TRUE24;
+			if( caps->workaroundFlag & _WAF_GRAY_FROM_COLOR ) {
+				DBG( _DBG_INFO, "* Gray(8-bit) from color set!\n" );
+				scan->fGrayFromColor = 2;
+				si->ImgDef.wDataType = COLOR_TRUE24;
+			}
+
+		} else if ( si->ImgDef.wDataType == COLOR_GRAY16 ) {
+			if( caps->workaroundFlag & _WAF_GRAY_FROM_COLOR ) {
+				DBG( _DBG_INFO, "* Gray(16-bit) from color set!\n" );
+				scan->fGrayFromColor = 2;
+				si->ImgDef.wDataType = COLOR_TRUE48;
+			}
+
+		} else if ( si->ImgDef.wDataType == COLOR_BW ) {
+			if( caps->workaroundFlag & _WAF_BIN_FROM_COLOR ) {
+				DBG( _DBG_INFO, "* Binary from color set!\n" );
+				scan->fGrayFromColor = 10;
+				si->ImgDef.wDataType = COLOR_TRUE24;
+			}
 		}
 	}
-	usb_SaveImageInfo( dev, &si->ImgDef );
-	usb_GetImageInfo ( dev, &si->ImgDef, &dev->scanning.sParam.Size );
 
-	/* Flags */
-	dev->scanning.dwFlag = si->ImgDef.dwFlag & 
-	              (SCANFLAG_bgr | SCANFLAG_BottomUp |
+	usb_SaveImageInfo( dev, &si->ImgDef );
+	usb_GetImageInfo ( dev, &si->ImgDef, &scan->sParam.Size );
+
+	/* mask the flags */
+	scan->dwFlag = si->ImgDef.dwFlag & 
+	              (SCANFLAG_bgr | SCANFLAG_BottomUp | SCANFLAG_Calibration |
 	               SCANFLAG_DWORDBoundary | SCANFLAG_RightAlign |
 	               SCANFLAG_StillModule | SCANDEF_Adf | SCANDEF_ContinuousScan);
 
@@ -919,27 +936,27 @@ static int usbDev_setScanEnv( Plustek_Device *dev, pScanInfo si )
 		DBG( _DBG_INFO, "* Preview Mode set!\n" );
 	} else {
 		DBG( _DBG_INFO, "* Preview Mode NOT set!\n" );
-		dev->scanning.dwFlag |= SCANDEF_QualityScan;
+		scan->dwFlag |= SCANDEF_QualityScan;
 	}
 
-	dev->scanning.sParam.brightness  = si->siBrightness;
-	dev->scanning.sParam.contrast    = si->siContrast;
+	scan->sParam.brightness  = si->siBrightness;
+	scan->sParam.contrast    = si->siContrast;
 
-	if( dev->scanning.sParam.bBitDepth <= 8 )
-		dev->scanning.dwFlag &= ~SCANFLAG_RightAlign;
+	if( scan->sParam.bBitDepth <= 8 )
+		scan->dwFlag &= ~SCANFLAG_RightAlign;
 
-	if( dev->scanning.dwFlag & SCANFLAG_DWORDBoundary ) {
-		if( dev->scanning.fGrayFromColor && dev->scanning.fGrayFromColor < 10)
-			dev->scanning.dwBytesLine = (dev->scanning.sParam.Size.dwBytes / 3 + 3) & 0xfffffffcUL;
+	if( scan->dwFlag & SCANFLAG_DWORDBoundary ) {
+		if( scan->fGrayFromColor && scan->fGrayFromColor < 10)
+			scan->dwBytesLine = (scan->sParam.Size.dwBytes / 3 + 3) & 0xfffffffcUL;
 		else
-			dev->scanning.dwBytesLine = (dev->scanning.sParam.Size.dwBytes + 3UL) & 0xfffffffcUL;
+			scan->dwBytesLine = (scan->sParam.Size.dwBytes + 3UL) & 0xfffffffcUL;
 
 	} else {
 
-		if( dev->scanning.fGrayFromColor && dev->scanning.fGrayFromColor < 10)
-			dev->scanning.dwBytesLine = dev->scanning.sParam.Size.dwBytes / 3;
+		if( scan->fGrayFromColor && scan->fGrayFromColor < 10)
+			scan->dwBytesLine = scan->sParam.Size.dwBytes / 3;
 		else
-			dev->scanning.dwBytesLine = dev->scanning.sParam.Size.dwBytes;
+			scan->dwBytesLine = scan->sParam.Size.dwBytes;
 	}
 
 	/* on CIS based devices we have to reconfigure the illumination
@@ -947,72 +964,73 @@ static int usbDev_setScanEnv( Plustek_Device *dev, pScanInfo si )
 	 */
 	usb_AdjustCISLampSettings( dev, SANE_TRUE );
 
-	if( dev->scanning.dwFlag & SCANFLAG_BottomUp)
-		dev->scanning.lBufAdjust = -(long)dev->scanning.dwBytesLine;
+	if( scan->dwFlag & SCANFLAG_BottomUp)
+		scan->lBufAdjust = -(long)scan->dwBytesLine;
 	else
-		dev->scanning.lBufAdjust = dev->scanning.dwBytesLine;
+		scan->lBufAdjust = scan->dwBytesLine;
 
 	/* LM9831 has a BUG in 16-bit mode,
 	 * so we generate pseudo 16-bit data from 8-bit
 	 */
-	if( dev->scanning.sParam.bBitDepth > 8 ) {
+	if( scan->sParam.bBitDepth > 8 ) {
 
 		if( _LM9831 == dev->usbDev.HwSetting.chip ) {
 
-			dev->scanning.sParam.bBitDepth = 8;
-			dev->scanning.dwFlag |= SCANFLAG_Pseudo48;
-			dev->scanning.sParam.Size.dwBytes >>= 1;
+			scan->sParam.bBitDepth = 8;
+			scan->dwFlag |= SCANFLAG_Pseudo48;
+			scan->sParam.Size.dwBytes >>= 1;
 		}
 	}
 
 	/* Source selection */
-	if( dev->scanning.sParam.bSource == SOURCE_Reflection ) {
+	if( scan->sParam.bSource == SOURCE_Reflection ) {
 
-		dev->usbDev.pSource = &dev->usbDev.Caps.Normal;
-		dev->scanning.sParam.Origin.x += dev->usbDev.pSource->DataOrigin.x +
+		dev->usbDev.pSource = &caps->Normal;
+		scan->sParam.Origin.x += dev->usbDev.pSource->DataOrigin.x +
 		                                      (u_long)dev->usbDev.Normal.lLeft;
-		dev->scanning.sParam.Origin.y += dev->usbDev.pSource->DataOrigin.y +
+		scan->sParam.Origin.y += dev->usbDev.pSource->DataOrigin.y +
 		                                        (u_long)dev->usbDev.Normal.lUp;
 
-	} else if( dev->scanning.sParam.bSource == SOURCE_Transparency ) {
+	} else if( scan->sParam.bSource == SOURCE_Transparency ) {
 
-		dev->usbDev.pSource = &dev->usbDev.Caps.Positive;
-		dev->scanning.sParam.Origin.x += dev->usbDev.pSource->DataOrigin.x +
+		dev->usbDev.pSource = &caps->Positive;
+		scan->sParam.Origin.x += dev->usbDev.pSource->DataOrigin.x +
 		                                    (u_long)dev->usbDev.Positive.lLeft;
-		dev->scanning.sParam.Origin.y += dev->usbDev.pSource->DataOrigin.y +
+		scan->sParam.Origin.y += dev->usbDev.pSource->DataOrigin.y +
 		                                      (u_long)dev->usbDev.Positive.lUp;
 
-	} else if( dev->scanning.sParam.bSource == SOURCE_Negative ) {
+	} else if( scan->sParam.bSource == SOURCE_Negative ) {
 
-		dev->usbDev.pSource = &dev->usbDev.Caps.Negative;
-		dev->scanning.sParam.Origin.x += dev->usbDev.pSource->DataOrigin.x +
+		dev->usbDev.pSource = &caps->Negative;
+		scan->sParam.Origin.x += dev->usbDev.pSource->DataOrigin.x +
 		                                    (u_long)dev->usbDev.Negative.lLeft;
-		dev->scanning.sParam.Origin.y += dev->usbDev.pSource->DataOrigin.y +
+		scan->sParam.Origin.y += dev->usbDev.pSource->DataOrigin.y +
 		                                      (u_long)dev->usbDev.Negative.lUp;
 
 	} else {
 
 		dev->usbDev.pSource = &dev->usbDev.Caps.Adf;
-		dev->scanning.sParam.Origin.x += dev->usbDev.pSource->DataOrigin.x +
+		scan->sParam.Origin.x += dev->usbDev.pSource->DataOrigin.x +
 		                                      (u_long)dev->usbDev.Normal.lLeft;
-		dev->scanning.sParam.Origin.y += dev->usbDev.pSource->DataOrigin.y +
+		scan->sParam.Origin.y += dev->usbDev.pSource->DataOrigin.y +
 		                                        (u_long)dev->usbDev.Normal.lUp;
 	}
 
-	if( dev->scanning.sParam.bSource == SOURCE_ADF ) {
+	if( scan->sParam.bSource == SOURCE_ADF ) {
 
-		if( dev->scanning.dwFlag & SCANDEF_ContinuousScan )
+		if( scan->dwFlag & SCANDEF_ContinuousScan )
 			dev->usbDev.fLastScanIsAdf = SANE_TRUE;
 		else
 			dev->usbDev.fLastScanIsAdf = SANE_FALSE;
 	}
 
-    return 0;
+	return 0;
 }
 
 /**
  */
-static int usbDev_stopScan( Plustek_Device *dev )
+static int
+usbDev_stopScan( Plustek_Device *dev )
 {
 	DBG( _DBG_INFO, "usbDev_stopScan()\n" );
 
@@ -1032,7 +1050,8 @@ static int usbDev_stopScan( Plustek_Device *dev )
 
 /**
  */
-static int usbDev_startScan( Plustek_Device *dev )
+static int
+usbDev_startScan( Plustek_Device *dev )
 {
 	ScanDef *scan = &dev->scanning;
 	DBG( _DBG_INFO, "usbDev_startScan()\n" );
@@ -1067,6 +1086,10 @@ static int usbDev_startScan( Plustek_Device *dev )
 	m_fStart    = m_fFirst = SANE_TRUE;
 	m_fAutoPark = (scan->dwFlag&SCANFLAG_StillModule)?SANE_FALSE:SANE_TRUE;
 
+	if( usb_IsSheetFedDevice(dev))
+		if(usb_InCalibrationMode(dev))
+			m_fAutoPark = SANE_FALSE;
+
 	usb_StopLampTimer( dev );
 	return 0;
 }
@@ -1076,7 +1099,8 @@ static int usbDev_startScan( Plustek_Device *dev )
  * first we perform the calibration step, and then we read the image
  * line for line
  */
-static int usbDev_Prepare( Plustek_Device *dev, SANE_Byte *buf )
+static int
+usbDev_Prepare( Plustek_Device *dev, SANE_Byte *buf )
 {
 	int       result;
 	SANE_Bool use_alt_cal = SANE_FALSE;
@@ -1089,12 +1113,11 @@ static int usbDev_Prepare( Plustek_Device *dev, SANE_Byte *buf )
 	/* check the current position of the sensor and move it back
 	 * to it's home position if necessary...
 	 */
-	usb_ModuleStatus( dev );
+	if( !usb_IsSheetFedDevice(dev))
+		usb_SensorStatus( dev );
 
-	/* the CanoScan CIS devices need special handling... */
-	if((dev->usbDev.vendor == 0x04A9) &&    
-		(dev->usbDev.product==0x2206 || dev->usbDev.product==0x2207 ||
-		 dev->usbDev.product==0x220D || dev->usbDev.product==0x220E)) {
+	/* CIS devices need special handling... */
+	if( usb_IsCISDevice(dev)) {
 		use_alt_cal = SANE_TRUE;
 		
 	} else {
@@ -1116,18 +1139,31 @@ static int usbDev_Prepare( Plustek_Device *dev, SANE_Byte *buf )
 
 	if( SANE_TRUE != result ) {
 		DBG( _DBG_ERROR, "calibration failed!!!\n" );
-		return result;
+		return _E_ABORT;
 	}
 
 	if( dev->adj.cacheCalData )
 		usb_SaveCalData( dev );
 
 	DBG( _DBG_INFO, "calibration done.\n" );
+	if( usb_InCalibrationMode(dev))
+		return 0;
 
-	if( !( scan->dwFlag & SCANFLAG_Scanning )) {
+	if( !(scan->dwFlag & SCANFLAG_Scanning)) {
 
 		usleep( 10 * 1000 );
 
+		/* need to preset that here, as we need it during parameter setting
+		 */
+		scan->bLinesToSkip = (u_char)(scan->sParam.PhyDpi.y / 50);
+
+		scan->dwLinesDiscard = 0;
+		if( scan->sParam.bChannels == 3 ) {
+			scan->dwLinesDiscard = (u_long)scaps->bSensorDistance *
+			                       scan->sParam.PhyDpi.y / scaps->OpticDpi.y;
+			scan->dwLinesDiscard <<= 1;
+		}
+
 		if( !usb_SetScanParameters( dev, &scan->sParam )) {
 			DBG( _DBG_ERROR, "Setting Scan Parameters failed!\n" );
 			return 0;
@@ -1136,7 +1172,7 @@ static int usbDev_Prepare( Plustek_Device *dev, SANE_Byte *buf )
 		/* if we bypass the calibration step, we wait on lamp warmup here...
 		 */
 		if( scaps->workaroundFlag & _WAF_BYPASS_CALIBRATION ) {
-    		if( !usb_Wait4Warmup( dev )) {
+			if( !usb_Wait4Warmup( dev )) {
 				DBG( _DBG_INFO, "usbDev_Prepare() - Cancel detected...\n" );
 				return 0;
 			}
@@ -1257,7 +1293,6 @@ static int usbDev_Prepare( Plustek_Device *dev, SANE_Byte *buf )
 		/* set a funtion to process the RAW data... */
 		usb_GetImageProc( dev );
 
-		scan->bLinesToSkip = (u_char)(scan->sParam.PhyDpi.y / 50);
 		if( scan->sParam.bSource == SOURCE_ADF )
 			scan->dwFlag |= SCANFLAG_StillModule;
 
@@ -1288,7 +1323,7 @@ static int usbDev_Prepare( Plustek_Device *dev, SANE_Byte *buf )
 	/* here the NT driver uses an extra reading thread...
 	 * as the SANE stuff already forked the driver to read data, I think
 	 * we should only read data by using a function...
-     */
+	 */
 	scan->dwLinesUser = scan->sParam.Size.dwLines; 
 	if( !scan->dwLinesUser )
 		return _E_BUFFER_TOO_SMALL;
@@ -1332,7 +1367,8 @@ static int usbDev_Prepare( Plustek_Device *dev, SANE_Byte *buf )
 
 /** as the name says, read one line...
  */
-static int usbDev_ReadLine( Plustek_Device *dev )
+static int
+usbDev_ReadLine( Plustek_Device *dev )
 {
 	int      wrap;
 	u_long   cur;
