@@ -7,7 +7,7 @@
  *  @brief Main defines for the USB devices.
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2005 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2006 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.40 - starting version of the USB support
@@ -37,6 +37,13 @@
  *          to struct DeviceDef
  *        - added CRYSTAL_FREQ
  *        - added IPCDef
+ * - 0.50 - cleanup
+ *        - removed obsolete _WAF_BLACKFINE
+ *        - added MODEL_CANON_LIDE25
+ * - 0.51 - added _WAF_MISC_IO_BUTTONS plus _BUTTON stuff
+ *        - added _WAF_USE_ALT_DESC
+ *        - added DEVCAPSFLAG_SheetFed
+ *        - added dpi_thresh and lineend to motor structure
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -117,56 +124,56 @@ typedef struct {
 	u_char Red;
 	u_char Green;
 	u_char Blue;
-} RGBByteDef, *pRGBByteDef;
+} RGBByteDef;
 
 typedef struct {
 	u_short Red;
 	u_short Green;
 	u_short Blue;
-} RGBUShortDef, *pRGBUShortDef;
+} RGBUShortDef;
 
 typedef struct {
 	u_long Red;
 	u_long Green;
 	u_long Blue;
-} RGBULongDef, *pRGBULongDef;
+} RGBULongDef;
 
 typedef struct {
 	u_char a_bColor[3];
-} ColorByteDef, *pColorByteDef;
+} ColorByteDef;
 
 typedef struct {
 	u_char bHi;
 	u_char bLo;
-} HiLoDef, *pHiLoDef;
+} HiLoDef;
 
 typedef union {
 	HiLoDef HiLo[3];
-	u_short	Colors[3];
-} ColorWordDef, *pColorWordDef;
+	u_short Colors[3];
+} ColorWordDef;
 
 typedef union {
-	HiLoDef	HiLo;
-	u_short	Mono;
-} MonoWordDef, *pMonoWordDef;
+	HiLoDef HiLo;
+	u_short Mono;
+} MonoWordDef;
 
 typedef union {
 
 	u_char       *pb;
 	u_short      *pw;
-	pMonoWordDef  pmw;
-	pColorByteDef pcb;
-	pColorWordDef pcw;
-	pRGBByteDef   pb_rgb;
-	pRGBUShortDef pw_rgb;
-	pHiLoDef      philo;
+	MonoWordDef  *pmw;
+	ColorByteDef *pcb;
+	ColorWordDef *pcw;
+	RGBByteDef   *pb_rgb;
+	RGBUShortDef *pw_rgb;
+	HiLoDef      *philo;
 
-} AnyPtr, *pAnyPtr;
+} AnyPtr;
 
 typedef struct {
 	unsigned short x;
 	unsigned short y;
-} XY, *pXY;
+} XY;
 
 #define _VAR_NOT_USED(x) ((x)=(x))
 
@@ -178,9 +185,6 @@ typedef struct {
 
 #define GAIN_Target                     65535UL
 
-#define DRAM_UsedByAsic8BitMode         216             /* in KB */
-#define DRAM_UsedByAsic16BitMode        196 /*192*/     /* in KB */
-
 /** Chip-types */
 typedef enum _CHIPSET
 {
@@ -226,7 +230,8 @@ enum _DEVCAPSFLAG
 	DEVCAPSFLAG_Negative = 0x0004,
 	DEVCAPSFLAG_TPA      = 0x0006,
 	DEVCAPSFLAG_Adf      = 0x0008,
-	DEVCAPSFLAG_LargeTPA = 0x0010
+	DEVCAPSFLAG_LargeTPA = 0x0010,
+	DEVCAPSFLAG_SheetFed = 0x0020
 };
 
 /** to allow some workarounds */
@@ -241,7 +246,9 @@ enum _WORKAROUNDS
 	_WAF_SKIP_FINE          = 0x00000020, /* skip the fine calbration        */
 	_WAF_SKIP_WHITEFINE     = 0x00000040, /* skip the fine white calbration  */
 	_WAF_BIN_FROM_COLOR     = 0x00000080, /* generate binary & gray images   */
-	_WAF_GRAY_FROM_COLOR    = 0x00000100  /* from color scans                */
+	_WAF_GRAY_FROM_COLOR    = 0x00000100, /* from color scans                */
+	_WAF_MISC_IO_BUTTONS    = 0x00000200, /* special handling for buttons    */
+	_WAF_USE_ALT_DESC       = 0x00000400  /* use alternate manufacturer      */
 };
 
 /** for lamps connected to the misc I/O pins*/
@@ -256,14 +263,27 @@ enum _LAMPS
 	_MIO6   = 0x0020
 };
 
+#define _BUTTON_SHIFT 16
+#define _BUTTON_MASK  0xFF0000
+#define _TPA_SHIFT    8
+#define _TPA_MASK     0xFF00
+
+enum _BUTTONS
+{
+	_NO_BUTTON = 0,
+	_PORT0     = ((_MIO1 | _MIO2) << _BUTTON_SHIFT),
+	_PORT1     = ((_MIO3 | _MIO4) << _BUTTON_SHIFT),
+	_PORT2     = ((_MIO5 | _MIO6) << _BUTTON_SHIFT)
+};
+
 /** for encoding a misc I/O register as TPA */
-#define _TPA(register) ((u_long)(register << 16))
+#define _TPA(register) ((u_long)(register << _TPA_SHIFT))
 
 /** Mask to check for available TPA */
-#define _HAS_TPA(flag) (flag & 0xFFFF0000)
+#define _HAS_TPA(flag) (flag & _TPA_MASK)
 
 /** Get the TPA misc I/O register */
-#define _GET_TPALAMP(flag) (flag >> 16)
+#define _GET_TPALAMP(flag) ((flag >> _TPA_SHIFT) & 0xFF)
 
 /** motor types */
 typedef enum
@@ -278,8 +298,10 @@ typedef enum
 	MODEL_CANON600 ,      /**< for CanoScan 600dpi models   */
 	MODEL_CANON1200,      /**< for Canon 1200dpi models     */
 	MODEL_CANONCCD1200,   /**< for Canon CCD 1200dpi models */
+	MODEL_CANON_LIDE25,   /**< for CanoScan LiDE25          */
 	MODEL_UMAX,           /**< for UMAX 3400/3450           */
 	MODEL_UMAX1200,       /**< for UMAX 5400                */
+	MODEL_TSCAN,          /**< for Syscan Travelscan        */
 	MODEL_LAST
 } eModelDef;
 
@@ -321,17 +343,18 @@ enum SCANFLAG
 	SCANFLAG_DWORDBoundary  = 0x00020000,
 	SCANFLAG_RightAlign     = 0x00040000,
 	SCANFLAG_StillModule    = 0x00080000, 
-	SCANFLAG_StartScan      = 0x40000000,
-	SCANFLAG_Scanning       = 0x20020000,
 	SCANFLAG_Pseudo48       = 0x08000000,
-	SCANFLAG_SampleY        = 0x04000000
+	SCANFLAG_SampleY        = 0x04000000,
+	SCANFLAG_Calibration    = 0x10000000,
+	SCANFLAG_Scanning       = 0x20020000,
+	SCANFLAG_StartScan      = 0x40000000
 };
 
 typedef	struct Origins
 {
 	long lLeft;  /* How many pix to move the scanning org left, in optic res */
-	long lUp;	 /* How many pix to move the scanning or up, in optic res    */
-} OrgDef, *pOrgDef;
+	long lUp;    /* How many pix to move the scanning or up, in optic res    */
+} OrgDef;
 
 typedef struct SrcAttr
 {
@@ -343,9 +366,8 @@ typedef struct SrcAttr
 	short  DarkShadOrgY;    /**< if the device has a dark calibration strip  */
 	XY     Size;            /**< Scanning width/height, in 300 DPI base.     */
 	XY     MinDpi;          /**< Minimum dpi supported for scanning          */
-	u_char bMinDataType;    /**< Minimum data type supports                  */
 
-} SrcAttrDef, *pSrcAttrDef;
+} SrcAttrDef;
 
 typedef struct DevCaps
 {
@@ -365,7 +387,7 @@ typedef struct DevCaps
 	                            /*   _WORKAROUNDS                            */
 	u_long     lamp;            /**< for lamp: loword: normal, hiword: tpa   */
 
-} DCapsDef, *pDCapsDef;
+} DCapsDef;
 
 /**
  * for keeping intial illumination settings
@@ -381,7 +403,7 @@ typedef struct
 	u_short blue_lamp_on;
 	u_short blue_lamp_off;
 
-} IllumiDef, *pIllumiDef;
+} IllumiDef;
 
 
 /** basic register settings
@@ -400,7 +422,7 @@ typedef struct HWDefault
 	u_short             wGreenPWMDutyCycleLow;
 	u_short             wGreenPWMDutyCycleHigh;
 	/* Registers */
-	u_char              bSensorConfiguration;	/* 0x0b */
+	u_char              bSensorConfiguration;           /* 0x0b */
 	/* Sensor control settings */
 	u_char              bReg_0x0c;
 	u_char              bReg_0x0d;
@@ -418,13 +440,13 @@ typedef struct HWDefault
 	/* initial illumination settings */
 	IllumiDef           illu_mono;
 	IllumiDef           illu_color;
-	
+
 	/* 0x1a & 0x1b, remember the u_char order is not Intel
 	 * format, you have to pay your attention when you
 	 * write this value to register.
 	 */
 	u_short             StepperPhaseCorrection;	
-	
+
 	/* Sensor Pixel Configuration
 	 * Actually, the wActivePixelsStart will be set to 0 for shading purpose.
 	 * We have to keep these values to adjust the origins when user does the
@@ -434,7 +456,7 @@ typedef struct HWDefault
 	u_char              bOpticBlackEnd;         /* 0x1d        */
 	u_short             wActivePixelsStart;     /* 0x1e & 0x1f */
 	u_short             wLineEnd;               /* 0x20 & 0x21 */
-	
+
 	/* illumination settings (runtime) */
 	u_short             red_lamp_on;            /* 0x2c & 0x2d */
 	u_short             red_lamp_off;           /* 0x2e & 0x2f */
@@ -442,7 +464,7 @@ typedef struct HWDefault
 	u_short             green_lamp_off;         /* 0x32 & 0x33 */
 	u_short             blue_lamp_on;           /* 0x34 & 0x35 */
 	u_short             blue_lamp_off;          /* 0x36 & 0x37 */
-	
+
 	/* Misc */
 	u_char              bReg_0x45;
 	u_short             wStepsAfterPaperSensor2;/* 0x4c & 0x4d */
@@ -459,11 +481,11 @@ typedef struct HWDefault
 	u_char              bReg_0x5c;
 	u_char              bReg_0x5d;
 	u_char              bReg_0x5e;
-	
+
 	eChipDef            chip;           /* chiptype               */
 	eModelDef           motorModel;     /* to identify used motor */
 	double              gamma;          /* gamma setting          */
-} HWDef, *pHWDef;
+} HWDef;
 
 /** device description during runtime
  */
@@ -474,7 +496,7 @@ typedef struct DeviceDef
 	int         product;       /**< product ID                               */
 	DCapsDef    Caps;          /**< pointer to the attribute of current dev  */
 	HWDef       HwSetting;     /**< Pointer to the characteristics of device */
-	pSrcAttrDef pSource;       /**< Scanning src, it's equal to Caps.Normal  */
+	SrcAttrDef *pSource;       /**< Scanning src, it's equal to Caps.Normal  */
 	                           /**< on the source that the user specified.   */
 	OrgDef      Normal;        /**< Reflection - Pix to adjust scanning orgs */
 	OrgDef      Positive;      /**< Pos film - Pix to adjust scanning orgs   */
@@ -488,17 +510,17 @@ typedef struct DeviceDef
 	SANE_Bool   fLastScanIsAdf;/**<                                          */
 	u_char      a_bRegs[0x80]; /**< our global register file                 */
 
-} DeviceDef, *pDeviceDef;
+} DeviceDef;
 
 
 typedef struct Settings
 {
-    char      *pIDString;
-	pDCapsDef  pDevCaps;
-	pHWDef     pHwDef;
+	char      *pIDString;
+	DCapsDef  *pDevCaps;
+	HWDef     *pHwDef;
 	char      *pModelString;
 
-} SetDef, *pSetDef;
+} SetDef;
 
 /**
  */
@@ -517,7 +539,7 @@ typedef struct
 	                      /*   request dpi (CCD lines distance)              */
 	u_long dwTotalBytes;  /**< Total bytes per scan                          */
 
-} WinInfo, *pWinInfo;
+} WinInfo;
 
 /**
  */
@@ -554,7 +576,7 @@ typedef struct
 	int     swOffset[3];  /**< for calibration adjustment                  */
 	int     swGain[3];    /**< for calibration adjustment                  */
 
-} ScanParam, *pScanParam;
+} ScanParam;
 
 struct Plustek_Device;
 
@@ -605,7 +627,7 @@ typedef struct ScanDef
 
 	u_char  bLinesToSkip;     /**< how many lines to skip at start */
 
-} ScanDef, *pScanDef;
+} ScanDef;
 
 
 /** max number of different colck settings */
@@ -619,7 +641,7 @@ typedef struct
 	u_char pwm_duty;            /**< PWM duty cycles */
 	u_char scan_lines_per_line; /**< lines to scan to obtain 1 real line
                                       will be used in 16bit color modes only */
-} MDef, *pMDef;
+} MDef;
 
 /** according to the CCD and motor, we provide various settings
  */
@@ -631,21 +653,24 @@ typedef struct {
 	u_char  pwm_duty_fast;  /**< PWM duty during fast movement */
 	u_char  mclk_fast;      /**< MCLK during fast movement     */
 
-    /**
-     * here we define some ranges for better supporting
-     * non-Plustek devices with it's different hardware
-     * we can set the MCLK and the motor PWM stuff for color
-     * and gray modes (8bit and 14/16bit modes)
-     *    0    1     2     3     4     5     6      7     8      9
-     * <= 75 <=100 <=150 <=200 <=300 <=400 <=600 <= 800 <=1200 <=2400DPI
-     */
+	u_short dpi_thresh;
+	u_short lineend;
+
+	/**
+	 * here we define some ranges for better supporting
+	 * non-Plustek devices with it's different hardware
+	 * we can set the MCLK and the motor PWM stuff for color
+	 * and gray modes (8bit and 14/16bit modes)
+	 *    0    1     2     3     4     5     6      7     8      9
+	 * <= 75 <=100 <=150 <=200 <=300 <=400 <=600 <= 800 <=1200 <=2400DPI
+	 */
 	MDef   motor_sets[_MAX_CLK];	/**< motor PWM settings during scan      */
 	double color_mclk_8[_MAX_CLK];  /**< MCLK settings for color scan        */
 	double color_mclk_16[_MAX_CLK]; /**< MCLK settings for color (16bit) scan*/
 	double gray_mclk_8[_MAX_CLK];   /**< MCLK settings for gray scan         */
 	double gray_mclk_16[_MAX_CLK];  /**< MCLK settings for gray (16bit) scan */
 
-} ClkMotorDef, *pClkMotorDef;
+} ClkMotorDef;
 
 /** for transferring some info between child and parent after calibration
  */
