@@ -7,7 +7,7 @@
  *  @brief Calibration routines for CanoScan CIS devices.
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2005 Gerhard Jaeger <gerhard@gjaeger.de><br>
+ * Copyright (C) 2001-2006 Gerhard Jaeger <gerhard@gjaeger.de><br>
  * Large parts Copyright (C) 2003 Christopher Montgomery <monty@xiph.org>
  *
  * Montys' comment:
@@ -40,6 +40,13 @@
  * - 0.48  - cleanup
  * - 0.49  - a_bRegs is now part of the device structure
  *         - fixed lampsetting in cano_AdjustLightsource()
+ * - 0.50  - tried to use the settings from SANE-1.0.13
+ *         - added _TWEAK_GAIN to allow increasing GAIN during
+ *           lamp coarse calibration
+ *         - added also speedtest
+ *         - fixed segfault in fine calibration
+ * - 0.51  - added fine calibration cache
+ *         - usb_SwitchLamp() now really switches off the sensor
  *
  * This file is part of the SANE package.
  *
@@ -81,26 +88,40 @@
  * <hr>
  */
 
-/** 0 for not ready,  1 pos white lamp on,  2 lamp off */ 
-static int strip_state=0; 
-  
+/* un-/comment the following to en-/disable lamp coarse calibration to tweak
+ * the initial AFE gain settings
+ */
+#define _TWEAK_GAIN 1
+
+/** 0 for not ready,  1 pos white lamp on,  2 lamp off */
+static int strip_state = 0;
+
 /** depending on the strip state, the sensor is moved to the shading position
  *  and the lamp ist switched on
  */
-static int cano_PrepareToReadWhiteCal( Plustek_Device *dev )
+static int
+cano_PrepareToReadWhiteCal( Plustek_Device *dev, SANE_Bool mv2shading_pos )
 {
-	pHWDef hw = &dev->usbDev.HwSetting;
+	SANE_Bool goto_shading_pos = SANE_TRUE;
+	HWDef     *hw = &dev->usbDev.HwSetting;
 	
 	switch (strip_state) {
 		case 0:
-			if(!usb_ModuleToHome( dev, SANE_TRUE )) {
-				DBG( _DBG_ERROR, "cano_PrepareToReadWhiteCal() failed\n" );
-				return _E_LAMP_NOT_IN_POS;
+			if( !usb_IsSheetFedDevice(dev)) {
+				if(!usb_ModuleToHome( dev, SANE_TRUE )) {
+					DBG( _DBG_ERROR, "cano_PrepareToReadWhiteCal() failed\n" );
+					return _E_LAMP_NOT_IN_POS;
+				}
+			} else {
+				goto_shading_pos = mv2shading_pos;
 			}
-			if( !usb_ModuleMove(dev, MOVE_Forward,
-				(u_long)dev->usbDev.pSource->ShadingOriginY)) {
-				DBG( _DBG_ERROR, "cano_PrepareToReadWhiteCal() failed\n" );
-				return _E_LAMP_NOT_IN_POS;
+
+			if( goto_shading_pos ) {
+				if( !usb_ModuleMove(dev, MOVE_Forward,
+					(u_long)dev->usbDev.pSource->ShadingOriginY)) {
+					DBG( _DBG_ERROR, "cano_PrepareToReadWhiteCal() failed\n" );
+					return _E_LAMP_NOT_IN_POS;
+				}
 			}
 	    	break;
 		case 2:
@@ -117,22 +138,25 @@ static int cano_PrepareToReadWhiteCal( Plustek_Device *dev )
 	return 0;
 }
 
-/**
+/** also here, depending on the strip state, the sensor will be moved to
+ * the shading position and the lamp will be switched off
  */
-static int cano_PrepareToReadBlackCal( Plustek_Device *dev )
+static int
+cano_PrepareToReadBlackCal( Plustek_Device *dev )
 {
 	if( strip_state == 0 )
-		if(cano_PrepareToReadWhiteCal(dev))
+		if(cano_PrepareToReadWhiteCal(dev, SANE_FALSE))
 			return SANE_FALSE;
 			
 	if( strip_state != 2 ) {
 	    /*
-		 * if we have dark shading strip, there's no need to switch
-	     * the lamp off
+		 * if we have a dark shading strip, there's no need to switch
+	     * the lamp off, leave in on a go to that strip
 		 */
 		if( dev->usbDev.pSource->DarkShadOrgY >= 0 ) {
 
-			usb_ModuleToHome( dev, SANE_TRUE );
+			if( !usb_IsSheetFedDevice(dev))
+				usb_ModuleToHome( dev, SANE_TRUE );
 			usb_ModuleMove  ( dev, MOVE_Forward,
 			                       (u_long)dev->usbDev.pSource->DarkShadOrgY );
 			dev->usbDev.a_bRegs[0x45] &= ~0x10;
@@ -149,9 +173,10 @@ static int cano_PrepareToReadBlackCal( Plustek_Device *dev )
 	return 0;
 }
 
-/**
+/** according to the strip-state we switch the lamp on
  */
-static int cano_LampOnAfterCalibration( Plustek_Device *dev )
+static int
+cano_LampOnAfterCalibration( Plustek_Device *dev )
 {
 	HWDef *hw = &dev->usbDev.HwSetting;
 
@@ -170,25 +195,24 @@ static int cano_LampOnAfterCalibration( Plustek_Device *dev )
 }
 
 /** function to adjust the CIS lamp-off setting for a given channel.
- * @param min - pointer to the ON point of the CIS-channel
- * @param max - pointer to the max OFF point of the CIS-channel
+ * @param min - pointer to the min OFF point for the CIS-channel
+ * @param max - pointer to the max OFF point for the CIS-channel
  * @param off - pointer to the current OFF point of the CIS-channel
  * @param val - current value to check
  * @return returns 0 if the value is fine, 1, if we need to adjust 
  */
-static int cano_adjLampSetting( u_short *min,
-                                u_short *max, u_short *off, u_short val )
+static int
+cano_adjLampSetting( u_short *min, u_short *max, u_short *off, u_short val )
 {
 	u_long newoff = *off;
 
 	/* perfect value, no need to adjust
-	 * val  [53440..59440] is perfect
+	 * val  [53440..61440] is perfect
 	 */
-	if((val < (IDEAL_GainNormal-2000)) && (val > (IDEAL_GainNormal-8000)))
+	if((val < (IDEAL_GainNormal)) && (val > (IDEAL_GainNormal-8000)))
 		return 0;
 
-	if(val >= (IDEAL_GainNormal-2000)) {
-
+	if(val >= (IDEAL_GainNormal-4000)) {
 		DBG(_DBG_INFO2, "* TOO BRIGHT --> reduce\n" );
 		*max   = newoff;
 		*off = ((newoff + *min)>>1);
@@ -222,10 +246,11 @@ static int cano_adjLampSetting( u_short *min,
  * [Monty changes]: On the CanoScan at least, the default lamp
  * settings are several *hundred* percent too high and vary from
  * scanner-to-scanner by 20-50%. This is only for CIS devices 
- * where the lamp_off parameter is adjustable; I'd make it more general, 
+ * where the lamp_off parameter is adjustable; I'd make it more general,
  * but I only have the CIS hardware to test.
  */
-static int cano_AdjustLightsource( Plustek_Device *dev )
+static int
+cano_AdjustLightsource( Plustek_Device *dev )
 {
 	char         tmp[40];
 	int          i;
@@ -241,17 +266,16 @@ static int cano_AdjustLightsource( Plustek_Device *dev )
 
 	DBG( _DBG_INFO, "cano_AdjustLightsource()\n" );
 
-	if( !(hw->bReg_0x26 & _ONE_CH_COLOR)) {
+	if( !usb_IsCISDevice(dev)) {
 		DBG( _DBG_INFO, "- function skipped, CCD device!\n" );
-		
-		if( !usb_Wait4Warmup( dev )) {
-			DBG( _DBG_ERROR, "cano_AdjustLightsource() - CANCEL detected\n" );
-			return SANE_FALSE;
-		}
-		return SANE_TRUE;	
+
+		/* HEINER: we might have to tweak the PWM for the lamps */
+		return SANE_TRUE;
 	}
 
-	/* define the strip to scan for coarse calibration;  done at 300dpi */
+	/* define the strip to scan for coarse calibration
+	 * done at optical resolution.
+	 */
 	m_ScanParam.Size.dwLines  = 1;
 	m_ScanParam.Size.dwPixels = scaps->Normal.Size.x *
 	                            scaps->OpticDpi.x / 300UL;
@@ -271,7 +295,8 @@ static int cano_AdjustLightsource( Plustek_Device *dev )
 	DBG( _DBG_INFO2, "* Bytes    = %lu\n", m_ScanParam.Size.dwBytes  );
 	DBG( _DBG_INFO2, "* Origin.X = %u\n",  m_ScanParam.Origin.x );
 
-	max_rgb.Red   = max_rgb.Green = max_rgb.Blue = 0xffff;
+	/* init... */
+	max_rgb.Red   = max_rgb.Green = max_rgb.Blue = 0x3fff;
 	min_rgb.Red   = hw->red_lamp_on;
 	min_rgb.Green = hw->green_lamp_on;
 	min_rgb.Blue  = hw->blue_lamp_on;
@@ -282,8 +307,10 @@ static int cano_AdjustLightsource( Plustek_Device *dev )
 		return SANE_TRUE;
 	}
 
-	/* we probably should preset gain to some reasonably good value */
-#if 0
+	/* we probably should preset gain to some reasonably good value
+	 * i.e. 0x0a as it's done by Canon within their Windoze driver!
+	 */
+#ifdef _TWEAK_GAIN
 	for( i=0x3b; i<0x3e; i++ )
 		dev->usbDev.a_bRegs[i] = 0x0a;
 #endif
@@ -329,16 +356,17 @@ static int cano_AdjustLightsource( Plustek_Device *dev )
 			for (dwLoop2 = dwDiv, dwR = dwG = dwB = 0; dwLoop2; dwLoop2--, dw++) {
 
 				if( m_ScanParam.bDataType == SCANDATATYPE_Color ) {
-					if( hw->bReg_0x26 & _ONE_CH_COLOR ) {
+
+					if( usb_IsCISDevice(dev)) {
 						dwR += ((u_short*)scanbuf)[dw];
 						dwG += ((u_short*)scanbuf)
-										[dw+m_ScanParam.Size.dwPhyPixels+1];
+						       [dw+m_ScanParam.Size.dwPhyPixels+1];
 						dwB += ((u_short*)scanbuf)
-									[dw+(m_ScanParam.Size.dwPhyPixels+1)*2];
+						       [dw+(m_ScanParam.Size.dwPhyPixels+1)*2];
             		} else {
-						dwR += ((pRGBUShortDef)scanbuf)[dw].Red;
-						dwG += ((pRGBUShortDef)scanbuf)[dw].Green;
-						dwB += ((pRGBUShortDef)scanbuf)[dw].Blue;
+						dwR += ((RGBUShortDef*)scanbuf)[dw].Red;
+						dwG += ((RGBUShortDef*)scanbuf)[dw].Green;
+						dwB += ((RGBUShortDef*)scanbuf)[dw].Blue;
 					}
 				} else {
 					dwG += ((u_short*)scanbuf)[dw];
@@ -388,9 +416,6 @@ static int cano_AdjustLightsource( Plustek_Device *dev )
 		res_g = cano_adjLampSetting( &min_rgb.Green, &max_rgb.Green,
 		                             &hw->green_lamp_off, tmp_rgb.Green );
 
-		/* need to be set in any case! */
-		usb_AdjustLamps(dev);
-
 		/* nothing adjusted, so stop here */
 		if((res_r == 0) && (res_g == 0) && (res_b == 0))
 			break;
@@ -401,19 +426,21 @@ static int cano_AdjustLightsource( Plustek_Device *dev )
 		if((res_r == 10) && (res_g == 10) && (res_b == 10))
 			break;
 
-		/* we raise the gain for channel, that have been limited */
+		/* we raise the gain for channels, that have been limited */
+#ifdef _TWEAK_GAIN
 		if( res_r == 10 ) {
-			if( dev->usbDev.a_bRegs[0x3b] < 0x0b)
+			if( dev->usbDev.a_bRegs[0x3b] < 0xf)
 				dev->usbDev.a_bRegs[0x3b]++;
 		}
 		if( res_g == 10 ) {
-			if( dev->usbDev.a_bRegs[0x3c] < 0x0b)
+			if( dev->usbDev.a_bRegs[0x3c] < 0x0f)
 				dev->usbDev.a_bRegs[0x3c]++;
 		}
 		if( res_b == 10 ) {
-			if( dev->usbDev.a_bRegs[0x3d] < 0x0b)
+			if( dev->usbDev.a_bRegs[0x3d] < 0x0f)
 				dev->usbDev.a_bRegs[0x3d]++;
 		}
+#endif
 
 		/* now decide what to do:
 		 * if we were too bright, we have to rerun the loop in any
@@ -424,12 +451,8 @@ static int cano_AdjustLightsource( Plustek_Device *dev )
 		if( i >= 10 ) {
 			DBG(_DBG_INFO, "* 10 times limit reached, still too dark!!!\n");
 			break;
-#if 0
-		} else {
-			DBG(_DBG_INFO2, "* CIS-Warmup, 1s!!!\n" );
-			sleep( 1 );
-#endif
 		}
+		usb_AdjustLamps(dev, SANE_TRUE);
 	}
 
 	DBG( _DBG_INFO, "* red_lamp_on    = %u\n", hw->red_lamp_on  );
@@ -479,7 +502,8 @@ cano_adjGainSetting( u_char *min, u_char *max, u_char *gain,u_long val )
  *
  * adjLightsource, above, steals most of this function's thunder.
  */
-static SANE_Bool cano_AdjustGain( Plustek_Device *dev )
+static SANE_Bool
+cano_AdjustGain( Plustek_Device *dev )
 {
 	char      tmp[40];
 	int       i = 0, adj = 1;
@@ -517,10 +541,8 @@ static SANE_Bool cano_AdjustGain( Plustek_Device *dev )
 
 	m_ScanParam.Size.dwBytes  = m_ScanParam.Size.dwPixels * 2;
 
-	if( hw->bReg_0x26 & _ONE_CH_COLOR &&
-    	m_ScanParam.bDataType == SCANDATATYPE_Color ) {
+	if( usb_IsCISDevice(dev) && m_ScanParam.bDataType == SCANDATATYPE_Color)
 		m_ScanParam.Size.dwBytes *=3;
-	}
 
 	m_ScanParam.Origin.x = (u_short)((u_long) hw->wActivePixelsStart *
 	                                                300UL / scaps->OpticDpi.x);
@@ -572,16 +594,16 @@ static SANE_Bool cano_AdjustGain( Plustek_Device *dev )
 				/* do some averaging... */
 				for (dwLoop2 = dwDiv, dwR=dwG=dwB=0; dwLoop2; dwLoop2--, dw++) {
 
-					if( hw->bReg_0x26 & _ONE_CH_COLOR ) {
+					if( usb_IsCISDevice(dev)) {
 						dwR += ((u_short*)scanbuf)[dw];
 						dwG += ((u_short*)scanbuf)
-										[dw+m_ScanParam.Size.dwPhyPixels+1];
+						       [dw+m_ScanParam.Size.dwPhyPixels+1];
 						dwB += ((u_short*)scanbuf)
-									[dw+(m_ScanParam.Size.dwPhyPixels+1)*2];
+						       [dw+(m_ScanParam.Size.dwPhyPixels+1)*2];
             		} else {
-						dwR += ((pRGBUShortDef)scanbuf)[dw].Red;
-						dwG += ((pRGBUShortDef)scanbuf)[dw].Green;
-						dwB += ((pRGBUShortDef)scanbuf)[dw].Blue;
+						dwR += ((RGBUShortDef*)scanbuf)[dw].Red;
+						dwG += ((RGBUShortDef*)scanbuf)[dw].Green;
+						dwB += ((RGBUShortDef*)scanbuf)[dw].Blue;
 					}
 				}
 				dwR = dwR / dwDiv;
@@ -597,8 +619,8 @@ static SANE_Bool cano_AdjustGain( Plustek_Device *dev )
 			}
 
 			DBG(_DBG_INFO2, "MAX(R,G,B)= 0x%04x(%u), 0x%04x(%u), 0x%04x(%u)\n",
-									max_rgb.Red, max_rgb.Red, max_rgb.Green,
-									max_rgb.Green, max_rgb.Blue, max_rgb.Blue );
+			                 max_rgb.Red, max_rgb.Red, max_rgb.Green,
+			                 max_rgb.Green, max_rgb.Blue, max_rgb.Blue );
 
 			adj  = cano_adjGainSetting(min  , max  ,dev->usbDev.a_bRegs+0x3b,max_rgb.Red  );
 			adj += cano_adjGainSetting(min+1, max+1,dev->usbDev.a_bRegs+0x3c,max_rgb.Green);
@@ -629,8 +651,9 @@ static SANE_Bool cano_AdjustGain( Plustek_Device *dev )
 
 /**
  */
-static int cano_GetNewOffset( Plustek_Device *dev, u_long *val, int channel, 
-                              signed char *low, signed char *now, signed char *high )
+static int
+cano_GetNewOffset( Plustek_Device *dev, u_long *val, int channel,
+                   signed char *low, signed char *now, signed char *high )
 {
 	/* if we're too black, we're likely off the low end */
 	if( val[channel] <= 16 ) {
@@ -670,12 +693,14 @@ static int cano_GetNewOffset( Plustek_Device *dev, u_long *val, int channel,
    Plustek's example code disagrees with NatSemi's docs; going by the
    docs works better, I will assume the docs are correct. --Monty */
 
-static int cano_AdjustOffset( Plustek_Device *dev )
+static int
+cano_AdjustOffset( Plustek_Device *dev )
 {
-	char   tmp[40];
-	int    i, adj;
-	u_long dw, dwPixels;
-	u_long dwSum[3];
+	char    tmp[40];
+	int     i, adj;
+	u_short r, g, b;
+	u_long  dw, dwPixels;
+	u_long  dwSum[3];
 
 	signed char low[3]  = {-32,-32,-32 };
 	signed char now[3]  = {  0,  0,  0 };
@@ -701,17 +726,15 @@ static int cano_AdjustOffset( Plustek_Device *dev )
 	m_ScanParam.Size.dwLines  = 1;
 	m_ScanParam.Size.dwPixels = scaps->Normal.Size.x*scaps->OpticDpi.x/300UL;
 
-	if( hw->bReg_0x26 & _ONE_CH_COLOR )
+	if( usb_IsCISDevice(dev))
 		dwPixels = m_ScanParam.Size.dwPixels;
 	else
-		dwPixels = (u_long)(hw->bOpticBlackEnd - hw->bOpticBlackStart );
+		dwPixels = (u_long)(hw->bOpticBlackEnd - hw->bOpticBlackStart);
 
 	m_ScanParam.Size.dwBytes = m_ScanParam.Size.dwPixels * 2;
 
-	if( hw->bReg_0x26 & _ONE_CH_COLOR &&
-		m_ScanParam.bDataType == SCANDATATYPE_Color ) {
+	if( usb_IsCISDevice(dev) && m_ScanParam.bDataType == SCANDATATYPE_Color)
 		m_ScanParam.Size.dwBytes *= 3;
-	}
 
 	m_ScanParam.Origin.x = (u_short)((u_long)hw->bOpticBlackStart * 300UL /
 	                                              dev->usbDev.Caps.OpticDpi.x);
@@ -723,7 +746,7 @@ static int cano_AdjustOffset( Plustek_Device *dev )
 		return SANE_FALSE;
 	}
 
-	DBG( _DBG_INFO2, "S.dwPixels  = %lu\n", m_ScanParam.Size.dwPixels );		
+	DBG( _DBG_INFO2, "S.dwPixels  = %lu\n", m_ScanParam.Size.dwPixels );
 	DBG( _DBG_INFO2, "dwPixels    = %lu\n", dwPixels );
 	DBG( _DBG_INFO2, "dwPhyBytes  = %lu\n", m_ScanParam.Size.dwPhyBytes );
 	DBG( _DBG_INFO2, "dwPhyPixels = %lu\n", m_ScanParam.Size.dwPhyPixels );
@@ -751,27 +774,30 @@ static int cano_AdjustOffset( Plustek_Device *dev )
 
 			for (dw = 0; dw < dwPixels; dw++) {
 
-				if( hw->bReg_0x26 & _ONE_CH_COLOR ) {
+				if( usb_IsCISDevice(dev)) {
 
-					dwSum[0] += ((u_short*)scanbuf)[dw];
-					dwSum[1] += ((u_short*)
-					       scanbuf)[dw+m_ScanParam.Size.dwPhyPixels+1];
-					dwSum[2] += ((u_short*)
-					       scanbuf)[dw+(m_ScanParam.Size.dwPhyPixels+1)*2];
+					r = ((u_short*)scanbuf)[dw];
+					g = ((u_short*)scanbuf)[dw+m_ScanParam.Size.dwPhyPixels+1];
+					b = ((u_short*)scanbuf)[dw+(m_ScanParam.Size.dwPhyPixels+1)*2];
 
 				} else {
-					dwSum[0] += ((pRGBUShortDef)scanbuf)[dw].Red;
-					dwSum[1] += ((pRGBUShortDef)scanbuf)[dw].Green;
-					dwSum[2] += ((pRGBUShortDef)scanbuf)[dw].Blue;
+					r = ((RGBUShortDef*)scanbuf)[dw].Red;
+					g = ((RGBUShortDef*)scanbuf)[dw].Green;
+					b = ((RGBUShortDef*)scanbuf)[dw].Blue;
 				}
+
+				dwSum[0] += r;
+				dwSum[1] += g;
+				dwSum[2] += b;
+
 			}
 
 			DBG( _DBG_INFO2, "RedSum   = %lu, ave = %lu\n",
-			                                    dwSum[0], dwSum[0]/dwPixels );
+			                            dwSum[0], dwSum[0]/dwPixels );
 			DBG( _DBG_INFO2, "GreenSum = %lu, ave = %lu\n",
-			                                    dwSum[1], dwSum[1] /dwPixels );
+			                           dwSum[1], dwSum[1] /dwPixels );
 			DBG( _DBG_INFO2, "BlueSum  = %lu, ave = %lu\n",
-			                                    dwSum[2], dwSum[2] /dwPixels );
+			                           dwSum[2], dwSum[2] /dwPixels );
 
 			/* do averaging for each channel */
 			dwSum[0] /= dwPixels;
@@ -795,7 +821,7 @@ static int cano_AdjustOffset( Plustek_Device *dev )
 			for( dw = 0; dw < dwPixels; dw++ )
 				dwSum[0] += ((u_short*)scanbuf)[dw];
 
-			dwSum [0] /= dwPixels;
+			dwSum[0] /= dwPixels;
 			DBG( _DBG_INFO2, "Sum=%lu, ave=%lu\n", dwSum[0],dwSum[0]/dwPixels);
 
 			adj = cano_GetNewOffset( dev, dwSum, 0, low, now, high );
@@ -815,8 +841,8 @@ static int cano_AdjustOffset( Plustek_Device *dev )
 	}
 
 	if( m_ScanParam.bDataType == SCANDATATYPE_Color ) {
-    	dev->usbDev.a_bRegs[0x38] = now[0];	
-	    dev->usbDev.a_bRegs[0x39] = now[1];	
+		dev->usbDev.a_bRegs[0x38] = now[0];
+		dev->usbDev.a_bRegs[0x39] = now[1];
 		dev->usbDev.a_bRegs[0x3a] = now[2];
 	} else {
 		dev->usbDev.a_bRegs[0x38] =
@@ -830,15 +856,13 @@ static int cano_AdjustOffset( Plustek_Device *dev )
 
 /** usb_AdjustDarkShading
  * fine calibration part 1
- *
  */
-static SANE_Bool cano_AdjustDarkShading( Plustek_Device *dev )
+static SANE_Bool
+cano_AdjustDarkShading( Plustek_Device *dev, u_short cal_dpi )
 {
 	char         tmp[40];
 	ScanParam   *param   = &dev->scanning.sParam;
 	ScanDef     *scan    = &dev->scanning;
-	DCapsDef    *scaps   = &dev->usbDev.Caps;
-	HWDef       *hw      = &dev->usbDev.HwSetting;
 	u_char      *scanbuf = scan->pScanBuffer;
 	u_short     *bufp;
 	unsigned int i, j;
@@ -849,34 +873,15 @@ static SANE_Bool cano_AdjustDarkShading( Plustek_Device *dev )
 	if( usb_IsEscPressed())
 		return SANE_FALSE;
 
-	m_ScanParam = scan->sParam;
-
-#if 0
-	if( m_ScanParam.PhyDpi.x > 75)
-		m_ScanParam.Size.dwLines = 64;
-	else
-#endif
-		m_ScanParam.Size.dwLines = 32;
-
-	m_ScanParam.Origin.y  = 0;
-	m_ScanParam.bBitDepth = 16;
-
-	m_ScanParam.UserDpi.y    = scaps->OpticDpi.y;
-	m_ScanParam.Size.dwBytes = m_ScanParam.Size.dwPixels * 2;
-	if( hw->bReg_0x26 & _ONE_CH_COLOR &&
-		m_ScanParam.bDataType == SCANDATATYPE_Color ) {
-		m_ScanParam.Size.dwBytes *= 3;
-	}
-
+	usb_PrepareFineCal( dev, &m_ScanParam, cal_dpi );
 	m_ScanParam.bCalibration = PARAM_DarkShading;
-	m_ScanParam.dMCLK        = dMCLK;
 
 	sprintf( tmp, "fine-dark.raw" );
 	dumpPicInit( &m_ScanParam, tmp );
 
 	usb_SetScanParameters( dev, &m_ScanParam );
 	if( usb_ScanBegin( dev, SANE_FALSE ) &&
-		usb_ScanReadImage( dev, scanbuf, m_ScanParam.Size.dwTotalBytes)) {
+	    usb_ScanReadImage( dev, scanbuf, m_ScanParam.Size.dwTotalBytes)) {
 
 		dumpPic( tmp, scanbuf, m_ScanParam.Size.dwTotalBytes );
 
@@ -892,7 +897,7 @@ static SANE_Bool cano_AdjustDarkShading( Plustek_Device *dev )
 	if( scan->sParam.bDataType == SCANDATATYPE_Color ) {
 		
 		stepW = m_ScanParam.Size.dwPhyPixels;
-		if( hw->bReg_0x26 & _ONE_CH_COLOR ) 
+		if( usb_IsCISDevice(dev))
 			step = m_ScanParam.Size.dwPhyPixels + 1;
 		else
 			step = (m_ScanParam.Size.dwPhyPixels*3) + 1;
@@ -902,17 +907,17 @@ static SANE_Bool cano_AdjustDarkShading( Plustek_Device *dev )
 			red   = 0;
 			green = 0;
 			blue  = 0;
-			if( hw->bReg_0x26 & _ONE_CH_COLOR )
+			if( usb_IsCISDevice(dev))
 				bufp = ((u_short *)scanbuf)+i;
 			else
 				bufp = ((u_short *)scanbuf)+(i*3);
 
 			for( j=0; j<m_ScanParam.Size.dwPhyLines; j++ ) {
 
-				if( hw->bReg_0x26 & _ONE_CH_COLOR ) {
-					red   += *bufp;   bufp+=step;
-					green += *bufp;   bufp+=step;
-					blue  += *bufp;   bufp+=step;
+				if( usb_IsCISDevice(dev)) {
+					red   += *bufp; bufp+=step;
+					green += *bufp; bufp+=step;
+					blue  += *bufp; bufp+=step;
 				} else {
 
 					red   += bufp[0];
@@ -945,31 +950,33 @@ static SANE_Bool cano_AdjustDarkShading( Plustek_Device *dev )
 			a_wDarkShading[i+stepW*2] = (u_short)val;
 		}
 
-		if(usb_HostSwap())
-			usb_Swap(a_wDarkShading, m_ScanParam.Size.dwPhyPixels * 2 * 3 );
-
 	} else {
+
 		step = m_ScanParam.Size.dwPhyPixels + 1;
-		
 		for( i=0; i<m_ScanParam.Size.dwPhyPixels; i++ ) {
 			
 			gray = 0;
 			bufp = ((u_short *)scanbuf)+i;
 
-			for( j=0; j<m_ScanParam.Size.dwPhyLines; j++) {
-				gray += *bufp;   bufp+=step;
+			for( j=0; j < m_ScanParam.Size.dwPhyLines; j++ ) {
+				gray += *bufp;
+				bufp += step;
 			}
 			a_wDarkShading[i]= gray/j + param->swOffset[0];
 		}
-		if(usb_HostSwap())
-			usb_Swap(a_wDarkShading, m_ScanParam.Size.dwPhyPixels * 2 );
 			
+		memcpy( a_wDarkShading + m_ScanParam.Size.dwPhyPixels,
+		        a_wDarkShading, m_ScanParam.Size.dwPhyPixels * 2);
 		memcpy( a_wDarkShading + m_ScanParam.Size.dwPhyPixels * 2,
 		        a_wDarkShading, m_ScanParam.Size.dwPhyPixels * 2);
-		memcpy( a_wDarkShading + m_ScanParam.Size.dwPhyPixels * 4,
-		        a_wDarkShading, m_ScanParam.Size.dwPhyPixels * 2);
 	}
 
+	if(usb_HostSwap())
+		usb_Swap(a_wDarkShading, m_ScanParam.Size.dwPhyPixels * 2 * 3 );
+
+	usb_line_statistics( "Dark", a_wDarkShading, m_ScanParam.Size.dwPhyPixels,
+	                     scan->sParam.bDataType == SCANDATATYPE_Color?1:0);
+
 	DBG( _DBG_INFO, "cano_AdjustDarkShading() done\n" );
 	return SANE_TRUE;
 }
@@ -978,13 +985,12 @@ static SANE_Bool cano_AdjustDarkShading( Plustek_Device *dev )
  * fine calibration part 2 - read the white calibration area and calculate
  * the gain coefficient for each pixel
  */
-static SANE_Bool cano_AdjustWhiteShading( Plustek_Device *dev )
+static SANE_Bool
+cano_AdjustWhiteShading( Plustek_Device *dev, u_short cal_dpi )
 {
 	char         tmp[40];
 	ScanParam   *param   = &dev->scanning.sParam;
 	ScanDef     *scan    = &dev->scanning;
-	DCapsDef    *scaps   = &dev->usbDev.Caps;
-	HWDef       *hw      = &dev->usbDev.HwSetting;
 	u_char      *scanbuf = scan->pScanBuffer;
 	u_short     *bufp;
 	unsigned int i, j;
@@ -995,27 +1001,8 @@ static SANE_Bool cano_AdjustWhiteShading( Plustek_Device *dev )
 	if( usb_IsEscPressed())
 		return SANE_FALSE;
 
-	m_ScanParam = scan->sParam;
-#if 0
-	if( m_ScanParam.PhyDpi.x > 75)
-		m_ScanParam.Size.dwLines = 64;
-	else
-#endif
-		m_ScanParam.Size.dwLines = 32;
-
-	m_ScanParam.Origin.y  = 0;
-	m_ScanParam.bBitDepth = 16;
-
-	m_ScanParam.UserDpi.y    = scaps->OpticDpi.y;
-	m_ScanParam.Size.dwBytes = m_ScanParam.Size.dwPixels * 2;
-
-	if( hw->bReg_0x26 & _ONE_CH_COLOR &&
-		m_ScanParam.bDataType == SCANDATATYPE_Color ) {
-		m_ScanParam.Size.dwBytes *= 3;
-	}
-
+	usb_PrepareFineCal( dev, &m_ScanParam, cal_dpi );
 	m_ScanParam.bCalibration = PARAM_WhiteShading;
-	m_ScanParam.dMCLK        = dMCLK;
 
 	sprintf( tmp, "fine-white.raw" );
 	DBG( _DBG_INFO2, "FINE WHITE Calibration Strip: %s\n", tmp );
@@ -1026,15 +1013,15 @@ static SANE_Bool cano_AdjustWhiteShading( Plustek_Device *dev )
 	dumpPicInit( &m_ScanParam, tmp );
 
 	if( usb_SetScanParameters( dev, &m_ScanParam ) &&
-		usb_ScanBegin( dev, SANE_FALSE ) &&
-		usb_ScanReadImage( dev, scanbuf, m_ScanParam.Size.dwTotalBytes)) {
+	    usb_ScanBegin( dev, SANE_FALSE ) &&
+	    usb_ScanReadImage( dev, scanbuf, m_ScanParam.Size.dwTotalBytes)) {
 
 		dumpPic( tmp, scanbuf, m_ScanParam.Size.dwTotalBytes );
 
 		if(usb_HostSwap())
 			usb_Swap((u_short *)scanbuf, m_ScanParam.Size.dwTotalBytes);
 
-		if (!usb_ScanEnd( dev )){
+		if (!usb_ScanEnd( dev )) {
 			DBG( _DBG_ERROR, "cano_AdjustWhiteShading() failed\n" );
 			return SANE_FALSE;
 		}
@@ -1047,7 +1034,7 @@ static SANE_Bool cano_AdjustWhiteShading( Plustek_Device *dev )
 	if( scan->sParam.bDataType == SCANDATATYPE_Color ) {
 
 		stepW = m_ScanParam.Size.dwPhyPixels;
-		if( hw->bReg_0x26 & _ONE_CH_COLOR ) 
+		if( usb_IsCISDevice(dev))
 			step = m_ScanParam.Size.dwPhyPixels + 1;
 		else
 			step = (m_ScanParam.Size.dwPhyPixels*3) + 1;
@@ -1057,39 +1044,35 @@ static SANE_Bool cano_AdjustWhiteShading( Plustek_Device *dev )
 			red   = 0;
 			green = 0;
 			blue  = 0;
-			if( hw->bReg_0x26 & _ONE_CH_COLOR )
+			if( usb_IsCISDevice(dev))
 				bufp = ((u_short *)scanbuf)+i;
 			else
 				bufp = ((u_short *)scanbuf)+(i*3);
 
 			for( j=0; j<m_ScanParam.Size.dwPhyLines; j++ ) {
 
-				if( hw->bReg_0x26 & _ONE_CH_COLOR ) {
-					red   += *bufp;  bufp+=step;
-					green += *bufp;  bufp+=step;
-					blue  += *bufp;  bufp+=step;
+				if( usb_IsCISDevice(dev)) {
+					red   += *bufp; bufp+=step;
+					green += *bufp; bufp+=step;
+					blue  += *bufp; bufp+=step;
 				} else {
-
 					red   += bufp[0];
 					green += bufp[1];
 					blue  += bufp[2];
-
-					bufp += step;
+					bufp  += step;
 				}
 			}
 
 			/* tweaked by the settings in swGain --> 1000/swGain[r,g,b] */
-			red   = (65535.*1000./param->swGain[0]) * 16384.*m_ScanParam.Size.dwPhyLines/red;
-			green = (65535.*1000./param->swGain[1]) * 16384.*m_ScanParam.Size.dwPhyLines/green;
-			blue  = (65535.*1000./param->swGain[2]) * 16384.*m_ScanParam.Size.dwPhyLines/blue;
+			red   = (65535.*1000./(double)param->swGain[0]) * 16384.*j/red;
+			green = (65535.*1000./(double)param->swGain[1]) * 16384.*j/green;
+			blue  = (65535.*1000./(double)param->swGain[2]) * 16384.*j/blue;
 
 			a_wWhiteShading[i]         = (red   > 65535 ? 65535:red  );
 			a_wWhiteShading[i+stepW]   = (green > 65535 ? 65535:green);
 			a_wWhiteShading[i+stepW*2] = (blue  > 65535 ? 65535:blue );
-	    }
+		}
 
-		if(usb_HostSwap())
-			usb_Swap(a_wWhiteShading, m_ScanParam.Size.dwPhyPixels * 2 * 3 );
 	} else {
 
 		step = m_ScanParam.Size.dwPhyPixels + 1;
@@ -1098,30 +1081,40 @@ static SANE_Bool cano_AdjustWhiteShading( Plustek_Device *dev )
 			bufp = ((u_short *)scanbuf)+i;
 
 			for( j=0; j<m_ScanParam.Size.dwPhyLines; j++ ) {
-				gray += *bufp;   bufp+=step;
+				gray += *bufp;
+				bufp += step;
 			}
 
-			gray = (65535.*1000./param->swGain[0]) * 16384.*j/gray;
+			gray = (65535.*1000./(double)param->swGain[0]) * 16384.*j/gray;
 
 			a_wWhiteShading[i]= (gray > 65535 ? 65535:gray);
 		}
-		if(usb_HostSwap())
-			usb_Swap(a_wWhiteShading, m_ScanParam.Size.dwPhyPixels * 2 );
 
-		memcpy(a_wWhiteShading+ m_ScanParam.Size.dwPhyPixels * 2,
-		                    a_wWhiteShading, m_ScanParam.Size.dwPhyPixels * 2);
-		memcpy(a_wWhiteShading+ m_ScanParam.Size.dwPhyPixels * 4,
-		                    a_wWhiteShading, m_ScanParam.Size.dwPhyPixels * 2);
+		memcpy( a_wWhiteShading + m_ScanParam.Size.dwPhyPixels,
+		        a_wWhiteShading, m_ScanParam.Size.dwPhyPixels * 2);
+		memcpy( a_wWhiteShading + m_ScanParam.Size.dwPhyPixels * 2,
+		        a_wWhiteShading, m_ScanParam.Size.dwPhyPixels * 2);
 	}
 
+	if(usb_HostSwap())
+		usb_Swap(a_wWhiteShading, m_ScanParam.Size.dwPhyPixels * 2 * 3 );
+
+	usb_SaveCalSetShading( dev, &m_ScanParam );
+
+	usb_line_statistics( "White", a_wWhiteShading, m_ScanParam.Size.dwPhyPixels,
+	                     scan->sParam.bDataType == SCANDATATYPE_Color?1:0);
+
 	DBG( _DBG_INFO, "cano_AdjustWhiteShading() done\n" );
 	return SANE_TRUE;
 }
 
 /** the entry function for the CIS calibration stuff.
  */
-static int cano_DoCalibration( Plustek_Device *dev )
+static int
+cano_DoCalibration( Plustek_Device *dev )
 {
+	u_short   dpi, idx, idx_end;
+	SANE_Bool skip_fine;
 	ScanDef  *scan  = &dev->scanning;
 	HWDef    *hw    = &dev->usbDev.HwSetting;
 	DCapsDef *scaps = &dev->usbDev.Caps;
@@ -1132,16 +1125,24 @@ static int cano_DoCalibration( Plustek_Device *dev )
 	DBG( _DBG_INFO, "cano_DoCalibration()\n" );
 
 	if( _IS_PLUSTEKMOTOR(hw->motorModel)){
-		DBG( _DBG_ERROR, "altCalibration can't work with this Plustek motor control setup\n" );
+		DBG( _DBG_ERROR, "altCalibration can't work with this "
+		                 "Plustek motor control setup\n" );
 		return SANE_FALSE; /* can't cal this  */
 	}
 
 	/* Don't allow calibration settings from the other driver to confuse our use of
 	 * a few of its functions.
 	 */
-	scaps->workaroundFlag &= ~_WAF_SKIP_WHITEFINE; 
-	scaps->workaroundFlag &= ~_WAF_SKIP_FINE; 
-	scaps->workaroundFlag &= ~_WAF_BYPASS_CALIBRATION; 
+	scaps->workaroundFlag &= ~_WAF_SKIP_WHITEFINE;
+	scaps->workaroundFlag &= ~_WAF_SKIP_FINE;
+	scaps->workaroundFlag &= ~_WAF_BYPASS_CALIBRATION;
+
+	if( !dev->adj.cacheCalData && !usb_IsSheetFedDevice(dev))
+		usb_SpeedTest( dev );
+
+	/* here we handle that warmup stuff for CCD devices */
+	if( !usb_AutoWarmup( dev ))
+		return SANE_FALSE;
 
 	/* Set the shading position to undefined */
 	strip_state = 0;
@@ -1150,14 +1151,18 @@ static int cano_DoCalibration( Plustek_Device *dev )
 	usb_SetMCLK( dev, &scan->sParam );
 
 	if( !scan->skipCoarseCalib ) {
+
+		if( !usb_Wait4ScanSample( dev ))
+			return SANE_FALSE;
+
 		DBG( _DBG_INFO2, "###### ADJUST LAMP (COARSE)#######\n" );
-		if( cano_PrepareToReadWhiteCal(dev))
+		if( cano_PrepareToReadWhiteCal(dev, SANE_TRUE))
 			return SANE_FALSE;
 
 		dev->usbDev.a_bRegs[0x45] &= ~0x10;
 		if( !cano_AdjustLightsource(dev)) {
 			DBG( _DBG_ERROR, "Coarse Calibration failed!!!\n" );
-			return _E_INTERNAL;
+			return SANE_FALSE;
 		}
 
 		DBG( _DBG_INFO2, "###### ADJUST OFFSET (COARSE) ####\n" );
@@ -1166,46 +1171,102 @@ static int cano_DoCalibration( Plustek_Device *dev )
 		
 		if( !cano_AdjustOffset(dev)) {
 			DBG( _DBG_ERROR, "Coarse Calibration failed!!!\n" );
-			return _E_INTERNAL;
+			return SANE_FALSE;
 		}
 
 		DBG( _DBG_INFO2, "###### ADJUST GAIN (COARSE)#######\n" );
-		if(cano_PrepareToReadWhiteCal(dev))
+		if(cano_PrepareToReadWhiteCal(dev, SANE_FALSE))
 			return SANE_FALSE;
 		
 		if( !cano_AdjustGain(dev)) {
 			DBG( _DBG_ERROR, "Coarse Calibration failed!!!\n" );
-			return _E_INTERNAL;
+			return SANE_FALSE;
 		}
 	} else {
 		strip_state = 1;
 		DBG( _DBG_INFO2, "###### COARSE calibration skipped #######\n" );
 	}
 
-	DBG( _DBG_INFO2, "###### ADJUST DARK (FINE) ########\n" );
-	if(cano_PrepareToReadBlackCal(dev))
-		return SANE_FALSE;
+	skip_fine = SANE_FALSE;
+	idx_end   = 2;
+	if( dev->adj.cacheCalData || usb_IsSheetFedDevice(dev)) {
 
-	dev->usbDev.a_bRegs[0x45] |= 0x10;
-	if( !cano_AdjustDarkShading(dev)) {
-		DBG( _DBG_ERROR, "Fine Calibration failed!!!\n" );
-		return _E_INTERNAL;
+		skip_fine = usb_FineShadingFromFile(dev);
+
+		/* we recalibrate in any case ! */
+		if( usb_InCalibrationMode(dev)) {
+			skip_fine = SANE_FALSE;
+			idx_end   = DIVIDER+1;
+
+			/* did I say any case? */
+			if (scan->sParam.bBitDepth != 8) {
+				skip_fine = SANE_TRUE;
+				DBG( _DBG_INFO2, "No fine calibration for non-8bit modes!\n" );
+			}
+
+		} else if( usb_IsSheetFedDevice(dev)) {
+
+			/* we only to the calibration upon request !*/
+			if( !skip_fine ) {
+				DBG( _DBG_INFO2, "SHEET-FED device, skip fine calibration!\n" );
+				skip_fine = SANE_TRUE;
+				scaps->workaroundFlag |= _WAF_BYPASS_CALIBRATION;
+			}
+		}
 	}
 
-	DBG( _DBG_INFO2, "###### ADJUST WHITE (FINE) #######\n" );
-	if(cano_PrepareToReadWhiteCal(dev))
-		return SANE_FALSE;
+	if( !skip_fine ) {
 
-	if(!usb_ModuleToHome( dev, SANE_TRUE ))
-		return SANE_FALSE;
+		for( idx = 1; idx < idx_end; idx++ ) {
 
-	if( !usb_ModuleMove(dev, MOVE_Forward,
-			(u_long)dev->usbDev.pSource->ShadingOriginY)) {
-		return _E_INTERNAL;
-	}
-	if( !cano_AdjustWhiteShading(dev)) {
-		DBG( _DBG_ERROR, "Fine Calibration failed!!!\n" );
-		return _E_INTERNAL;
+			dpi = 0;
+			if( usb_InCalibrationMode(dev))
+				dpi = usb_get_res( scaps->OpticDpi.x, idx );
+
+			DBG( _DBG_INFO2, "###### ADJUST DARK (FINE) ########\n" );
+			if(cano_PrepareToReadBlackCal(dev))
+				return SANE_FALSE;
+
+			dev->usbDev.a_bRegs[0x45] |= 0x10;
+			if( !cano_AdjustDarkShading(dev, dpi)) {
+				DBG( _DBG_ERROR, "Fine Calibration failed!!!\n" );
+				return SANE_FALSE;
+			}
+
+			DBG( _DBG_INFO2, "###### ADJUST WHITE (FINE) #######\n" );
+			if(cano_PrepareToReadWhiteCal(dev, SANE_FALSE))
+				return SANE_FALSE;
+
+			if( !usb_IsSheetFedDevice(dev)) {
+				if(!usb_ModuleToHome( dev, SANE_TRUE ))
+					return SANE_FALSE;
+
+				if( !usb_ModuleMove(dev, MOVE_Forward,
+					(u_long)dev->usbDev.pSource->ShadingOriginY)) {
+					return SANE_FALSE;
+				}
+			}
+			if( !cano_AdjustWhiteShading(dev, dpi)) {
+				DBG( _DBG_ERROR, "Fine Calibration failed!!!\n" );
+				return SANE_FALSE;
+			}
+
+			/* force to go back */
+			strip_state = 0;
+		}
+	} else {
+		DBG( _DBG_INFO2, "###### FINE calibration skipped #######\n" );
+
+		dev->usbDev.a_bRegs[0x45] |= 0x10;
+		strip_state = 2;
+
+		m_ScanParam = scan->sParam;
+		usb_GetPhyPixels( dev, &m_ScanParam );
+
+		usb_line_statistics( "Dark", a_wDarkShading, m_ScanParam.Size.dwPhyPixels,
+		                      m_ScanParam.bDataType == SCANDATATYPE_Color?1:0);
+		usb_line_statistics( "White", a_wWhiteShading, m_ScanParam.Size.dwPhyPixels,
+		                      m_ScanParam.bDataType == SCANDATATYPE_Color?1:0);
 	}
 
 	/* Lamp on if it's not */
@@ -1214,7 +1275,8 @@ static int cano_DoCalibration( Plustek_Device *dev )
 
 	/* home the sensor after calibration
 	 */
-	usb_ModuleToHome( dev, SANE_TRUE );
+	if( !usb_IsSheetFedDevice(dev))
+		usb_ModuleToHome( dev, SANE_TRUE );
 	scan->fCalibrated = SANE_TRUE;
 	
 	DBG( _DBG_INFO, "cano_DoCalibration() done\n" );
@@ -1224,12 +1286,12 @@ static int cano_DoCalibration( Plustek_Device *dev )
 	DBG( _DBG_INFO, "REG[0x3c] = %u\n", dev->usbDev.a_bRegs[0x3c] );
 	DBG( _DBG_INFO, "REG[0x3d] = %u\n", dev->usbDev.a_bRegs[0x3d] );
 	DBG( _DBG_INFO, "Static Offset:\n" );
-	DBG( _DBG_INFO, "REG[0x38] = %u\n", dev->usbDev.a_bRegs[0x38] );
-	DBG( _DBG_INFO, "REG[0x39] = %u\n", dev->usbDev.a_bRegs[0x39] );
-	DBG( _DBG_INFO, "REG[0x3a] = %u\n", dev->usbDev.a_bRegs[0x3a] );
+	DBG( _DBG_INFO, "REG[0x38] = %i\n", dev->usbDev.a_bRegs[0x38] );
+	DBG( _DBG_INFO, "REG[0x39] = %i\n", dev->usbDev.a_bRegs[0x39] );
+	DBG( _DBG_INFO, "REG[0x3a] = %i\n", dev->usbDev.a_bRegs[0x3a] );
 	DBG( _DBG_INFO, "-------------------------\n" );
 
 	return SANE_TRUE;
 }
 
-/* END PLUSTEK-USBCAL.C_.....................................................*/
+/* END PLUSTEK-USBCAL.C .....................................................*/
