@@ -7,7 +7,7 @@
  *  @brief Scanning...
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2005 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2006 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.40 - starting version of the USB support
@@ -27,6 +27,8 @@
  * - 0.49 - a_bRegs is now part of the device structure
  *        - using now PhyDpi.y as selector for the motor MCLK range
  *        - changed usb_MapDownload prototype
+ * - 0.50 - cleanup
+ * - 0.51 - added usb_get_res() and usb_GetPhyPixels()
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -69,6 +71,8 @@
  * <hr>
  */
 
+#define DIVIDER 8
+
 /** array used to get motor-settings and mclk-settings
  */
 static int dpi_ranges[] = { 75,100,150,200,300,400,600,800,1200,2400 };
@@ -79,7 +83,7 @@ static SANE_Bool  m_fAutoPark;
 static SANE_Bool  m_fFirst;
 static double     m_dHDPIDivider;
 static double     m_dMCLKDivider;
-static pScanParam m_pParam;
+static ScanParam *m_pParam;
 static u_char     m_bLineRateColor;
 static u_char     m_bCM;
 static u_char	  m_bIntTimeAdjust;
@@ -99,11 +103,11 @@ static SANE_Bool usb_DownloadShadingData( Plustek_Device*, u_char );
  * @param val2 - second parameter
  * @return val1 if val1 < val2, else val1
  */
-static u_long usb_min( u_long val1, u_long val2 )
+static u_long
+usb_min( u_long val1, u_long val2 )
 {
 	if( val1 > val2 )
 		return val2;
-
 	return val1;
 }
 
@@ -112,14 +116,30 @@ static u_long usb_min( u_long val1, u_long val2 )
  * @param val2 - second parameter
  * @return val1 if val1 > val2, else val2
  */
-static u_long usb_max( u_long val1, u_long val2 )
+static u_long
+usb_max( u_long val1, u_long val2 )
 {
 	if( val1 > val2 )
 		return val1;
-
 	return val2;
 }
 
+/** function to get the possible resolution for a specific base resolution,
+ * according to the dividers a LM983x offers.
+ * @param base - scanners optical resolution
+ * @param idx  - which divider to use
+ */
+static u_short
+usb_get_res( u_short base, u_short idx )
+{
+	double div_list[DIVIDER] = { 12.0, 8.0, 6.0, 4.0, 3.0, 2.0, 1.5, 1.0 };
+
+	if( idx == 0 || idx > DIVIDER )
+		return 0;
+
+	return (u_short)((double)base/div_list[idx-1]);
+}
+
 /** Set the horizontal DPI divider.
  * Affected registers:<br>
  * 0x09 - Horizontal DPI divider HDPI_DIV<br>
@@ -129,16 +149,16 @@ static u_long usb_max( u_long val1, u_long val2 )
  * @param xdpi - user specified horizontal resolution
  * @return - the function returns the "normalized" horizontal resolution.
  */
-static u_short usb_SetAsicDpiX( Plustek_Device *dev, u_short xdpi )
+static u_short
+usb_SetAsicDpiX( Plustek_Device *dev, u_short xdpi )
 {
 	u_short   res;
-    pScanDef  scanning = &dev->scanning;
-	pDCapsDef scaps    = &dev->usbDev.Caps;
+	ScanDef  *scanning = &dev->scanning;
+	DCapsDef *scaps    = &dev->usbDev.Caps;
 	u_char   *regs     = dev->usbDev.a_bRegs;
 
-	/*
-     * limit xdpi to lower value for certain devices...
-     */
+	/* limit xdpi to lower value for certain devices...
+	 */
 	if( scaps->OpticDpi.x == 1200 &&
 		scanning->sParam.bDataType != SCANDATATYPE_Color &&
 		xdpi < 150 &&
@@ -198,7 +218,6 @@ static u_short usb_SetAsicDpiX( Plustek_Device *dev, u_short xdpi )
 	res = (u_short)((double)scaps->OpticDpi.x / m_dHDPIDivider);
 
 	DBG( _DBG_INFO2, "* XDPI=%u, HDPI=%.3f\n", res, m_dHDPIDivider );
-
 	return res;
 }
 
@@ -208,19 +227,20 @@ static u_short usb_SetAsicDpiX( Plustek_Device *dev, u_short xdpi )
  * @param ydpi - user specified vertical resolution
  * @return -
  */
-static u_short usb_SetAsicDpiY( Plustek_Device *dev, u_short ydpi )
+static u_short
+usb_SetAsicDpiY( Plustek_Device *dev, u_short ydpi )
 {
-    pScanDef  scanning = &dev->scanning;
-	pDCapsDef sCaps    = &dev->usbDev.Caps;
-	pHWDef    hw       = &dev->usbDev.HwSetting;
+	ScanDef  *scanning = &dev->scanning;
+	DCapsDef *sCaps    = &dev->usbDev.Caps;
+	HWDef    *hw       = &dev->usbDev.HwSetting;
 
-	u_short	wMinDpi, wDpi;
+	u_short wMinDpi, wDpi;
 
 	if(0 != sCaps->bSensorDistance )
 		wMinDpi = sCaps->OpticDpi.y / sCaps->bSensorDistance;
 	else
 		wMinDpi = 75;
-	
+
 	/* Here we might have to check against the MinDpi value ! */
 	wDpi = (ydpi + wMinDpi - 1) / wMinDpi * wMinDpi;
 
@@ -260,23 +280,22 @@ static u_short usb_SetAsicDpiY( Plustek_Device *dev, u_short ydpi )
  * @param pParam - pointer to the current scan parameters
  * @return - Nothing
  */
-static void usb_SetColorAndBits( Plustek_Device *dev, pScanParam pParam )
+static void
+usb_SetColorAndBits( Plustek_Device *dev, ScanParam *pParam )
 {
-	pHWDef    hw   = &dev->usbDev.HwSetting;
+	HWDef    *hw   = &dev->usbDev.HwSetting;
 	u_char   *regs = dev->usbDev.a_bRegs;
 
-	/*
-     * Set pixel packing, data mode and AFE operation
-     */
+	/* Set pixel packing, data mode and AFE operation */
 	switch( pParam->bDataType ) {
 		case SCANDATATYPE_Color:
 			m_bCM = 3;
 			regs[0x26] = hw->bReg_0x26 & 0x7;
 
 			/* if set to one channel color, we select the blue channel
-             * as input source, this is the default, but I don't know
-             * what happens, if we deselect this
-             */
+			 * as input source, this is the default, but I don't know
+			 * what happens, if we deselect this
+			 */
 			if( regs[0x26] & _ONE_CH_COLOR )
 				regs[0x26] |= (_BLUE_CH | 0x01);
 
@@ -295,7 +314,7 @@ static void usb_SetColorAndBits( Plustek_Device *dev, pScanParam pParam )
 			memcpy( &regs[0x0f], hw->bReg_0x0f_Mono, 10 );
 			break;
 	}
-			
+
 	regs[0x27] = hw->bReg_0x27;
 
 	if( pParam->bBitDepth > 8 ) {
@@ -306,6 +325,45 @@ static void usb_SetColorAndBits( Plustek_Device *dev, pScanParam pParam )
 	}
 }
 
+/**
+ * Data output from NS983X should be times of 2-byte and every line
+ * will append 2 status bytes
+ */
+static void
+usb_GetPhyPixels( Plustek_Device *dev, ScanParam *sp )
+{
+	sp->Size.dwValidPixels = sp->Size.dwPixels * sp->PhyDpi.x / sp->UserDpi.x;
+
+	if (sp->bBitDepth == 1) {
+
+		/* Pixels should be times of 16 */
+		sp->Size.dwPhyPixels =
+		            (sp->Size.dwValidPixels + 15UL) & 0xfffffff0UL;
+		sp->Size.dwPhyBytes = sp->Size.dwPhyPixels / 8UL + 2UL;
+
+	} else if (sp->bBitDepth == 8) {
+
+		/* Pixels should be times of 2 */
+		sp->Size.dwPhyPixels = (sp->Size.dwValidPixels + 1UL) & 0xfffffffeUL;
+		sp->Size.dwPhyBytes  = sp->Size.dwPhyPixels * sp->bChannels + 2UL;
+
+		/* need to be adjusted fo CIS devices in color mode */
+		if(usb_IsCISDevice( dev ) && (sp->bDataType == SCANDATATYPE_Color)) {
+			sp->Size.dwPhyBytes *= 3;
+		}
+	}
+	else /* sp->bBitDepth == 16 */
+	{
+		sp->Size.dwPhyPixels = sp->Size.dwValidPixels;
+		sp->Size.dwPhyBytes  = sp->Size.dwPhyPixels * 2 * sp->bChannels + 2UL;
+
+		/* need to be adjusted fo CIS devices in color mode */
+		if(usb_IsCISDevice( dev ) && (sp->bDataType == SCANDATATYPE_Color)) {
+			sp->Size.dwPhyBytes *= 3;
+		}
+	}
+}
+
 /** Calculate basic image settings like the number of physical bytes per line
  * etc...
  * Affected registers:<br>
@@ -318,84 +376,45 @@ static void usb_SetColorAndBits( Plustek_Device *dev, pScanParam pParam )
  * @param pParam - pointer to the current scan parameters
  * @return - Nothing
  */
-static void usb_GetScanRect( Plustek_Device *dev, pScanParam pParam )
+static void
+usb_GetScanRect( Plustek_Device *dev, ScanParam *sp )
 {
 	u_short   wDataPixelStart, wLineEnd;
 
-	pDCapsDef sCaps = &dev->usbDev.Caps;
-	pHWDef    hw    = &dev->usbDev.HwSetting;
+	DCapsDef *sCaps = &dev->usbDev.Caps;
+	HWDef    *hw    = &dev->usbDev.HwSetting;
 	u_char   *regs  = dev->usbDev.a_bRegs;
 
 	/* Convert pixels to physical dpi based */
-	pParam->Size.dwValidPixels = pParam->Size.dwPixels *
-	                             pParam->PhyDpi.x / pParam->UserDpi.x;
-
-/* HEINER: check ADF stuff... */
-#if 0
-	if(pParam->bCalibration != PARAM_Gain &&
-		pParam->bCalibration != PARAM_Offset && ScanInf.m_fADF)
-		wDataPixelStart = 2550 * sCaps->OpticDpi.x / 300UL -
-				(u_short)(m_dHDPIDivider * pParam->Size.dwValidPixels + 0.5);
-	else
-#endif
-		wDataPixelStart = (u_short)((u_long) pParam->Origin.x *
-		                                            sCaps->OpticDpi.x / 300UL);
-
-	/* Data output from NS983X should be times of 2-byte and every line
-     * will append 2 status bytes
-     */
-	if (pParam->bBitDepth == 1)
-	{
-		/* Pixels should be times of 16 */
-		pParam->Size.dwPhyPixels =
-							(pParam->Size.dwValidPixels + 15UL) & 0xfffffff0UL;
-		pParam->Size.dwPhyBytes = pParam->Size.dwPhyPixels / 8UL + 2UL;
-	}
-	else if (pParam->bBitDepth == 8)
-	{
-		/* Pixels should be times of 2 */
-		pParam->Size.dwPhyPixels =
-							(pParam->Size.dwValidPixels + 1UL) & 0xfffffffeUL;
-		pParam->Size.dwPhyBytes =
-							pParam->Size.dwPhyPixels * pParam->bChannels + 2UL;
-
-		if((hw->bReg_0x26 & _ONE_CH_COLOR) &&
-								(pParam->bDataType == SCANDATATYPE_Color)) {
-			pParam->Size.dwPhyBytes *= 3;
-		}
-	}
-	else /* pParam->bBitDepth == 16 */
-	{
-		pParam->Size.dwPhyPixels = pParam->Size.dwValidPixels;
-		pParam->Size.dwPhyBytes =
-						pParam->Size.dwPhyPixels * 2 * pParam->bChannels + 2UL;
-
-		if((hw->bReg_0x26 & _ONE_CH_COLOR) &&
-								(pParam->bDataType == SCANDATATYPE_Color)) {
-			pParam->Size.dwPhyBytes *= 3;
-		}
-	}
+	usb_GetPhyPixels( dev, sp );
 
 	/* Compute data start pixel */
-	wDataPixelStart = (u_short)((u_long)pParam->Origin.x *
-	                                                sCaps->OpticDpi.x / 300UL);
+#if 0
+/* HEINER: check ADF stuff... */
+	if(sp->bCalibration != PARAM_Gain &&
+		sp->bCalibration != PARAM_Offset && ScanInf.m_fADF)
+		wDataPixelStart = 2550 * sCaps->OpticDpi.x / 300UL -
+				(u_short)(m_dHDPIDivider * sp->Size.dwValidPixels + 0.5);
+	else
+#endif
+		wDataPixelStart = (u_short)((u_long)sp->Origin.x *
+		                                            sCaps->OpticDpi.x / 300UL);
 
 	/* during the calibration steps, we read the entire CCD data
 	 */
-	if((pParam->bCalibration != PARAM_Gain) &&
-	   (pParam->bCalibration != PARAM_Offset)) {
+	if((sp->bCalibration != PARAM_Gain)&&(sp->bCalibration != PARAM_Offset)) {
 /* HEINER: check ADF stuff... */
 #if 0
 		if(ScanInf.m_fADF) {
 			wDataPixelStart = 2550 * sCaps->OpticDpi.x / 300UL -
-				(u_short)(m_dHDPIDivider * pParam->Size.dwValidPixels + 0.5);
-		}				
+			    (u_short)(m_dHDPIDivider * sp->Size.dwValidPixels + 0.5);
+		}
 #endif
 		wDataPixelStart += hw->wActivePixelsStart;
 	}
 
 	wLineEnd = wDataPixelStart + (u_short)(m_dHDPIDivider *
-	                                           pParam->Size.dwPhyPixels + 0.5);
+	                                               sp->Size.dwPhyPixels + 0.5);
 
 	DBG( _DBG_INFO2, "* DataPixelStart=%u, LineEnd=%u\n",
 	                                               wDataPixelStart, wLineEnd );
@@ -419,79 +438,81 @@ static void usb_GetScanRect( Plustek_Device *dev, pScanParam pParam )
 	                                 wLineEnd-wDataPixelStart, m_dHDPIDivider);
 
 	/* Y origin */
-	if( pParam->bCalibration == PARAM_Scan ) {
+	if( sp->bCalibration == PARAM_Scan ) {
 
 		if( hw->motorModel == MODEL_Tokyo600 ) {
 
-			if(pParam->PhyDpi.x <= 75)
-				pParam->Origin.y += 20;
-			else if(pParam->PhyDpi.x <= 100)
+			if(sp->PhyDpi.x <= 75)
+				sp->Origin.y += 20;
+			else if(sp->PhyDpi.x <= 100)
 			{
-				if (pParam->bDataType == SCANDATATYPE_Color)
-					pParam->Origin.y += 0;
+				if (sp->bDataType == SCANDATATYPE_Color)
+					sp->Origin.y += 0;
 				else
-					pParam->Origin.y -= 6;
+					sp->Origin.y -= 6;
 			}
-			else if(pParam->PhyDpi.x <= 150)
+			else if(sp->PhyDpi.x <= 150)
 			{
-				if (pParam->bDataType == SCANDATATYPE_Color)
-					pParam->Origin.y -= 0;
+				if (sp->bDataType == SCANDATATYPE_Color)
+					sp->Origin.y -= 0;
 			}
-			else if(pParam->PhyDpi.x <= 200)
+			else if(sp->PhyDpi.x <= 200)
 			{
-				if (pParam->bDataType == SCANDATATYPE_Color)
-					pParam->Origin.y -= 10;
+				if (sp->bDataType == SCANDATATYPE_Color)
+					sp->Origin.y -= 10;
 				else
-					pParam->Origin.y -= 4;
+					sp->Origin.y -= 4;
 			}
-			else if(pParam->PhyDpi.x <= 300)
+			else if(sp->PhyDpi.x <= 300)
 			{
-				if (pParam->bDataType == SCANDATATYPE_Color)
-					pParam->Origin.y += 16;
+				if (sp->bDataType == SCANDATATYPE_Color)
+					sp->Origin.y += 16;
 				else
-					pParam->Origin.y -= 18;
+					sp->Origin.y -= 18;
 			}
-			else if(pParam->PhyDpi.x <= 400)
+			else if(sp->PhyDpi.x <= 400)
 			{
-				if (pParam->bDataType == SCANDATATYPE_Color)
-					pParam->Origin.y += 15;
-				else if(pParam->bDataType == SCANDATATYPE_BW)
-					pParam->Origin.y += 4;
+				if (sp->bDataType == SCANDATATYPE_Color)
+					sp->Origin.y += 15;
+				else if(sp->bDataType == SCANDATATYPE_BW)
+					sp->Origin.y += 4;
 			}
-			else /* if(pParam->PhyDpi.x <= 600) */
+			else /* if(sp->PhyDpi.x <= 600) */
 			{
-				if (pParam->bDataType == SCANDATATYPE_Gray)
-					pParam->Origin.y += 4;
+				if (sp->bDataType == SCANDATATYPE_Gray)
+					sp->Origin.y += 4;
 			}
 		}
 
 		/* Add gray mode offset (Green offset, we assume the CCD are
 		 * always be RGB or BGR order).
 		 */
-		if (pParam->bDataType != SCANDATATYPE_Color)
-			pParam->Origin.y += (u_long)(300UL *
-							       sCaps->bSensorDistance / sCaps->OpticDpi.y);
+		if (sp->bDataType != SCANDATATYPE_Color)
+			sp->Origin.y += (u_long)(300UL *
+			           sCaps->bSensorDistance / sCaps->OpticDpi.y);
 	}
 
-	pParam->Origin.y=(u_short)((u_long)pParam->Origin.y * hw->wMotorDpi/300UL);
+	sp->Origin.y=(u_short)((u_long)sp->Origin.y * hw->wMotorDpi/300UL);
 
 	/* Something wrong, but I can not find it. */
 	if( hw->motorModel == MODEL_HuaLien && sCaps->OpticDpi.x == 600)
-		pParam->Origin.y = pParam->Origin.y * 297 / 298;
+		sp->Origin.y = sp->Origin.y * 297 / 298;
 
-	DBG(_DBG_INFO2,"* Full Steps to Skip at Start = 0x%04x\n",pParam->Origin.y);
+	DBG(_DBG_INFO2,"* Full Steps to Skip at Start = 0x%04x\n",
+	                sp->Origin.y);
 
-	regs[0x4a] = _HIBYTE( pParam->Origin.y );
-	regs[0x4b] = _LOBYTE( pParam->Origin.y );
+	regs[0x4a] = _HIBYTE( sp->Origin.y );
+	regs[0x4b] = _LOBYTE( sp->Origin.y );
 }
 
 /** preset scan stepsize and fastfeed stepsize
  */
-static void usb_PresetStepSize( Plustek_Device *dev, pScanParam pParam )
+static void
+usb_PresetStepSize( Plustek_Device *dev, ScanParam *pParam )
 {
 	u_short ssize;
 	double  mclkdiv = pParam->dMCLK;
-	pHWDef  hw      = &dev->usbDev.HwSetting;
+	HWDef  *hw      = &dev->usbDev.HwSetting;
 	u_char *regs    = dev->usbDev.a_bRegs;
 
 	ssize = (u_short)((double)CRYSTAL_FREQ / ( mclkdiv * 8.0 *
@@ -507,47 +528,48 @@ static void usb_PresetStepSize( Plustek_Device *dev, pScanParam pParam )
 
 /** calculate default phase difference DPD
  */
-static void usb_GetDPD( Plustek_Device *dev  )
+static void
+usb_GetDPD( Plustek_Device *dev  )
 {
 	int    qtcnt;	/* quarter speed count count reg 51 b2..3 */
 	int    hfcnt;	/* half speed count reg 51 b0..1          */
-	int    strev;	/* steps to reverse reg 50                */
-	int    dpd;	    /* calculated dpd reg 52:53               */
-	int    st;		/* step size reg 46:47                    */
+	int    strev;   /* steps to reverse reg 50                */
+	int    dpd;     /* calculated dpd reg 52:53               */
+	int    st;      /* step size reg 46:47                    */
 
-	pHWDef  hw    = &dev->usbDev.HwSetting;
+	HWDef  *hw    = &dev->usbDev.HwSetting;
 	u_char *regs  = dev->usbDev.a_bRegs;
 
-	qtcnt = (regs[0x51] & 0x30) >> 4;	/* quarter speed count */
-	hfcnt = (regs[0x51] & 0xc0) >> 6;	/* half speed count    */
+	qtcnt = (regs[0x51] & 0x30) >> 4;  /* quarter speed count */
+	hfcnt = (regs[0x51] & 0xc0) >> 6;  /* half speed count    */
 
 	if( _LM9831 == hw->chip )
-		strev = regs[0x50] & 0x3f;		/* steps to reverse */
+		strev = regs[0x50] & 0x3f;    /* steps to reverse */
 	else /* LM9832/3 */
 	{
 		if (qtcnt == 3)
 			qtcnt = 8;
 		if (hfcnt == 3)
 			hfcnt = 8;
-		strev = regs[0x50]; /* steps to reverse */
+		strev = regs[0x50];           /* steps to reverse */
 	}
 
-	st = regs[0x46] * 256 + regs[0x47]; /* scan step size */
+	st = regs[0x46] * 256 + regs[0x47];     /* scan step size */
 
 	if (m_wLineLength == 0)
 		dpd = 0;
 	else
 	{
 		dpd = (((qtcnt * 4) + (hfcnt * 2) + strev) * 4 * st) %
-					 (m_wLineLength * m_bLineRateColor);
+		                            (m_wLineLength * m_bLineRateColor);
 		DBG( _DBG_INFO2, "* DPD =%u (0x%04x)\n", dpd, dpd );
 		dpd = m_wLineLength * m_bLineRateColor - dpd;
 	}
 
 	DBG( _DBG_INFO2, "* DPD =%u (0x%04x), step size=%u, steps2rev=%u\n",
-														dpd, dpd, st, strev);
+	                  dpd, dpd, st, strev);
 	DBG( _DBG_INFO2, "* llen=%u, lineRateColor=%u, qtcnt=%u, hfcnt=%u\n",
-						  m_wLineLength, m_bLineRateColor, qtcnt, hfcnt );
+	                  m_wLineLength, m_bLineRateColor, qtcnt, hfcnt );
 
 	regs[0x51] |= (u_char)((dpd >> 16) & 0x03);
 	regs[0x52] = (u_char)(dpd >> 8);
@@ -560,7 +582,8 @@ static void usb_GetDPD( Plustek_Device *dev  )
 
 /**
  */
-static int usb_GetMCLKDiv( Plustek_Device *dev )
+static int
+usb_GetMCLKDiv( Plustek_Device *dev )
 {
 	int     j, pixelbits, pixelsperline, r;
 	int     minmclk, maxmclk, mclkdiv;
@@ -574,7 +597,7 @@ static int usb_GetMCLKDiv( Plustek_Device *dev )
 	if ((regs[0x26] & 7) == 0) 
 		r = 24; /* pixel rate */
 
-	/*use high or low res min integration time */
+	/* use high or low res min integration time */
 	min_int_time = ((regs[0x9]&7) > 2 ? hw->dMinIntegrationTimeLowres:
 	                                    hw->dMinIntegrationTimeHighres);
 
@@ -631,13 +654,14 @@ static int usb_GetMCLKDiv( Plustek_Device *dev )
  * at least we give the master clock divider and adjust the step size
  * and integration time (for 14/16 bit modes)
  */
-static double usb_GetMCLKDivider( Plustek_Device *dev, pScanParam pParam )
+static double
+usb_GetMCLKDivider( Plustek_Device *dev, ScanParam *pParam )
 {
 	double dMaxIntegrationTime;
 	double dMaxMCLKDivider;
 
-	pDCapsDef sCaps = &dev->usbDev.Caps;
-	pHWDef    hw    = &dev->usbDev.HwSetting;
+	DCapsDef *sCaps = &dev->usbDev.Caps;
+	HWDef    *hw    = &dev->usbDev.HwSetting;
 	u_char   *regs  = dev->usbDev.a_bRegs;
 
 	DBG( _DBG_INFO, "usb_GetMCLKDivider()\n" );
@@ -662,8 +686,8 @@ static double usb_GetMCLKDivider( Plustek_Device *dev, pScanParam pParam )
 		if( pParam->bBitDepth > 8 )	{
 
 			while( pParam->Size.dwPhyBytes >
-					(m_dMCLKDivider * m_bCM * m_wLineLength / 6 * 9 / 10) *
- 													  (1 + m_bIntTimeAdjust)) {
+			       (m_dMCLKDivider * m_bCM * m_wLineLength / 6 * 9 / 10) *
+ 			       (1 + m_bIntTimeAdjust)) {
 				m_bIntTimeAdjust++;
 			}	
 
@@ -694,7 +718,7 @@ static double usb_GetMCLKDivider( Plustek_Device *dev, pScanParam pParam )
 	if( m_bIntTimeAdjust != 0 ) {
 
 		m_wStepSize = (u_short)((u_long) m_wStepSize *
-									(m_bIntTimeAdjust + 1) / m_bIntTimeAdjust);
+		                    (m_bIntTimeAdjust + 1) / m_bIntTimeAdjust);
 		if( m_wStepSize < 2 )
 			m_wStepSize = 2;
 
@@ -707,11 +731,11 @@ static double usb_GetMCLKDivider( Plustek_Device *dev, pScanParam pParam )
 	}
 	
 	/* Compute maximum MCLK divider base on maximum integration time for
-     * high lamp PWM, use equation 4
-     */
+	 * high lamp PWM, use equation 4
+	 */
 	dMaxIntegrationTime = hw->dIntegrationTimeHighLamp;
 	dMaxMCLKDivider = (double)CRYSTAL_FREQ * dMaxIntegrationTime /
-					   (1000 * 8 * m_bCM * m_wLineLength);
+	                                    (1000 * 8 * m_bCM * m_wLineLength);
 
 	/* Determine lamp PWM setting */
 	if( m_dMCLKDivider > dMaxMCLKDivider ) {
@@ -733,19 +757,20 @@ static double usb_GetMCLKDivider( Plustek_Device *dev, pScanParam pParam )
 
 /** calculate the step size of each scan step
  */
-static void usb_GetStepSize( Plustek_Device *dev, pScanParam pParam )
+static void
+usb_GetStepSize( Plustek_Device *dev, ScanParam *pParam )
 {
-	pHWDef  hw   = &dev->usbDev.HwSetting;
+	HWDef  *hw  = &dev->usbDev.HwSetting;
 	u_char *regs = dev->usbDev.a_bRegs;
 
 	/* Compute step size using equation 1 */
 	if (m_bIntTimeAdjust != 0) {
 		m_wStepSize = (u_short)(((u_long) pParam->PhyDpi.y * m_wLineLength *
-								   m_bLineRateColor * (m_bIntTimeAdjust + 1)) /
-									 (4 * hw->wMotorDpi * m_bIntTimeAdjust));
+		              m_bLineRateColor * (m_bIntTimeAdjust + 1)) /
+		              (4 * hw->wMotorDpi * m_bIntTimeAdjust));
 	} else {
 		m_wStepSize = (u_short)(((u_long) pParam->PhyDpi.y * m_wLineLength *
-								   m_bLineRateColor) / (4 * hw->wMotorDpi));
+		               m_bLineRateColor) / (4 * hw->wMotorDpi));
 	}
 
 	if (m_wStepSize < 2)
@@ -762,49 +787,51 @@ static void usb_GetStepSize( Plustek_Device *dev, pScanParam pParam )
 
 /**
  */
-static void usb_GetLineLength( Plustek_Device *dev )
+static void
+usb_GetLineLength( Plustek_Device *dev, ScanParam *param )
 {
 /* [note]
- *	The ITA in this moment is always 0, it will be changed later when we
+ *  The ITA in this moment is always 0, it will be changed later when we
  *  calculate MCLK. This is very strange why this routine will not call
  *  again to get all new value after ITA was changed? If this routine
  *  never call again, maybe we remove all factor with ITA here.
  */
 	int tr;
-	int tpspd;	/* turbo/preview mode speed reg 0a b2..3                 */
-	int tpsel;	/* turbo/preview mode select reg 0a b0..1                */
-	int gbnd;	/* guardband duration reg 0e b4..7                       */
-	int dur;	/* pulse duration reg 0e b0..3                           */
-	int ntr;	/* number of tr pulses reg 0d b7                         */
-	int afeop;	/* scan mode, 0=pixel rate, 1=line rate,                 */
-				/* 4=1 channel mode a, 5=1 channel mode b, reg 26 b0..2  */
-	int ctmode;	/* CIS tr timing mode reg 19 b0..1                       */
-	int tp;		/* tpspd or 1 if tpsel=0                                 */
-	int b;		/* if ctmode=0, (ntr+1)*((2*gbnd)+dur+1), otherwise 1    */
-	int tradj;	/* ITA                                                   */
+	int tpspd;  /* turbo/preview mode speed reg 0a b2..3                 */
+	int tpsel;  /* turbo/preview mode select reg 0a b0..1                */
+	int gbnd;   /* guardband duration reg 0e b4..7                       */
+	int dur;    /* pulse duration reg 0e b0..3                           */
+	int ntr;    /* number of tr pulses reg 0d b7                         */
+	int afeop;  /* scan mode, 0=pixel rate, 1=line rate,                 */
+	            /* 4=1 channel mode a, 5=1 channel mode b, reg 26 b0..2  */
+	int ctmode; /* CIS tr timing mode reg 19 b0..1                       */
+	int tp;     /* tpspd or 1 if tpsel=0                                 */
+	int b;      /* if ctmode=0, (ntr+1)*((2*gbnd)+dur+1), otherwise 1    */
+	int tradj;  /* ITA                                                   */
 	int en_tradj;
+	u_short      le;
+	HWDef       *hw    = &dev->usbDev.HwSetting;
+	ClkMotorDef *motor = usb_GetMotorSet( hw->motorModel );
+	u_char      *regs  = dev->usbDev.a_bRegs;
 
-	pHWDef  hw   = &dev->usbDev.HwSetting;
-	u_char *regs = dev->usbDev.a_bRegs;
+	tpspd = (regs[0x0a] & 0x0c) >> 2;       /* turbo/preview mode speed  */
+	tpsel = regs[0x0a] & 3;                 /* turbo/preview mode select */
 
-	tpspd = (regs[0x0a] & 0x0c) >> 2; /* turbo/preview mode speed  */
-	tpsel = regs[0x0a] & 3; 			 /* turbo/preview mode select */
+	gbnd = (regs[0x0e] & 0xf0) >> 4;        /* TR fi1 guardband duration */
+	dur = (regs[0x0e] & 0xf);               /* TR pulse duration         */
 
-	gbnd = (regs[0x0e] & 0xf0) >> 4;	 /* TR fi1 guardband duration */
-	dur = (regs[0x0e] & 0xf);		 /* TR pulse duration         */
+	ntr = regs[0x0d] / 128;                 /* number of tr pulses       */
 
-	ntr = regs[0x0d] / 128;			 /* number of tr pulses       */
+	afeop = regs[0x26] & 7;           /* afe op - 3 channel or 1 channel */
 
-	afeop = regs[0x26] & 7;			 /* afe op - 3 channel or 1 channel */
-
-	tradj = regs[0x19] & 0x7f;		 /* integration time adjust */
+	tradj = regs[0x19] & 0x7f;                /* integration time adjust */
 	en_tradj = (tradj) ? 1 : 0;
 
-	ctmode = (regs[0x0b] >> 3) & 3;	 /* cis tr timing mode */
+	ctmode = (regs[0x0b] >> 3) & 3;                /* cis tr timing mode */
 
 	m_bLineRateColor = 1;	
 	if (afeop == 1 || afeop == 5) /* if 3 channel line or 1 channel mode b */
-		m_bLineRateColor = 3;	
+		m_bLineRateColor = 3;
 
 	/* according to turbo/preview mode to set value */
 	if( tpsel == 0 ) {
@@ -823,8 +850,21 @@ static void usb_GetLineLength( Plustek_Device *dev )
 	}
 	if( ctmode == 2 )   /* CIS mode scanner */
 	    b = 3;
-	
-	tr = m_bLineRateColor * (hw->wLineEnd + tp * (b + 3 - ntr));
+
+	/* it might be necessary to tweak this value, to keep the
+	 * MCLK constant - see some sheet-fed devices
+	 */
+	le = hw->wLineEnd;
+	if (motor->dpi_thresh != 0) {
+		if( param->PhyDpi.y <= motor->dpi_thresh) {
+			le = motor->lineend;
+			DBG( _DBG_INFO2, "* Adjusting lineend: %u\n", le);
+		}
+		regs[0x20] = _HIBYTE( le );
+		regs[0x21] = _LOBYTE( le );
+	}
+
+	tr = m_bLineRateColor * (le + tp * (b + 3 - ntr));
 
 	if( tradj == 0 ) {
 		if( ctmode == 0 )
@@ -837,7 +877,7 @@ static void usb_GetLineLength( Plustek_Device *dev )
 		if( afeop != 0 ) {
 		
 			le_phi      = (tradj + 1) / 2 + 1 + 6;
-			num_byteclk = ((le_phi + 8 * hw->wLineEnd + 8 * b + 4) /
+			num_byteclk = ((le_phi + 8 * le + 8 * b + 4) /
 						   (8 * tradj)) + 1;
 			num_mclkf   = 8 * tradj * num_byteclk;
 			tr_fast_pix = num_byteclk;
@@ -846,14 +886,14 @@ static void usb_GetLineLength( Plustek_Device *dev )
 		else /* 3 channel pixel rate color */
 		{
 			le_phi      = (tradj + 1) / 2 + 1 + 10 + 12;
-			num_byteclk = ((le_phi + 3 * 8 * hw->wLineEnd + 3 * 8 * b + 3 * 4) /
+			num_byteclk = ((le_phi + 3 * 8 * le + 3 * 8 * b + 3 * 4) /
 						   (3 * 8 * tradj)) + 1;
 			num_mclkf   = 3 * 8 * tradj * num_byteclk;
 			tr_fast_pix = num_byteclk;
 			extra_pix   = (num_mclkf - le_phi) % (3 * 8);
 		}
 		
-		tr = b + hw->wLineEnd + 4 + tr_fast_pix;
+		tr = b + le + 4 + tr_fast_pix;
 		if (extra_pix == 0)
 			tr++;
 		tr *= m_bLineRateColor;
@@ -867,23 +907,24 @@ static void usb_GetLineLength( Plustek_Device *dev )
 /** usb_GetMotorParam
  * registers 0x56, 0x57
  */
-static void usb_GetMotorParam( Plustek_Device *dev, pScanParam pParam )
+static void
+usb_GetMotorParam( Plustek_Device *dev, ScanParam *pParam )
 {
 	int          idx, i;
-	pClkMotorDef clk;
-	pMDef        md;
-	pDCapsDef    sCaps = &dev->usbDev.Caps;
-	pHWDef       hw    = &dev->usbDev.HwSetting;
+	ClkMotorDef *clk;
+	MDef        *md;
+	DCapsDef    *sCaps = &dev->usbDev.Caps;
+	HWDef       *hw    = &dev->usbDev.HwSetting;
 	u_char      *regs  = dev->usbDev.a_bRegs;
 
-	if( !_IS_PLUSTEKMOTOR(hw->motorModel)) {
-	
+	if (!_IS_PLUSTEKMOTOR(hw->motorModel)) {
+
 		clk = usb_GetMotorSet( hw->motorModel );
 		md  = clk->motor_sets;
- 		idx = 0;
+		idx = 0;
 		for( i = 0; i < _MAX_CLK; i++ ) {
 			if( pParam->PhyDpi.y <= dpi_ranges[i] )
-  				break;
+				break;
 			idx++;
 		}
 		if( idx >= _MAX_CLK )
@@ -904,44 +945,44 @@ static void usb_GetMotorParam( Plustek_Device *dev, pScanParam pParam )
 				regs[0x44] = md[idx].scan_lines_per_line;
 
 				DBG( _DBG_INFO2, "* Line Skipping : 0x43=0x%02x, 0x44=0x%02x\n",
-	  											regs[0x43], regs[0x44] );
+				                  regs[0x43], regs[0x44] );
 			}
 		}
+	} else {
 
-    } else {
-        if( sCaps->OpticDpi.x == 1200 ) {
+		if( sCaps->OpticDpi.x == 1200 ) {
 
-        	switch( hw->motorModel ) {
+			switch( hw->motorModel ) {
 
-        	case MODEL_HuaLien:
-        	case MODEL_KaoHsiung:
-        	default:
-        		if(pParam->PhyDpi.x <= 200)
-        		{
-        			regs[0x56] = 1;
-        			regs[0x57] = 48;	/* 63; */
-        		}
-        		else if(pParam->PhyDpi.x <= 300)
-        		{
-        			regs[0x56] = 2;	/* 8;  */
-        			regs[0x57] = 48;	/* 56; */
-        		}
-        		else if(pParam->PhyDpi.x <= 400)
-        		{
-        			regs[0x56] = 8;	
-        			regs[0x57] = 48;	
-        		}
-        		else if(pParam->PhyDpi.x <= 600)
-        		{
-        			regs[0x56] = 2;	/* 10; */
-        			regs[0x57] = 48;	/* 56; */
-        		}
-        		else /* pParam->PhyDpi.x == 1200) */
-        		{
-        			regs[0x56] = 1;	/* 8;  */
-        			regs[0x57] = 48;	/* 56; */
-        		}
-        		break;
+			case MODEL_HuaLien:
+			case MODEL_KaoHsiung:
+			default:
+				if(pParam->PhyDpi.x <= 200)
+				{
+					regs[0x56] = 1;
+					regs[0x57] = 48;  /* 63; */
+				}
+				else if(pParam->PhyDpi.x <= 300)
+				{
+					regs[0x56] = 2;   /* 8;  */
+					regs[0x57] = 48;  /* 56; */
+				}
+				else if(pParam->PhyDpi.x <= 400)
+				{
+					regs[0x56] = 8;	
+					regs[0x57] = 48;	
+				}
+				else if(pParam->PhyDpi.x <= 600)
+				{
+					regs[0x56] = 2;   /* 10; */
+					regs[0x57] = 48;  /* 56; */
+				}
+				else /* pParam->PhyDpi.x == 1200) */
+				{
+					regs[0x56] = 1;   /* 8;  */
+					regs[0x57] = 48;  /* 56; */
+				}
+				break;
         	}
         } else {
         	switch ( hw->motorModel ) {
@@ -1011,17 +1052,18 @@ static void usb_GetMotorParam( Plustek_Device *dev, pScanParam pParam )
         	}
         }
 	}
-	
+
 	DBG( _DBG_INFO2, "* MOTOR-Settings: PWM=0x%02x, PWM_DUTY=0x%02x\n",
 	                 regs[0x56], regs[0x57] );
 }
 
 /**
  */
-static void usb_GetPauseLimit( Plustek_Device *dev, pScanParam pParam )
+static void
+usb_GetPauseLimit( Plustek_Device *dev, ScanParam *pParam )
 {
 	int     coeffsize, scaler;
-	pHWDef  hw   = &dev->usbDev.HwSetting;
+	HWDef  *hw   = &dev->usbDev.HwSetting;
 	u_char *regs = dev->usbDev.a_bRegs;
 
 	scaler = 1;
@@ -1051,13 +1093,13 @@ static void usb_GetPauseLimit( Plustek_Device *dev, pScanParam pParam )
 	/* If not reversing, take into account the steps to reverse */
 	if( regs[0x50] == 0 )
 		m_dwPauseLimit -= ((regs[0x54] & 7) *
-							(pParam->Size.dwPhyBytes * scaler) + 1023) / 1024;
+		                  (pParam->Size.dwPhyBytes * scaler) + 1023) / 1024;
 
 	DBG( _DBG_INFO2, "* PL=%lu, coeffsize=%u, scaler=%u\n",
 	                  m_dwPauseLimit, coeffsize, scaler );
 
 	m_dwPauseLimit = usb_max( usb_min(m_dwPauseLimit,
-						(u_long)ceil(pParam->Size.dwTotalBytes / 1024.0)), 2);
+	                  (u_long)ceil(pParam->Size.dwTotalBytes / 1024.0)), 2);
 
 	regs[0x4e] = (u_char)floor((m_dwPauseLimit*512.0)/(2.0*hw->wDRAMSize));
 
@@ -1078,9 +1120,10 @@ static void usb_GetPauseLimit( Plustek_Device *dev, pScanParam pParam )
 
 /** usb_GetScanLinesAndSize
  */
-static void usb_GetScanLinesAndSize( Plustek_Device *dev, pScanParam pParam )
+static void
+usb_GetScanLinesAndSize( Plustek_Device *dev, ScanParam *pParam )
 {
-	pDCapsDef sCaps = &dev->usbDev.Caps;
+	DCapsDef *sCaps = &dev->usbDev.Caps;
 
 	pParam->Size.dwPhyLines = (u_long)ceil((double) pParam->Size.dwLines *
 	                                     pParam->PhyDpi.y / pParam->UserDpi.y);
@@ -1104,12 +1147,14 @@ static void usb_GetScanLinesAndSize( Plustek_Device *dev, pScanParam pParam )
 
 /** function to preset/reset the merlin registers
  */
-static SANE_Bool usb_SetScanParameters( Plustek_Device *dev, pScanParam pParam )
+static SANE_Bool
+usb_SetScanParameters( Plustek_Device *dev, ScanParam *pParam )
 {
 	static u_char reg8, reg38[6], reg48[2];
 
-	pScanParam pdParam = &dev->scanning.sParam;
-	pHWDef     hw      = &dev->usbDev.HwSetting;
+	ScanDef   *scan    = &dev->scanning;
+	ScanParam *pdParam = &dev->scanning.sParam;
+	HWDef     *hw      = &dev->usbDev.HwSetting;
 	u_char    *regs    = dev->usbDev.a_bRegs;
 
 	m_pParam = pParam;
@@ -1129,8 +1174,8 @@ static SANE_Bool usb_SetScanParameters( Plustek_Device *dev, pScanParam pParam )
 	}
 
 	/*
-     * calculate the basic settings...
-     */
+	 * calculate the basic settings...
+	 */
 	pParam->PhyDpi.x = usb_SetAsicDpiX( dev, pParam->UserDpi.x );
 	pParam->PhyDpi.y = usb_SetAsicDpiY( dev, pParam->UserDpi.y );
 
@@ -1172,7 +1217,7 @@ static SANE_Bool usb_SetScanParameters( Plustek_Device *dev, pScanParam pParam )
           pParam->bSource == SOURCE_ADF && dev->usbDev.fLastScanIsAdf )) {
 
 		DBG( _DBG_INFO2, "* Scan calculations...\n" );
-		usb_GetLineLength ( dev );
+		usb_GetLineLength ( dev, pParam );
 		usb_GetStepSize   ( dev, pParam );
 		usb_GetDPD        ( dev );
 		usb_GetMCLKDivider( dev, pParam );
@@ -1207,12 +1252,12 @@ static SANE_Bool usb_SetScanParameters( Plustek_Device *dev, pScanParam pParam )
 	usb_GetScanLinesAndSize( dev, pParam );
 	
 	/* Pause limit should be bounded by total bytes to read
-     * so that the chassis will not move too far.
-     */
+	 * so that the chassis will not move too far.
+	 */
 	usb_GetPauseLimit( dev, pParam );
 
 	/* For ADF .... */
-	if(pParam->bCalibration == PARAM_Scan && pParam->bSource == SOURCE_ADF)	{
+	if(pParam->bCalibration == PARAM_Scan && pParam->bSource == SOURCE_ADF) {
 
 		if( dev->usbDev.fLastScanIsAdf ) {
 
@@ -1248,16 +1293,21 @@ static SANE_Bool usb_SetScanParameters( Plustek_Device *dev, pScanParam pParam )
 	memset( &regs[0x03], 0, 3 );
 	memset( &regs[0x5f], 0, 0x7f-0x5f+1 );
 
-	if(pParam->bCalibration == PARAM_Scan && pParam->bSource != SOURCE_ADF) {
+	if( !usb_IsSheetFedDevice(dev)) {
+		/* we limit the possible scansteps to avoid, that the sensors bumps
+		 * against the scanbed - not necessary for sheet-fed devices
+		 */
+		if(pParam->bCalibration==PARAM_Scan && pParam->bSource!=SOURCE_ADF) {
 
-		u_short scansteps = (u_short)ceil((double)(pParam->Size.dwPhyLines + 10)*
-		                                     hw->wMotorDpi / pParam->PhyDpi.y);
-		DBG( _DBG_INFO, "* Scansteps=%u (%lu*%u/%u)\n", scansteps, 
-		                 pParam->Size.dwPhyLines, hw->wMotorDpi, pParam->PhyDpi.y );
-#if 1
-		regs[0x4c] = _HIBYTE(scansteps);
-		regs[0x4d] = _LOBYTE(scansteps);
-#endif
+			u_long  lines     = pParam->Size.dwPhyLines + scan->bLinesToSkip +
+			                                          scan->dwLinesDiscard + 5;
+			u_short scansteps = (u_short)ceil((double)lines*
+			                                 hw->wMotorDpi / pParam->PhyDpi.y);
+			DBG( _DBG_INFO, "* Scansteps=%u (%lu*%u/%u)\n", scansteps,  lines,
+			                hw->wMotorDpi, pParam->PhyDpi.y );
+			regs[0x4c] = _HIBYTE(scansteps);
+			regs[0x4d] = _LOBYTE(scansteps);
+		}
 	}
 
 	/* set the merlin registers */
@@ -1275,16 +1325,20 @@ static SANE_Bool usb_SetScanParameters( Plustek_Device *dev, pScanParam pParam )
 
 /**
  */
-static SANE_Bool usb_ScanBegin( Plustek_Device *dev, SANE_Bool auto_park )
+static SANE_Bool
+usb_ScanBegin( Plustek_Device *dev, SANE_Bool auto_park )
 {
 	u_char  value;
 	u_short inches;
-	pHWDef       hw   = &dev->usbDev.HwSetting;
-	pDCapsDef    sc   = &dev->usbDev.Caps;
+	HWDef       *hw   = &dev->usbDev.HwSetting;
+	DCapsDef    *sc   = &dev->usbDev.Caps;
 	u_char      *regs = dev->usbDev.a_bRegs;
 
 	DBG( _DBG_INFO, "usb_ScanBegin()\n" );
 
+	if( !usb_Wait4ScanSample( dev ))
+		return SANE_FALSE;
+
 	/* save the request for usb_ScanEnd () */
 	m_fAutoPark = auto_park;
 
@@ -1299,7 +1353,7 @@ static SANE_Bool usb_ScanBegin( Plustek_Device *dev, SANE_Bool auto_park )
 	if( !usb_IsScannerReady(dev))
 		return SANE_FALSE;
 
-	/* Flush cache - only LM9831 (Source: National Semiconductors */
+	/* Flush cache - only LM9831 (Source: National Semiconductors) */
 	if( _LM9831 == hw->chip ) {
 
 		for(;;) {
@@ -1328,10 +1382,10 @@ static SANE_Bool usb_ScanBegin( Plustek_Device *dev, SANE_Bool auto_park )
 	}
 
 	/* Download map & Shading data */
-	if(( m_pParam->bCalibration == PARAM_Scan &&
-		!usb_MapDownload( dev )) ||
-		!usb_DownloadShadingData( dev, m_pParam->bCalibration ))
+	if(( m_pParam->bCalibration == PARAM_Scan && !usb_MapDownload( dev )) ||
+	    !usb_DownloadShadingData( dev, m_pParam->bCalibration )) {
 		return SANE_FALSE;
+	}
 
 	/* Move chassis and start to read image data */
 	if (!usbio_WriteReg( dev->fd, 0x07, 3 ))
@@ -1357,7 +1411,8 @@ static SANE_Bool usb_ScanBegin( Plustek_Device *dev, SANE_Bool auto_park )
 /** usb_ScanEnd
  * stop all the processing stuff and reposition sensor back home
  */
-static SANE_Bool usb_ScanEnd( Plustek_Device *dev )
+static SANE_Bool
+usb_ScanEnd( Plustek_Device *dev )
 {
 	u_char value;
 
@@ -1382,7 +1437,8 @@ static SANE_Bool usb_ScanEnd( Plustek_Device *dev )
 
 /**
  */
-static SANE_Bool usb_IsDataAvailableInDRAM( Plustek_Device *dev )
+static SANE_Bool
+usb_IsDataAvailableInDRAM( Plustek_Device *dev )
 {
 	/* Compute polling timeout
 	 *	Height (Inches) / MaxScanSpeed (Inches/Second) = Seconds to move the
@@ -1435,8 +1491,8 @@ static SANE_Bool usb_IsDataAvailableInDRAM( Plustek_Device *dev )
 
 /**
  */
-static SANE_Bool usb_ScanReadImage( Plustek_Device *dev,
-                                    void *pBuf, u_long dwSize )
+static SANE_Bool
+usb_ScanReadImage( Plustek_Device *dev, void *pBuf, u_long dwSize )
 {
 	static u_long dwBytes = 0;
 	u_char       *regs = dev->usbDev.a_bRegs;
@@ -1478,7 +1534,7 @@ static SANE_Bool usb_ScanReadImage( Plustek_Device *dev,
 
 			if(!dwBytes)
 			{
-				WriteRegister(0x07, 0);	// To stop scanning
+				WriteRegister(0x07, 0);	/* To stop scanning */ 
 				ScanInf.m_fADF++;
 				if(dwSize > dw)
 					ScanReadImage((PBYTE)pBuf + dw, dwSize - dw);
@@ -1525,7 +1581,8 @@ static SANE_Bool usb_ScanReadImage( Plustek_Device *dev,
 /** calculate the number of pixels per line and lines out of a given
  * crop-area. The size of the area is given on a 300dpi base!
  */
-static void usb_GetImageInfo( Plustek_Device *dev, pImgDef pInfo, pWinInfo pSize )
+static void
+usb_GetImageInfo( Plustek_Device *dev, ImgDef *pInfo, WinInfo *pSize )
 {
 	DBG( _DBG_INFO, "usb_GetImageInfo()\n" );
 
@@ -1566,7 +1623,8 @@ static void usb_GetImageInfo( Plustek_Device *dev, pImgDef pInfo, pWinInfo pSize
 
 /**
  */
-static void usb_SaveImageInfo( Plustek_Device *dev, pImgDef pInfo )
+static void
+usb_SaveImageInfo( Plustek_Device *dev, ImgDef *pInfo )
 {
 	HWDef     *hw     = &dev->usbDev.HwSetting;
 	ScanParam *pParam = &dev->scanning.sParam;
@@ -1622,6 +1680,34 @@ static void usb_SaveImageInfo( Plustek_Device *dev, pImgDef pInfo )
 		pParam->bSource = SOURCE_ADF;
 	else
 		pParam->bSource = SOURCE_Reflection;
+
+	/* it seems, that we need to adjust the Origin.x when we have a
+	 * sheetfed device to avoid stripes in the resulting pictures
+	 */
+	if( usb_IsSheetFedDevice(dev)) {
+	
+		int step, div, org, xdpi;
+
+		xdpi = usb_SetAsicDpiX( dev, pParam->UserDpi.x );
+
+		if ((xdpi * 2) <= 300)
+			div = 300;
+		else if ((xdpi * 2) <= 600)
+			div = 600;
+		else if ((xdpi * 2) <= 1200)
+			div = 1200;
+		else
+			div = 2400;
+
+		step = div / xdpi;
+		org  = pParam->Origin.x;
+
+		pParam->Origin.x = (pParam->Origin.x / step) * step;
+
+		if (org != pParam->Origin.x)
+			DBG(_DBG_INFO, "* Origin.x adjusted: %i -> %i\n",
+			               org, pParam->Origin.x);
+	}
 }
 
 /* END PLUSTEK-USBSCAN.C ....................................................*/
