@@ -68,12 +68,20 @@
      So far this strategy has worked flawlessly. Thanks Dmitri!
 */
 
-/* $Id: snapscan-usb.c,v 1.20 2005/07/18 17:37:37 oliver-guest Exp $
+/* $Id: snapscan-usb.c,v 1.22 2006/01/26 17:42:30 hmg-guest Exp $
    SnapScan backend scan data sources */
 
 #include "snapscan-usb.h"
 #include "snapscan-mutex.c"
 
+#ifndef SHM_R
+#define SHM_R 0
+#endif
+
+#ifndef SHM_W
+#define SHM_W 0
+#endif
+
 /* Global variables */
 
 static snapscan_mutex_t snapscan_mutex;
@@ -300,7 +308,8 @@ static SANE_Status usb_read(SANE_Int fd, void *buf, size_t n) {
     return status;
 }
 
-static SANE_Status usb_read_status(int fd, int *scsistatus, int *transaction_status)
+static SANE_Status usb_read_status(int fd, int *scsistatus, int *transaction_status,
+                                   char command)
 {
     static const char me[] = "usb_read_status";
     unsigned char status_buf[8];
@@ -322,7 +331,13 @@ static SANE_Status usb_read_status(int fd, int *scsistatus, int *transaction_sta
         return SANE_STATUS_GOOD;
     case CHECK_CONDITION:
         if (usb_pss != NULL) {
-            return usb_request_sense(usb_pss);
+            if (command != REQUEST_SENSE) {
+                return usb_request_sense(usb_pss);
+            }
+            else {
+                /* Avoid recursive calls of usb_request_sense */
+                return SANE_STATUS_GOOD;
+            }
         } else {
             DBG (DL_MAJOR_ERROR, "%s: scanner structure not set, returning default error\n",
                 me);
@@ -343,6 +358,7 @@ static SANE_Status usb_cmd(int fd, const void *src, size_t src_size,
   static const char me[] = "usb_cmd";
   int status,tstatus;
   int cmdlen,datalen;
+  char command;
 
   DBG (DL_CALL_TRACE, "%s(%d,0x%lx,%lu,0x%lx,0x%lx (%lu))\n", me,
        fd, (u_long) src,(u_long) src_size,(u_long) dst, (u_long) dst_size,(u_long) (dst_size ? *dst_size : 0));
@@ -350,7 +366,8 @@ static SANE_Status usb_cmd(int fd, const void *src, size_t src_size,
   /* Since the  "Send Diagnostic" command isn't supported by
      all Snapscan USB-scanners it's disabled .
   */
-  if(((const char *)src)[0] == SEND_DIAGNOSTIC)
+  command = ((const char *)src)[0];
+  if(command == SEND_DIAGNOSTIC)
       return(SANE_STATUS_GOOD);
 
   cmdlen = usb_cmdlen(*((const char *)src));
@@ -362,7 +379,7 @@ static SANE_Status usb_cmd(int fd, const void *src, size_t src_size,
   RETURN_ON_FAILURE( usb_write(fd,src,cmdlen) );
 
   /* Read status */
-  RETURN_ON_FAILURE( usb_read_status(fd, NULL, &tstatus) );
+  RETURN_ON_FAILURE( usb_read_status(fd, NULL, &tstatus, command) );
 
   /* Send data only if the scanner is expecting it */
   if(datalen > 0 && (tstatus == TRANSACTION_WRITE)) {
@@ -370,7 +387,7 @@ static SANE_Status usb_cmd(int fd, const void *src, size_t src_size,
       RETURN_ON_FAILURE( usb_write(fd, ((const SANE_Byte *) src) + cmdlen, datalen) );
 
       /* Read status */
-      RETURN_ON_FAILURE( usb_read_status(fd, NULL, &tstatus) );
+      RETURN_ON_FAILURE( usb_read_status(fd, NULL, &tstatus, command) );
   }
 
   /* Receive data only when new data is waiting */
@@ -378,7 +395,7 @@ static SANE_Status usb_cmd(int fd, const void *src, size_t src_size,
       RETURN_ON_FAILURE( usb_read(fd,dst,*dst_size) );
 
       /* Read status */
-      RETURN_ON_FAILURE( usb_read_status(fd, NULL, &tstatus) );
+      RETURN_ON_FAILURE( usb_read_status(fd, NULL, &tstatus, command) );
   }
 
   if(tstatus != TRANSACTION_COMPLETED) {
@@ -563,6 +580,12 @@ static void snapscani_usb_shm_exit(void)
 #endif
 /*
  * $Log: snapscan-usb.c,v $
+ * Revision 1.22  2006/01/26 17:42:30  hmg-guest
+ * Added #defines for SHM_R/W for cygwin (patch from Philip Aston <philipa@mail.com>).
+ *
+ * Revision 1.21  2005/11/02 19:22:06  oliver-guest
+ * Fixes for Benq 5000
+ *
  * Revision 1.20  2005/07/18 17:37:37  oliver-guest
  * ZETA changes for SnapScan backend
  *
