@@ -1,4 +1,4 @@
-static const char RCSid[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c.c,v 1.5 2003/12/27 17:48:38 hmg-guest Exp $";
+static const char RCSid[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c.c,v 1.13 2006/03/29 20:48:50 hmg-guest Exp $";
 /* sane - Scanner Access Now Easy.
 
    This file is part of the SANE package.
@@ -45,6 +45,32 @@ static const char RCSid[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c.
 
 /*
  * $Log: sp15c.c,v $
+ * Revision 1.13  2006/03/29 20:48:50  hmg-guest
+ * Fixed ADF support. Patch from Andreas Degert <ad@papyrus-gmbh.de>.
+ *
+ * Revision 1.12  2005/10/01 17:06:25  hmg-guest
+ * Fixed some warnings (bug #302290).
+ *
+ * Revision 1.11  2005/07/15 18:12:49  hmg-guest
+ * Better 4->8 bit depth expansion algorithm (from Mattias Ellert
+ * <mattias.ellert@tsl.uu.se>).
+ *
+ * Revision 1.10  2004/10/06 15:59:40  hmg-guest
+ * Don't eject medium twice after each page.
+ *
+ * Revision 1.9  2004/06/20 00:34:10  ellert-guest
+ * Missed one...
+ *
+ * Revision 1.8  2004/05/29 10:27:47  hmg-guest
+ * Fixed the fix of the sanei_thread fix (from Mattias Ellert).
+ *
+ * Revision 1.7  2004/05/28 18:52:53  hmg-guest
+ * Fixed sanei_thread fix (bug #300634, by Mattias Ellert).
+ *
+ * Revision 1.6  2004/05/23 17:28:56  hmg-guest
+ * Use sanei_thread instead of fork() in the unmaintained backends.
+ * Patches from Mattias Ellert (bugs: 300635, 300634, 300633, 300629).
+ *
  * Revision 1.5  2003/12/27 17:48:38  hmg-guest
  * Silenced some compilation warnings.
  *
@@ -176,14 +202,13 @@ static const char RCSid[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c.
 #include <string.h>
 
 #include <sys/types.h>
-#include <sys/wait.h>
 #include <unistd.h>
 
 #include "sane/sanei_backend.h"
 #include "sane/sanei_scsi.h"
 #include "sane/saneopts.h"
 #include "sane/sanei_config.h"
-
+#include "sane/sanei_thread.h"
 
 #include "sp15c-scsi.h"
 #include "sp15c.h"
@@ -241,6 +266,8 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   DBG_INIT ();
   DBG (10, "sane_init\n");
 
+  sanei_thread_init ();
+
   if (version_code)
     *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
   fp = sanei_config_open (SP15C_CONFIG_FILE);
@@ -819,27 +846,12 @@ sane_start (SANE_Handle handle)
       return SANE_STATUS_IO_ERROR;
     }
 
-  scanner->reader_pid = fork ();
-  if (scanner->reader_pid == 0)
-    {
-      /* reader_pid = 0 ===> child process */
-      sigset_t ignore_set;
-      struct SIGACTION act;
-
-      close (fds[0]);
-
-      sigfillset (&ignore_set);
-      sigdelset (&ignore_set, SIGTERM);
-      sigprocmask (SIG_SETMASK, &ignore_set, 0);
-
-      memset (&act, 0, sizeof (act));
-      sigaction (SIGTERM, &act, 0);
-
-      /* don't use exit() since that would run the atexit() handlers... */
-      _exit (reader_process (scanner, fds[1]));
-    }
-  close (fds[1]);
   scanner->pipe = fds[0];
+  scanner->reader_pipe = fds[1];
+  scanner->reader_pid = sanei_thread_begin (reader_process, (void *) scanner);
+
+  if (sanei_thread_is_forked ())
+    close (scanner->reader_pipe);
 
   DBG (10, "sane_start: ok\n");
   return SANE_STATUS_GOOD;
@@ -1232,9 +1244,9 @@ sp15c_identify_scanner (struct sp15c *s)
       return SANE_STATUS_INVAL;
     }
 
-  get_IN_vendor (s->buffer, vendor);
-  get_IN_product (s->buffer, product);
-  get_IN_version (s->buffer, version);
+  get_IN_vendor ((char *) s->buffer, vendor);
+  get_IN_product ((char *)s->buffer, product);
+  get_IN_version ((char *)s->buffer, version);
 
   if (strncmp ("FCPA    ", vendor, 8))
     {
@@ -1568,10 +1580,14 @@ sp15c_free_scanner (struct sp15c *s)
 {
   int ret;
   DBG (10, "sp15c_free_scanner\n");
+#if 0
+  /* hmg: reports from several people show that this code ejects two pages
+     instead of one. So I've commented it out for now. */
   ret = sp15c_object_discharge (s);
   if (ret)
     return ret;
-
+#endif
+  
   wait_scanner (s);
 
   ret = do_scsi_cmd (s->sfd, release_unitB.cmd, release_unitB.size, NULL, 0);
@@ -1724,7 +1740,6 @@ do_cancel (struct sp15c *scanner)
 {
   DBG (10, "do_cancel\n");
   swap_res (scanner);
-  scanner->scanning = SANE_FALSE;
 
   do_eof (scanner);             /* close pipe and reposition scanner */
 
@@ -1733,10 +1748,9 @@ do_cancel (struct sp15c *scanner)
       int exit_status;
       DBG (10, "do_cancel: kill reader_process\n");
       /* ensure child knows it's time to stop: */
-      kill (scanner->reader_pid, SIGTERM);
-      while (wait (&exit_status) != scanner->reader_pid)
-        DBG (50, "wait for scanner to stop\n");
-      ;
+      sanei_thread_kill (scanner->reader_pid);
+      DBG (50, "wait for scanner to stop\n");
+      sanei_thread_waitpid (scanner->reader_pid, &exit_status);
       scanner->reader_pid = 0;
     }
 
@@ -1929,12 +1943,16 @@ sigterm_handler (int signal)
 
 /* This function is executed as a child process. */
 static int
-reader_process (struct sp15c *scanner, int pipe_fd)
+reader_process (void *data)
 {
+  struct sp15c *scanner = (struct sp15c *) data;
+  int pipe_fd = scanner->reader_pipe;
+
   int status;
   unsigned int data_left;
   unsigned int data_to_read;
   FILE *fp;
+  sigset_t ignore_set;
   sigset_t sigterm_set;
   struct SIGACTION act;
   unsigned int i;
@@ -1942,6 +1960,19 @@ reader_process (struct sp15c *scanner, int pipe_fd)
 
   DBG (10, "reader_process started\n");
 
+  if (sanei_thread_is_forked ()) 
+    close (scanner->pipe);
+
+  sigfillset (&ignore_set);
+  sigdelset (&ignore_set, SIGTERM);
+#if defined (__APPLE__) && defined (__MACH__)
+  sigdelset (&ignore_set, SIGUSR2);
+#endif
+  sigprocmask (SIG_SETMASK, &ignore_set, 0);
+
+  memset (&act, 0, sizeof (act));
+  sigaction (SIGTERM, &act, 0);
+
   sigemptyset (&sigterm_set);
   sigaddset (&sigterm_set, SIGTERM);
 
@@ -2002,8 +2033,9 @@ reader_process (struct sp15c *scanner, int pipe_fd)
           dst = &scanner->buffer[data_to_read * 2 - 1];
           for (i = 0; i < data_to_read; i++)
             {
-              *dst-- = (*src << 4) & 0xf0;
-              *dst-- = (*src--) & 0xf0;
+              *dst-- = ((*src << 4) & 0xf0) + ((*src)        & 0x0f);
+              *dst-- = ((*src)      & 0xf0) + ((*src >> 4) & 0x0f);
+	      --src;
             }
           data_to_read *= 2;
         }
@@ -2031,6 +2063,7 @@ do_eof (struct sp15c *scanner)
 {
   DBG (10, "do_eof\n");
 
+  scanner->scanning = SANE_FALSE;
   if (scanner->pipe >= 0)
     {
       close (scanner->pipe);
