@@ -42,7 +42,7 @@
 */
 
 /*
-   $Id: teco1.c,v 1.15 2005/07/07 11:55:42 fzago-guest Exp $
+   $Id: teco1.c,v 1.16 2005/09/19 19:57:48 fzago-guest Exp $
    Some Relisys scanners AVEC and RELI series
 */
 
@@ -576,7 +576,7 @@ teco_get_inquiry_82 (Teco_Scanner * dev)
 /* SCSI sense handler. Callback for SANE.
  * These scanners never set asc or ascq. */
 static SANE_Status
-teco_sense_handler (int __unused__ scsi_fd, unsigned char *result, void __unused__ *arg)
+teco_sense_handler (int __sane_unused__ scsi_fd, unsigned char *result, void __sane_unused__ *arg)
 {
   int sensekey;
   int len;
@@ -1468,7 +1468,7 @@ do_cancel (Teco_Scanner * dev)
 /* Sane entry points */
 
 SANE_Status
-sane_init (SANE_Int * version_code, SANE_Auth_Callback __unused__ authorize)
+sane_init (SANE_Int * version_code, SANE_Auth_Callback __sane_unused__ authorize)
 {
   FILE *fp;
   char dev_name[PATH_MAX];
@@ -1515,7 +1515,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback __unused__ authorize)
 }
 
 SANE_Status
-sane_get_devices (const SANE_Device *** device_list, SANE_Bool __unused__ local_only)
+sane_get_devices (const SANE_Device *** device_list, SANE_Bool __sane_unused__ local_only)
 {
   Teco_Scanner *dev;
   int i;
@@ -2160,7 +2160,7 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
 }
 
 SANE_Status
-sane_set_io_mode (SANE_Handle __unused__ handle, SANE_Bool __unused__ non_blocking)
+sane_set_io_mode (SANE_Handle __sane_unused__ handle, SANE_Bool __sane_unused__ non_blocking)
 {
   SANE_Status status;
   Teco_Scanner *dev = handle;
@@ -2187,7 +2187,7 @@ sane_set_io_mode (SANE_Handle __unused__ handle, SANE_Bool __unused__ non_blocki
 }
 
 SANE_Status
-sane_get_select_fd (SANE_Handle __unused__ handle, SANE_Int __unused__ *fd)
+sane_get_select_fd (SANE_Handle __sane_unused__ handle, SANE_Int __sane_unused__ *fd)
 {
   DBG (DBG_proc, "sane_get_select_fd: enter\n");
 
