@@ -42,7 +42,7 @@
 */
 
 /*
-   $Id: teco3.c,v 1.7 2005/07/07 11:55:42 fzago-guest Exp $
+   $Id: teco3.c,v 1.8 2005/09/19 19:57:48 fzago-guest Exp $
    VM3552 (and maybe VM4552 and VM6552)
 */
 
@@ -422,7 +422,7 @@ teco_identify_scanner (Teco_Scanner * dev)
 /* SCSI sense handler. Callback for SANE.
  * These scanners never set asc or ascq. */
 static SANE_Status
-teco_sense_handler (int __unused__ scsi_fd, unsigned char *result, void __unused__ *arg)
+teco_sense_handler (int __sane_unused__ scsi_fd, unsigned char *result, void __sane_unused__ *arg)
 {
   int sensekey;
   int len;
@@ -1490,7 +1490,7 @@ do_cancel (Teco_Scanner * dev)
 /* Sane entry points */
 
 SANE_Status
-sane_init (SANE_Int * version_code, SANE_Auth_Callback __unused__ authorize)
+sane_init (SANE_Int * version_code, SANE_Auth_Callback __sane_unused__ authorize)
 {
   FILE *fp;
   char dev_name[PATH_MAX];
@@ -1537,7 +1537,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback __unused__ authorize)
 }
 
 SANE_Status
-sane_get_devices (const SANE_Device *** device_list, SANE_Bool __unused__ local_only)
+sane_get_devices (const SANE_Device *** device_list, SANE_Bool __sane_unused__ local_only)
 {
   Teco_Scanner *dev;
   int i;
@@ -2152,7 +2152,7 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
 }
 
 SANE_Status
-sane_set_io_mode (SANE_Handle __unused__ handle, SANE_Bool __unused__ non_blocking)
+sane_set_io_mode (SANE_Handle __sane_unused__ handle, SANE_Bool __sane_unused__ non_blocking)
 {
   SANE_Status status;
   Teco_Scanner *dev = handle;
@@ -2179,7 +2179,7 @@ sane_set_io_mode (SANE_Handle __unused__ handle, SANE_Bool __unused__ non_blocki
 }
 
 SANE_Status
-sane_get_select_fd (SANE_Handle __unused__ handle, SANE_Int __unused__ * fd)
+sane_get_select_fd (SANE_Handle __sane_unused__ handle, SANE_Int __sane_unused__ * fd)
 {
   DBG (DBG_proc, "sane_get_select_fd: enter\n");
 
