@@ -1,6 +1,6 @@
 /* sane-find-scanner.c
 
-   Copyright (C) 1997-2004 Oliver Rauch, Henning Meier-Geinitz, and others.
+   Copyright (C) 1997-2005 Oliver Rauch, Henning Meier-Geinitz, and others.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -625,6 +625,9 @@ check_libusb_device (struct usb_device *dev, SANE_Bool from_file)
 	  ++is_scanner;
 	  break;
 	case USB_CLASS_PER_INTERFACE:
+	  if (dev->config[0].interface[interface_nr].num_altsetting == 0 || 
+	      !dev->config[0].interface[interface_nr].altsetting)
+	    break;
 	  switch (dev->config[0].interface[interface_nr].altsetting[0].bInterfaceClass)
 	    {
 	    case USB_CLASS_VENDOR_SPEC:
@@ -1391,27 +1394,7 @@ main (int argc, char **argv)
 	"/dev/usbscanner10", "/dev/usbscanner11",
 	"/dev/usbscanner12", "/dev/usbscanner13",
 	"/dev/usbscanner14", "/dev/usbscanner15",
-#elif defined(__FreeBSD__)
-	"/dev/uscanner",
-	"/dev/uscanner0", "/dev/uscanner1",
-	"/dev/uscanner2", "/dev/uscanner3",
-	"/dev/uscanner4", "/dev/uscanner5",
-	"/dev/uscanner6", "/dev/uscanner7",
-	"/dev/uscanner8", "/dev/uscanner9",
-	"/dev/uscanner10", "/dev/uscanner11",
-	"/dev/uscanner12", "/dev/uscanner13",
-	"/dev/uscanner14", "/dev/uscanner15",
-#elif defined(__NetBSD__)
-	"/dev/uscanner",
-	"/dev/uscanner0", "/dev/uscanner1",
-	"/dev/uscanner2", "/dev/uscanner3",
-	"/dev/uscanner4", "/dev/uscanner5",
-	"/dev/uscanner6", "/dev/uscanner7",
-	"/dev/uscanner8", "/dev/uscanner9",
-	"/dev/uscanner10", "/dev/uscanner11",
-	"/dev/uscanner12", "/dev/uscanner13",
-	"/dev/uscanner14", "/dev/uscanner15",
-#elif defined(__OpenBSD__)
+#elif defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
 	"/dev/uscanner",
 	"/dev/uscanner0", "/dev/uscanner1",
 	"/dev/uscanner2", "/dev/uscanner3",
@@ -1439,9 +1422,10 @@ main (int argc, char **argv)
   if (verbose > 1)
     printf ("This is sane-find-scanner from %s\n", PACKAGE_STRING);
 
-  printf ("\n  # sane-find-scanner will now attempt to detect your scanner. If the"
-	  "\n  # result is different from what you expected, first make sure your"
-	  "\n  # scanner is powered up and properly connected to your computer.\n\n");
+  if (verbose > 0)
+    printf ("\n  # sane-find-scanner will now attempt to detect your scanner. If the"
+	    "\n  # result is different from what you expected, first make sure your"
+	    "\n  # scanner is powered up and properly connected to your computer.\n\n");
 
   if (verbose > 1)
     printf ("searching for SCSI scanners:\n");
@@ -1483,7 +1467,7 @@ main (int argc, char **argv)
       if (verbose > 0)
 	printf
 	  ("  # No SCSI scanners found. If you expected something different, "
-	   "make sure that\n  # you have loaded a SCSI driver for your SCSI "
+	   "make sure that\n  # you have loaded a kernel SCSI driver for your SCSI "
 	   "adapter.\n");
       if (!check_sg ())
 	{
@@ -1583,8 +1567,15 @@ main (int argc, char **argv)
       if (verbose > 0)
 	printf
 	  ("  # No USB scanners found. If you expected something different, "
-	   "make sure that\n  # you have loaded a driver for your USB host "
-	   "controller and have installed a\n  # kernel scanner module.\n");
+	   "make sure that\n  # you have loaded a kernel driver for your USB host "
+	   "controller and have setup\n  # the USB system correctly. "
+	   "See man sane-usb for details.\n");
+#ifndef HAVE_LIBUSB
+      if (verbose > 0)
+	printf ("  # SANE has been built without libusb support. This may be a "
+		"reason\n  # for not detecting USB scanners. Read README for "
+		"more details.\n");
+#endif
     }
   if (enable_pp_checks == SANE_TRUE) 
     {
